// -*- C++ -*-
/* GG is a GUI for SDL and OpenGL.
   Copyright (C) 2003-2008 T. Zachary Laine

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1
   of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
    
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA

   If you do not wish to comply with the terms of the LGPL please
   contact the author as other terms are available for a fee.
    
   Zach Laine
   whatwasthataddress@gmail.com

Copyright for Vera*.ttf:

   Copyright (c) 2003 by Bitstream, Inc. All Rights Reserved. Bitstream
   Vera is a trademark of Bitstream, Inc.

   Permission is hereby granted, free of charge, to any person obtaining
   a copy of the fonts accompanying this license ("Fonts") and associated
   documentation files (the "Font Software"), to reproduce and distribute
   the Font Software, including without limitation the rights to use,
   copy, merge, publish, distribute, and/or sell copies of the Font
   Software, and to permit persons to whom the Font Software is furnished
   to do so, subject to the following conditions:

   The above copyright and trademark notices and this permission notice
   shall be included in all copies of one or more of the Font Software
   typefaces.

   The Font Software may be modified, altered, or added to, and in
   particular the designs of glyphs or characters in the Fonts may be
   modified and additional glyphs or characters may be added to the
   Fonts, only if the fonts are renamed to names not containing either
   the words "Bitstream" or the word "Vera".

   This License becomes null and void to the extent applicable to Fonts
   or Font Software that has been modified and is distributed under the
   "Bitstream Vera" names.

   The Font Software may be sold as part of a larger software package but
   no copy of one or more of the Font Software typefaces may be sold by
   itself.

   THE FONT SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
   OF COPYRIGHT, PATENT, TRADEMARK, OR OTHER RIGHT. IN NO EVENT SHALL
   BITSTREAM OR THE GNOME FOUNDATION BE LIABLE FOR ANY CLAIM, DAMAGES OR
   OTHER LIABILITY, INCLUDING ANY GENERAL, SPECIAL, INDIRECT, INCIDENTAL,
   OR CONSEQUENTIAL DAMAGES, WHETHER IN AN ACTION OF CONTRACT, TORT OR
   OTHERWISE, ARISING FROM, OUT OF THE USE OR INABILITY TO USE THE FONT
   SOFTWARE OR FROM OTHER DEALINGS IN THE FONT SOFTWARE.

   Except as contained in this notice, the names of Gnome, the Gnome
   Foundation, and Bitstream Inc., shall not be used in advertising or
   otherwise to promote the sale, use or other dealings in this Font
   Software without prior written authorization from the Gnome Foundation
   or Bitstream Inc., respectively. For further information, contact:
   fonts at gnome dot org.
*/

namespace {
    void DecodeBase64(std::vector<unsigned char>& data, const std::string& str)
    {
        static std::vector<unsigned int> table(256, 0);

        if (!table['A']) {
            for (unsigned char c = 'A'; c <= 'Z'; ++c) {
                table[c] = c - 'A';
            }
            for (unsigned char c = 'a'; c <= 'z'; ++c) {
                table[c] = 26 + c - 'a';
            }
            for (unsigned char c = '0'; c <= '9'; ++c) {
                table[c] = 52 + c - '0';
            }
            table['+'] = 62;
            table['/'] = 63;
        }

        int len = str.size();
        int groups = len / 4;
        data.resize(groups * 3);

        int data_posn = 0;
        int str_posn = 0;
        for (int i = 0; i < groups - 1; ++i) {
            boost::uint32_t group_value = (table[str[str_posn + 0]] << 18)
                | (table[str[str_posn + 1]] << 12)
                | (table[str[str_posn + 2]] << 6)
                | (table[str[str_posn + 3]] << 0);
            data[data_posn + 0] = group_value << 8 >> 24;
            data[data_posn + 1] = group_value << 16 >> 24;
            data[data_posn + 2] = group_value << 24 >> 24;
            data_posn += 3;
            str_posn += 4;
        }
        boost::uint32_t group_value = (table[str[str_posn + 0]] << 18)
            | (table[str[str_posn + 1]] << 12)
            | (table[str[str_posn + 2]] << 6)
            | (table[str[str_posn + 3]] << 0);
        if (data.size() - data_posn == 3) {
            data[data_posn + 0] = group_value << 8 >> 24;
            data[data_posn + 1] = group_value << 16 >> 24;
            data[data_posn + 2] = group_value << 24 >> 24;
        } else if (data.size() - data_posn == 2) {
            data[data_posn + 0] = group_value << 8 >> 24;
            data[data_posn + 1] = group_value << 16 >> 24;
        } else if (data.size() - data_posn == 1) {
            data[data_posn + 0] = group_value << 8 >> 24;
        }
    }

    const char VERA_TTF_PART_1[] =
        "AAEAAAARAQAABAAQT1MvMrRf9GMAAOtwAAAAVlBDTFTRil6XAADryAAAADZj"
        "bWFwpMPooAAAsWwAAANYY3Z0IP/THTkAAB78AAAB/GZwZ23ntPHEAAAmYAAA"
        "AItnYXNwAAcABwABAUgAAAAMZ2x5Zgx0Qc8AACbsAACKfmhkbXg08CEOAADs"
        "AAAAFUhoZWFk3YSi0AABAVQAAAA2aGhlYRBFCG8AAOtMAAAAJGhtdHgJxo6y"
        "AAC0xAAABDBrZXJu3FLVmQAAvaAAAC2KbG9jYfPL0j0AALuEAAACGm1heHAF"
        "RwY6AADrLAAAACBuYW1l2bzItQAAARwAAB3fcG9zdLRaL7sAALj0AAACjnBy"
        "ZXA7B/EAAAAg+AAABWgAAAAWAQ4AAQAAAAAAAAA6AAAAAQAAAAAAAQATADoA"
        "AQAAAAAAAgAFAF8AAQAAAAAAAwATADoAAQAAAAAABAATADoAAQAAAAAABQAM"
        "AGQAAQAAAAAABgAXAE0AAQAAAAAABwAwAK0AAQAAAAAACAAOCGwAAQAAAAAA"
        "CwAYCYMAAQAAAAAADQkTAHAAAwABBAkAAAB0CZsAAwABBAkAAQAmCg8AAwAB"
        "BAkAAgAKClkAAwABBAkAAwAmCg8AAwABBAkABAAmCg8AAwABBAkABQAYCmMA"
        "AwABBAkABgAuCjUAAwABBAkABwBgCvUAAwABBAkACAAcGnMAAwABBAkACwAw"
        "HKEAAwABBAkADRImCntDb3B5cmlnaHQgKGMpIDIwMDMgYnkgQml0c3RyZWFt"
        "LCBJbmMuIEFsbCBSaWdodHMgUmVzZXJ2ZWQuQml0c3RyZWFtIFZlcmEgU2Fu"
        "c0JpdHN0cmVhbVZlcmFTYW5zLVJvbWFuUmVsZWFzZSAxLjEwQ29weXJpZ2h0"
        "IChjKSAyMDAzIGJ5IEJpdHN0cmVhbSwgSW5jLg0KQWxsIFJpZ2h0cyBSZXNl"
        "cnZlZC4NCkJpdHN0cmVhbSBWZXJhIGlzIGEgdHJhZGVtYXJrIG9mIEJpdHN0"
        "cmVhbSwgSW5jLg0KDQpQZXJtaXNzaW9uIGlzIGhlcmVieSBncmFudGVkLCBm"
        "cmVlIG9mIGNoYXJnZSwgdG8gYW55IHBlcnNvbiBvYnRhaW5pbmcgYSBjb3B5"
        "IG9mIHRoZSBmb250cyBhY2NvbXBhbnlpbmcgdGhpcyBsaWNlbnNlICgiRm9u"
        "dHMiKSBhbmQgYXNzb2NpYXRlZCBkb2N1bWVudGF0aW9uIGZpbGVzICh0aGUg"
        "IkZvbnQgU29mdHdhcmUiKSwgdG8gcmVwcm9kdWNlIGFuZCBkaXN0cmlidXRl"
        "IHRoZSBGb250IFNvZnR3YXJlLCBpbmNsdWRpbmcgd2l0aG91dCBsaW1pdGF0"
        "aW9uIHRoZSByaWdodHMgdG8gdXNlLCBjb3B5LCBtZXJnZSwgcHVibGlzaCwg"
        "ZGlzdHJpYnV0ZSwgYW5kL29yIHNlbGwgY29waWVzIG9mIHRoZSBGb250IFNv"
        "ZnR3YXJlLCBhbmQgdG8gcGVybWl0IHBlcnNvbnMgdG8gd2hvbSB0aGUgRm9u"
        "dCBTb2Z0d2FyZSBpcyBmdXJuaXNoZWQgdG8gZG8gc28sIHN1YmplY3QgdG8g"
        "dGhlIGZvbGxvd2luZyBjb25kaXRpb25zOg0KDQpUaGUgYWJvdmUgY29weXJp"
        "Z2h0IGFuZCB0cmFkZW1hcmsgbm90aWNlcyBhbmQgdGhpcyBwZXJtaXNzaW9u"
        "IG5vdGljZSBzaGFsbCBiZSBpbmNsdWRlZCBpbiBhbGwgY29waWVzIG9mIG9u"
        "ZSBvciBtb3JlIG9mIHRoZSBGb250IFNvZnR3YXJlIHR5cGVmYWNlcy4NCg0K"
        "VGhlIEZvbnQgU29mdHdhcmUgbWF5IGJlIG1vZGlmaWVkLCBhbHRlcmVkLCBv"
        "ciBhZGRlZCB0bywgYW5kIGluIHBhcnRpY3VsYXIgdGhlIGRlc2lnbnMgb2Yg"
        "Z2x5cGhzIG9yIGNoYXJhY3RlcnMgaW4gdGhlIEZvbnRzIG1heSBiZSBtb2Rp"
        "ZmllZCBhbmQgYWRkaXRpb25hbCBnbHlwaHMgb3IgY2hhcmFjdGVycyBtYXkg"
        "YmUgYWRkZWQgdG8gdGhlIEZvbnRzLCBvbmx5IGlmIHRoZSBmb250cyBhcmUg"
        "cmVuYW1lZCB0byBuYW1lcyBub3QgY29udGFpbmluZyBlaXRoZXIgdGhlIHdv"
        "cmRzICJCaXRzdHJlYW0iIG9yIHRoZSB3b3JkICJWZXJhIi4NCg0KVGhpcyBM"
        "aWNlbnNlIGJlY29tZXMgbnVsbCBhbmQgdm9pZCB0byB0aGUgZXh0ZW50IGFw"
        "cGxpY2FibGUgdG8gRm9udHMgb3IgRm9udCBTb2Z0d2FyZSB0aGF0IGhhcyBi"
        "ZWVuIG1vZGlmaWVkIGFuZCBpcyBkaXN0cmlidXRlZCB1bmRlciB0aGUgIkJp"
        "dHN0cmVhbSBWZXJhIiBuYW1lcy4NCg0KVGhlIEZvbnQgU29mdHdhcmUgbWF5"
        "IGJlIHNvbGQgYXMgcGFydCBvZiBhIGxhcmdlciBzb2Z0d2FyZSBwYWNrYWdl"
        "IGJ1dCBubyBjb3B5IG9mIG9uZSBvciBtb3JlIG9mIHRoZSBGb250IFNvZnR3"
        "YXJlIHR5cGVmYWNlcyBtYXkgYmUgc29sZCBieSBpdHNlbGYuDQoNClRIRSBG"
        "T05UIFNPRlRXQVJFIElTIFBST1ZJREVEICJBUyBJUyIsIFdJVEhPVVQgV0FS"
        "UkFOVFkgT0YgQU5ZIEtJTkQsIEVYUFJFU1MgT1IgSU1QTElFRCwgSU5DTFVE"
        "SU5HIEJVVCBOT1QgTElNSVRFRCBUTyBBTlkgV0FSUkFOVElFUyBPRiBNRVJD"
        "SEFOVEFCSUxJVFksIEZJVE5FU1MgRk9SIEEgUEFSVElDVUxBUiBQVVJQT1NF"
        "IEFORCBOT05JTkZSSU5HRU1FTlQgT0YgQ09QWVJJR0hULCBQQVRFTlQsIFRS"
        "QURFTUFSSywgT1IgT1RIRVIgUklHSFQuIElOIE5PIEVWRU5UIFNIQUxMIEJJ"
        "VFNUUkVBTSBPUiBUSEUgR05PTUUgRk9VTkRBVElPTiBCRSBMSUFCTEUgRk9S"
        "IEFOWSBDTEFJTSwgREFNQUdFUyBPUiBPVEhFUiBMSUFCSUxJVFksIElOQ0xV"
        "RElORyBBTlkgR0VORVJBTCwgU1BFQ0lBTCwgSU5ESVJFQ1QsIElOQ0lERU5U"
        "QUwsIE9SIENPTlNFUVVFTlRJQUwgREFNQUdFUywgV0hFVEhFUiBJTiBBTiBB"
        "Q1RJT04gT0YgQ09OVFJBQ1QsIFRPUlQgT1IgT1RIRVJXSVNFLCBBUklTSU5H"
        "IEZST00sIE9VVCBPRiBUSEUgVVNFIE9SIElOQUJJTElUWSBUTyBVU0UgVEhF"
        "IEZPTlQgU09GVFdBUkUgT1IgRlJPTSBPVEhFUiBERUFMSU5HUyBJTiBUSEUg"
        "Rk9OVCBTT0ZUV0FSRS4NCg0KRXhjZXB0IGFzIGNvbnRhaW5lZCBpbiB0aGlz"
        "IG5vdGljZSwgdGhlIG5hbWVzIG9mIEdub21lLCB0aGUgR25vbWUgRm91bmRh"
        "dGlvbiwgYW5kIEJpdHN0cmVhbSBJbmMuLCBzaGFsbCBub3QgYmUgdXNlZCBp"
        "biBhZHZlcnRpc2luZyBvciBvdGhlcndpc2UgdG8gcHJvbW90ZSB0aGUgc2Fs"
        "ZSwgdXNlIG9yIG90aGVyIGRlYWxpbmdzIGluIHRoaXMgRm9udCBTb2Z0d2Fy"
        "ZSB3aXRob3V0IHByaW9yIHdyaXR0ZW4gYXV0aG9yaXphdGlvbiBmcm9tIHRo"
        "ZSBHbm9tZSBGb3VuZGF0aW9uIG9yIEJpdHN0cmVhbSBJbmMuLCByZXNwZWN0"
        "aXZlbHkuIEZvciBmdXJ0aGVyIGluZm9ybWF0aW9uLCBjb250YWN0OiBmb250"
        "cyBhdCBnbm9tZSBkb3Qgb3JnLmh0dHA6Ly93d3cuYml0c3RyZWFtLmNvbQBD"
        "AG8AcAB5AHIAaQBnAGgAdAAgACgAYwApACAAMgAwADAAMwAgAGIAeQAgAEIA"
        "aQB0AHMAdAByAGUAYQBtACwAIABJAG4AYwAuACAAQQBsAGwAIABSAGkAZwBo"
        "AHQAcwAgAFIAZQBzAGUAcgB2AGUAZAAuAEIAaQB0AHMAdAByAGUAYQBtACAA"
        "VgBlAHIAYQAgAFMAYQBuAHMAQgBpAHQAcwB0AHIAZQBhAG0AVgBlAHIAYQBT"
        "AGEAbgBzAC0AUgBvAG0AYQBuAFIAZQBsAGUAYQBzAGUAIAAxAC4AMQAwAEMA"
        "bwBwAHkAcgBpAGcAaAB0ACAAKABjACkAIAAyADAAMAAzACAAYgB5ACAAQgBp"
        "AHQAcwB0AHIAZQBhAG0ALAAgAEkAbgBjAC4ADQAKAEEAbABsACAAUgBpAGcA"
        "aAB0AHMAIABSAGUAcwBlAHIAdgBlAGQALgANAAoAQgBpAHQAcwB0AHIAZQBh"
        "AG0AIABWAGUAcgBhACAAaQBzACAAYQAgAHQAcgBhAGQAZQBtAGEAcgBrACAA"
        "bwBmACAAQgBpAHQAcwB0AHIAZQBhAG0ALAAgAEkAbgBjAC4ADQAKAA0ACgBQ"
        "AGUAcgBtAGkAcwBzAGkAbwBuACAAaQBzACAAaABlAHIAZQBiAHkAIABnAHIA"
        "YQBuAHQAZQBkACwAIABmAHIAZQBlACAAbwBmACAAYwBoAGEAcgBnAGUALAAg"
        "AHQAbwAgAGEAbgB5ACAAcABlAHIAcwBvAG4AIABvAGIAdABhAGkAbgBpAG4A"
        "ZwAgAGEAIABjAG8AcAB5ACAAbwBmACAAdABoAGUAIABmAG8AbgB0AHMAIABh"
        "AGMAYwBvAG0AcABhAG4AeQBpAG4AZwAgAHQAaABpAHMAIABsAGkAYwBlAG4A"
        "cwBlACAAKAAiAEYAbwBuAHQAcwAiACkAIABhAG4AZAAgAGEAcwBzAG8AYwBp"
        "AGEAdABlAGQAIABkAG8AYwB1AG0AZQBuAHQAYQB0AGkAbwBuACAAZgBpAGwA"
        "ZQBzACAAKAB0AGgAZQAgACIARgBvAG4AdAAgAFMAbwBmAHQAdwBhAHIAZQAi"
        "ACkALAAgAHQAbwAgAHIAZQBwAHIAbwBkAHUAYwBlACAAYQBuAGQAIABkAGkA"
        "cwB0AHIAaQBiAHUAdABlACAAdABoAGUAIABGAG8AbgB0ACAAUwBvAGYAdAB3"
        "AGEAcgBlACwAIABpAG4AYwBsAHUAZABpAG4AZwAgAHcAaQB0AGgAbwB1AHQA"
        "IABsAGkAbQBpAHQAYQB0AGkAbwBuACAAdABoAGUAIAByAGkAZwBoAHQAcwAg"
        "AHQAbwAgAHUAcwBlACwAIABjAG8AcAB5ACwAIABtAGUAcgBnAGUALAAgAHAA"
        "dQBiAGwAaQBzAGgALAAgAGQAaQBzAHQAcgBpAGIAdQB0AGUALAAgAGEAbgBk"
        "AC8AbwByACAAcwBlAGwAbAAgAGMAbwBwAGkAZQBzACAAbwBmACAAdABoAGUA"
        "IABGAG8AbgB0ACAAUwBvAGYAdAB3AGEAcgBlACwAIABhAG4AZAAgAHQAbwAg"
        "AHAAZQByAG0AaQB0ACAAcABlAHIAcwBvAG4AcwAgAHQAbwAgAHcAaABvAG0A"
        "IAB0AGgAZQAgAEYAbwBuAHQAIABTAG8AZgB0AHcAYQByAGUAIABpAHMAIABm"
        "AHUAcgBuAGkAcwBoAGUAZAAgAHQAbwAgAGQAbwAgAHMAbwAsACAAcwB1AGIA"
        "agBlAGMAdAAgAHQAbwAgAHQAaABlACAAZgBvAGwAbABvAHcAaQBuAGcAIABj"
        "AG8AbgBkAGkAdABpAG8AbgBzADoADQAKAA0ACgBUAGgAZQAgAGEAYgBvAHYA"
        "ZQAgAGMAbwBwAHkAcgBpAGcAaAB0ACAAYQBuAGQAIAB0AHIAYQBkAGUAbQBh"
        "AHIAawAgAG4AbwB0AGkAYwBlAHMAIABhAG4AZAAgAHQAaABpAHMAIABwAGUA"
        "cgBtAGkAcwBzAGkAbwBuACAAbgBvAHQAaQBjAGUAIABzAGgAYQBsAGwAIABi"
        "AGUAIABpAG4AYwBsAHUAZABlAGQAIABpAG4AIABhAGwAbAAgAGMAbwBwAGkA"
        "ZQBzACAAbwBmACAAbwBuAGUAIABvAHIAIABtAG8AcgBlACAAbwBmACAAdABo"
        "AGUAIABGAG8AbgB0ACAAUwBvAGYAdAB3AGEAcgBlACAAdAB5AHAAZQBmAGEA"
        "YwBlAHMALgANAAoADQAKAFQAaABlACAARgBvAG4AdAAgAFMAbwBmAHQAdwBh"
        "AHIAZQAgAG0AYQB5ACAAYgBlACAAbQBvAGQAaQBmAGkAZQBkACwAIABhAGwA"
        "dABlAHIAZQBkACwAIABvAHIAIABhAGQAZABlAGQAIAB0AG8ALAAgAGEAbgBk"
        "ACAAaQBuACAAcABhAHIAdABpAGMAdQBsAGEAcgAgAHQAaABlACAAZABlAHMA"
        "aQBnAG4AcwAgAG8AZgAgAGcAbAB5AHAAaABzACAAbwByACAAYwBoAGEAcgBh"
        "AGMAdABlAHIAcwAgAGkAbgAgAHQAaABlACAARgBvAG4AdABzACAAbQBhAHkA"
        "IABiAGUAIABtAG8AZABpAGYAaQBlAGQAIABhAG4AZAAgAGEAZABkAGkAdABp"
        "AG8AbgBhAGwAIABnAGwAeQBwAGgAcwAgAG8AcgAgAGMAaABhAHIAYQBjAHQA"
        "ZQByAHMAIABtAGEAeQAgAGIAZQAgAGEAZABkAGUAZAAgAHQAbwAgAHQAaABl"
        "ACAARgBvAG4AdABzACwAIABvAG4AbAB5ACAAaQBmACAAdABoAGUAIABmAG8A"
        "bgB0AHMAIABhAHIAZQAgAHIAZQBuAGEAbQBlAGQAIAB0AG8AIABuAGEAbQBl"
        "AHMAIABuAG8AdAAgAGMAbwBuAHQAYQBpAG4AaQBuAGcAIABlAGkAdABoAGUA"
        "cgAgAHQAaABlACAAdwBvAHIAZABzACAAIgBCAGkAdABzAHQAcgBlAGEAbQAi"
        "ACAAbwByACAAdABoAGUAIAB3AG8AcgBkACAAIgBWAGUAcgBhACIALgANAAoA"
        "DQAKAFQAaABpAHMAIABMAGkAYwBlAG4AcwBlACAAYgBlAGMAbwBtAGUAcwAg"
        "AG4AdQBsAGwAIABhAG4AZAAgAHYAbwBpAGQAIAB0AG8AIAB0AGgAZQAgAGUA"
        "eAB0AGUAbgB0ACAAYQBwAHAAbABpAGMAYQBiAGwAZQAgAHQAbwAgAEYAbwBu"
        "AHQAcwAgAG8AcgAgAEYAbwBuAHQAIABTAG8AZgB0AHcAYQByAGUAIAB0AGgA"
        "YQB0ACAAaABhAHMAIABiAGUAZQBuACAAbQBvAGQAaQBmAGkAZQBkACAAYQBu"
        "AGQAIABpAHMAIABkAGkAcwB0AHIAaQBiAHUAdABlAGQAIAB1AG4AZABlAHIA"
        "IAB0AGgAZQAgACIAQgBpAHQAcwB0AHIAZQBhAG0AIABWAGUAcgBhACIAIABu"
        "AGEAbQBlAHMALgANAAoADQAKAFQAaABlACAARgBvAG4AdAAgAFMAbwBmAHQA"
        "dwBhAHIAZQAgAG0AYQB5ACAAYgBlACAAcwBvAGwAZAAgAGEAcwAgAHAAYQBy"
        "AHQAIABvAGYAIABhACAAbABhAHIAZwBlAHIAIABzAG8AZgB0AHcAYQByAGUA"
        "IABwAGEAYwBrAGEAZwBlACAAYgB1AHQAIABuAG8AIABjAG8AcAB5ACAAbwBm"
        "ACAAbwBuAGUAIABvAHIAIABtAG8AcgBlACAAbwBmACAAdABoAGUAIABGAG8A"
        "bgB0ACAAUwBvAGYAdAB3AGEAcgBlACAAdAB5AHAAZQBmAGEAYwBlAHMAIABt"
        "AGEAeQAgAGIAZQAgAHMAbwBsAGQAIABiAHkAIABpAHQAcwBlAGwAZgAuAA0A"
        "CgANAAoAVABIAEUAIABGAE8ATgBUACAAUwBPAEYAVABXAEEAUgBFACAASQBT"
        "ACAAUABSAE8AVgBJAEQARQBEACAAIgBBAFMAIABJAFMAIgAsACAAVwBJAFQA"
        "SABPAFUAVAAgAFcAQQBSAFIAQQBOAFQAWQAgAE8ARgAgAEEATgBZACAASwBJ"
        "AE4ARAAsACAARQBYAFAAUgBFAFMAUwAgAE8AUgAgAEkATQBQAEwASQBFAEQA"
        "LAAgAEkATgBDAEwAVQBEAEkATgBHACAAQgBVAFQAIABOAE8AVAAgAEwASQBN"
        "AEkAVABFAEQAIABUAE8AIABBAE4AWQAgAFcAQQBSAFIAQQBOAFQASQBFAFMA"
        "IABPAEYAIABNAEUAUgBDAEgAQQBOAFQAQQBCAEkATABJAFQAWQAsACAARgBJ"
        "AFQATgBFAFMAUwAgAEYATwBSACAAQQAgAFAAQQBSAFQASQBDAFUATABBAFIA"
        "IABQAFUAUgBQAE8AUwBFACAAQQBOAEQAIABOAE8ATgBJAE4ARgBSAEkATgBH"
        "AEUATQBFAE4AVAAgAE8ARgAgAEMATwBQAFkAUgBJAEcASABUACwAIABQAEEA"
        "VABFAE4AVAAsACAAVABSAEEARABFAE0AQQBSAEsALAAgAE8AUgAgAE8AVABI"
        "AEUAUgAgAFIASQBHAEgAVAAuACAASQBOACAATgBPACAARQBWAEUATgBUACAA"
        "UwBIAEEATABMACAAQgBJAFQAUwBUAFIARQBBAE0AIABPAFIAIABUAEgARQAg"
        "AEcATgBPAE0ARQAgAEYATwBVAE4ARABBAFQASQBPAE4AIABCAEUAIABMAEkA"
        "QQBCAEwARQAgAEYATwBSACAAQQBOAFkAIABDAEwAQQBJAE0ALAAgAEQAQQBN"
        "AEEARwBFAFMAIABPAFIAIABPAFQASABFAFIAIABMAEkAQQBCAEkATABJAFQA"
        "WQAsACAASQBOAEMATABVAEQASQBOAEcAIABBAE4AWQAgAEcARQBOAEUAUgBB"
        "AEwALAAgAFMAUABFAEMASQBBAEwALAAgAEkATgBEAEkAUgBFAEMAVAAsACAA"
        "SQBOAEMASQBEAEUATgBUAEEATAAsACAATwBSACAAQwBPAE4AUwBFAFEAVQBF"
        "AE4AVABJAEEATAAgAEQAQQBNAEEARwBFAFMALAAgAFcASABFAFQASABFAFIA"
        "IABJAE4AIABBAE4AIABBAEMAVABJAE8ATgAgAE8ARgAgAEMATwBOAFQAUgBB"
        "AEMAVAAsACAAVABPAFIAVAAgAE8AUgAgAE8AVABIAEUAUgBXAEkAUwBFACwA"
        "IABBAFIASQBTAEkATgBHACAARgBSAE8ATQAsACAATwBVAFQAIABPAEYAIABU"
        "AEgARQAgAFUAUwBFACAATwBSACAASQBOAEEAQgBJAEwASQBUAFkAIABUAE8A"
        "IABVAFMARQAgAFQASABFACAARgBPAE4AVAAgAFMATwBGAFQAVwBBAFIARQAg"
        "AE8AUgAgAEYAUgBPAE0AIABPAFQASABFAFIAIABEAEUAQQBMAEkATgBHAFMA"
        "IABJAE4AIABUAEgARQAgAEYATwBOAFQAIABTAE8ARgBUAFcAQQBSAEUALgAN"
        "AAoADQAKAEUAeABjAGUAcAB0ACAAYQBzACAAYwBvAG4AdABhAGkAbgBlAGQA"
        "IABpAG4AIAB0AGgAaQBzACAAbgBvAHQAaQBjAGUALAAgAHQAaABlACAAbgBh"
        "AG0AZQBzACAAbwBmACAARwBuAG8AbQBlACwAIAB0AGgAZQAgAEcAbgBvAG0A"
        "ZQAgAEYAbwB1AG4AZABhAHQAaQBvAG4ALAAgAGEAbgBkACAAQgBpAHQAcwB0"
        "AHIAZQBhAG0AIABJAG4AYwAuACwAIABzAGgAYQBsAGwAIABuAG8AdAAgAGIA"
        "ZQAgAHUAcwBlAGQAIABpAG4AIABhAGQAdgBlAHIAdABpAHMAaQBuAGcAIABv"
        "AHIAIABvAHQAaABlAHIAdwBpAHMAZQAgAHQAbwAgAHAAcgBvAG0AbwB0AGUA"
        "IAB0AGgAZQAgAHMAYQBsAGUALAAgAHUAcwBlACAAbwByACAAbwB0AGgAZQBy"
        "ACAAZABlAGEAbABpAG4AZwBzACAAaQBuACAAdABoAGkAcwAgAEYAbwBuAHQA"
        "IABTAG8AZgB0AHcAYQByAGUAIAB3AGkAdABoAG8AdQB0ACAAcAByAGkAbwBy"
        "ACAAdwByAGkAdAB0AGUAbgAgAGEAdQB0AGgAbwByAGkAegBhAHQAaQBvAG4A"
        "IABmAHIAbwBtACAAdABoAGUAIABHAG4AbwBtAGUAIABGAG8AdQBuAGQAYQB0"
        "AGkAbwBuACAAbwByACAAQgBpAHQAcwB0AHIAZQBhAG0AIABJAG4AYwAuACwA"
        "IAByAGUAcwBwAGUAYwB0AGkAdgBlAGwAeQAuACAARgBvAHIAIABmAHUAcgB0"
        "AGgAZQByACAAaQBuAGYAbwByAG0AYQB0AGkAbwBuACwAIABjAG8AbgB0AGEA"
        "YwB0ADoAIABmAG8AbgB0AHMAIABhAHQAIABnAG4AbwBtAGUAIABkAG8AdAAg"
        "AG8AcgBnAC4AaAB0AHQAcAA6AC8ALwB3AHcAdwAuAGIAaQB0AHMAdAByAGUA"
        "YQBtAC4AYwBvAG0AATUAuADLAMsAwQCqAJwBpgC4AGYAAABxAMsAoAKyAIUA"
        "dQC4AMMBywGJAi0AywCmAPAA0wCqAIcAywOqBAABSgAzAMsAAADZBQIA9AFU"
        "ALQAnAE5ARQBOQcGBAAETgS0BFIEuATnBM0ANwRzBM0EYARzATMDogVWBaYF"
        "VgU5A8UCEgDJAB8AuAHfAHMAugPpAzMDvAREBA4A3wPNA6oA5QOqBAQAAADL"
        "AI8ApAB7ALgAFAFvAH8CewJSAI8AxwXNAJoAmgBvAMsAzQGeAdMA8AC6AYMA"
        "1QCYAwQCSACeAdUAwQDLAPYAgwNUAn8AAAMzAmYA0wDHAKQAzQCPAJoAcwQA"
        "BdUBCgD+AisApAC0AJwAAABiAJwAAAAdAy0F1QXVBdUF8AB/AHsAVACkBrgG"
        "FAcjAdMAuADLAKYBwwHsBpMAoADTA1wDcQPbAYUEIwSoBEgAjwE5ARQBOQNg"
        "AI8F1QGaBhQHIwZmAXkEYARgBGAEewCcAAACdwRgAaoA6QRgB2IAewDFAH8C"
        "ewAAALQCUgXNAGYAvABmAHcGEADNATsBhQOJAI8AewAAAB0AzQdKBC8AnACc"
        "AAAHfQBvAAAAbwM1AGoAbwB7AK4AsgAtA5YAjwJ7APYAgwNUBjcF9gCPAJwE"
        "4QJmAI8BjQL2AM0DRAApAGYE7gBzAAAUALgCgED/+/4D+hQD+SUD+DID95YD"
        "9g4D9f4D9P4D8yUD8g4D8ZYD8CUD74pBBe/+A+6WA+2WA+z6A+v6A+r+A+k6"
        "A+hCA+f+A+YyA+XkUwXllgPkikEF5FMD4+IvBeP6A+IvA+H+A+D+A98yA94U"
        "A92WA9z+A9sSA9p9A9m7A9j+A9aKQQXWfQPV1EcF1X0D1EcD09IbBdP+A9Ib"
        "A9H+A9D+A8/+A87+A82WA8zLHgXM/gPLHgPKMgPJ/gPGhREFxhwDxRYDxP4D"
        "w/4Dwv4Dwf4DwP4Dv/4Dvv4Dvf4DvP4Du/4DuhEDuYYlBbn+A7i3uwW4/gO3"
        "tl0Ft7sDt4AEtrUlBbZdQP8DtkAEtSUDtP4Ds5YDsv4Dsf4DsP4Dr/4DrmQD"
        "rQ4DrKslBaxkA6uqEgWrJQOqEgOpikEFqfoDqP4Dp/4Dpv4DpRIDpP4Do6IO"
        "BaMyA6IOA6FkA6CKQQWglgOf/gOenQwFnv4DnQwDnJsZBZxkA5uaEAWbGQOa"
        "EAOZCgOY/gOXlg0Fl/4Dlg0DlYpBBZWWA5STDgWUKAOTDgOS+gORkLsFkf4D"
        "kI9dBZC7A5CABI+OJQWPXQOPQASOJQON/gOMiy4FjP4Diy4DioYlBYpBA4mI"
        "CwWJFAOICwOHhiUFh2QDhoURBYYlA4URA4T+A4OCEQWD/gOCEQOB/gOA/gN/"
        "/gNA/359fQV+/gN9fQN8ZAN7VBUFeyUDev4Def4DeA4DdwwDdgoDdf4DdPoD"
        "c/oDcvoDcfoDcP4Db/4Dbv4DbCEDa/4DahFCBWpTA2n+A2h9A2cRQgVm/gNl"
        "/gNk/gNj/gNi/gNhOgNg+gNeDANd/gNb/gNa/gNZWAoFWfoDWAoDVxYZBVcy"
        "A1b+A1VUFQVVQgNUFQNTARAFUxgDUhQDUUoTBVH+A1ALA0/+A05NEAVO/gNN"
        "EANM/gNLShMFS/4DSkkQBUoTA0kdDQVJEANIDQNH/gNGlgNFlgNE/gNDAi0F"
        "Q/oDQrsDQUsDQP4DP/4DPj0SBT4UAz08DwU9EgM8Ow0FPED/DwM7DQM6/gM5"
        "/gM4NxQFOPoDNzYQBTcUAzY1CwU2EAM1CwM0HgMzDQMyMQsFMv4DMQsDMC8L"
        "BTANAy8LAy4tCQUuEAMtCQMsMgMrKiUFK2QDKikSBSolAykSAygnJQUoQQMn"
        "JQMmJQsFJg8DJQsDJP4DI/4DIg8DIQEQBSESAyBkAx/6Ax4dDQUeZAMdDQMc"
        "EUIFHP4DG/oDGkIDGRFCBRn+AxhkAxcWGQUX/gMWARAFFhkDFf4DFP4DE/4D"
        "EhFCBRL+AxECLQURQgMQfQMPZAMO/gMNDBYFDf4DDAEQBQwWAwv+AwoQAwn+"
        "AwgCLQUI/gMHFAMGZAMEARAFBP4DQBUDAi0FA/4DAgEQBQItAwEQAwD+AwG4"
        "AWSFjQErKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysr"
        "KysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysr"
        "KysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysr"
        "KysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysAKysrKysrKysr"
        "KysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysr"
        "KysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysr"
        "KysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysrKysr"
        "KysrKysrKysrKysrKx22BgUEAwIBACwgELACJUlksEBRWCDIWSEtLLACJUlk"
        "sEBRWCDIWSEtLCAQByCwAFCwDXkguP//UFgEGwVZsAUcsAMlCLAEJSPhILAA"
        "ULANeSC4//9QWAQbBVmwBRywAyUI4S0sS1BYILD9RURZIS0ssAIlRWBELSxL"
        "U1iwAiWwAiVFRFkhIS0sRUQtAAACAGb+lgRmBaQAAwAHABpADAT7AAb7AQgF"
        "fwIEAC/E1OwxABDU7NTsMBMRIRElIREhZgQA/HMDG/zl/pYHDvjycgYpAAIB"
        "NQAAAgAF1QADAAkAQEAPBwCDBIECCAcFAQMEAAAKEPw87DI5OTEAL+T8zDAB"
        "S7ALVFi9AAoAQAABAAoACv/AOBE3OFm2AAsgC1ALA10lMxUjETMRAyMDATXL"
        "y8sUohX+/gXV/XH+mwFlAAIAxQOqAukF1QADAAcATUAPBQGEBACBCAQFBgAF"
        "AgQIEPz83OwxABD0POwyMAFLsBJUS7ATVFtYvQAIAEAAAQAIAAj/wDgRNzhZ"
        "QA8wCUAJUAlgCXAJoAm/CQddAREjESERIxEBb6oCJKoF1f3VAiv91QIrAAAC"
        "AJ4AAAYXBb4AAwAfAGBAMRsLAIcHBB0JBRkNAocXEw8VER8eHBsaFxYVFBMS"
        "ERAODQwJCAcGBQQDAgEAGgoYBiAQ/MwXOTEALzzUPDz8PDzUPDzEMuwyMjBA"
        "EQsBCwILDAsNFAQaERoSFB8IAV0BIQMhCwEhEzMDIRUhAyEVIQMjEyEDIxMh"
        "NSETITUhEwQX/t1UASVEaAEkaaBnATj+oVIBPv6baKBn/ttnoWj+xQFgVP6+"
        "AWlmA4X+sgOH/mEBn/5hmv6ymf5iAZ7+YgGemQFOmgGfAAADAKr+0wRtBhQA"
        "IQAoAC8A1UBVIgIKCwonASYoAgsLCh0BHhwCLykvGwIpKS9CExEQIgobKQQX"
        "BgkqIQUCF4YWBoYFESMaihaJEAAqigWJAi0IFgoeBykaEgMACSIQCQMBByYI"
        "DQUGMBD8POz0Fzz8Fzz05OwxAC/k7MTU5OwyxBDuEO4REjkRORESFzkREjkw"
        "S1NYBxAE7QcQDu0RFzkHEA7tERc5BxAE7VkiAUuwCVRYvQAwAEAAAQAwADD/"
        "wDgRNzhZAUuwDFRLsBBUW0uwD1RbWL0AMP/AAAEAMAAwAEA4ETc4WQEjAy4B"
        "JzUeARcRLgE1NDY3NTMVHgEXFS4BJxEeARUUBgcDEQ4BFRQWFxE+ATU0JgK0"
        "ZAFp0mpm0W/dydrMZF2uU1OvXOPW49ZkdHpx4X+Be/7TAS0CLS20QEEBAcgk"
        "rJajvA7r6AQfG68qLgT+VSO0nKnDDwMAAZoNalhWYNX+TxFuWlhoAAAFAHH/"
        "4wcpBfAACwAXACMAJwAzAJVANiQPJSYlJg8nJCdCAJIMHpIujRiSJAaSDI0m"
        "EowoJJE0JyEbJQkDDRUOCQ0PIQ0rDhsNDzELNBD8xOz07BDu9u4RORESOTEA"
        "EOQy9Dzk7BDu9u4Q7jBLU1gHEAXtBxAF7VkiAUuwCVRLsAtUW0uwDFRbS7AU"
        "VFtLsA5UW0uwDVRbWL0ANABAAAEANAA0/8A4ETc4WQEiBhUUFjMyNjU0Jicy"
        "FhUUBiMiJjU0NgEiBhUUFjMyNjU0JiUzASMTMhYVFAYjIiY1NDYF0VdjY1dV"
        "Y2NVnrq7naC6u/yXVmNiV1djZAMxoPxaoB+evLufn7m6ApGUhIKVlYKDlX/c"
        "u7vb27u82wJhlYKElJSEgZZ/+fMGDdu7vdrbvLrcAAACAIH/4wX+BfAACQAw"
        "Ac1Alg0BDgyGERIRC4YKCxISEQmGAAkVFhUHAQYIhhYWFQIBAwGGHR4dAIYJ"
        "AB4eHSAfAiEeEQoTChcWFQMYFBETCgcIAgYJERMTCgIBAgMAEQoTChcWAhgV"
        "ERMKFBETEwpCEgsJAwYACh4DKBUOBignBpUYK5UnlCSRGIwOEwouCw4JAC4S"
        "FScOHgMuEichDhEPEyEDEhsQMRD87MTU1OwQxu4RORESOTkROTkRORE5MQAv"
        "xuT25u4Q7hDGERI5ERc5ERc5MEtTWAcQBe0HBe0RFzkHEAXtERc5BxAF7REX"
        "OQcF7REXOQcQBe0RFzkHEAjtBxAO7REXOQcQDu0RFzkHEAjtBxAI7QcQDu0R"
        "FzlZIrIPMgEBXUCyBwsFIgkpHAAcAR8CFwsqACoBJhI6ADQSRAteAFkBWgpV"
        "EloaWh9ZMGceewCbAJoBmQKXCJULkxWVFpUimS0fCQsJDAgRDCcMKBgCGwkZ"
        "CxkMGREcFBwVFh0fMicAJwEpCSMSKhMqFCgVLzI7CTQSORM/MkoJTBRLFUYZ"
        "TzJWAVoJWQxVElkTXB9fMmoMaRFgMnUBeQx6EZMAkwGXApUFnAecCJ8Imgmb"
        "C5oMkDKgMrAyOV0AXQEOARUUFjMyNjcJAT4BNzMGAgcBIycOASMiADU0Njcu"
        "ATU0NjMyFhcVLgEjIgYVFBYB8ltV1KBfpkn+ewH8O0IGugxoXQEX/I9o5IPx"
        "/s6GhjAy3rhTpVVXnkRpgzsDI1GhWJLCP0ACj/34WctyhP7+fv7jk1lXARPX"
        "gOFjP308osUkJLYvMW9YM2cAAQDFA6oBbwXVAAMAQkAKAYQAgQQABQIEBBD8"
        "7DEAEPTsMAFLsBJUS7ATVFtYvQAEAEAAAQAEAAT/wDgRNzhZQA1ABVAFYAVw"
        "BZAFoAUGXQERIxEBb6oF1f3VAisAAAEAsP7yAnsGEgANAE9ADwaYAJcODQcA"
        "AxIGABMKDhDc5DLsETk5MQAQ/OwwAUuwE1RYvQAOAEAAAQAOAA7/wDgRNzhZ"
        "AUuwD1RYvQAO/8AAAQAOAA4AQDgRNzhZAQYCFRQSFyMmAjU0EjcCe4aCg4Wg"
        "lpWUlwYS5v4+5+f+O+XrAcbg3wHE7AABAKT+8gJvBhIADQAfQA8HmACXDgcB"
        "AAsSBBMIAA4Q3Dz07BE5OTEAEPzsMBMzFhIVFAIHIzYSNTQCpKCWlZWWoIWD"
        "gwYS7P483+D+OuvlAcXn5wHCAAEAPQJKA8MF8AARAE5ALBANCwAEDAkHBAIE"
        "CAOZBREMmQoBDpESCAwKAwkGEQMBAwIAFA8ECwkUDQYSENQ85DLcPOQyFzkR"
        "Ehc5MQAQ9NQ87DLE7DIXORIXOTABBQUHJREjEQUnJSU3BREzESUDw/6ZAWc6"
        "/rBy/rA6AWf+mToBUHIBUATfwsNiy/6HAXnLYsPCY8sBef6HywABANkAAAXb"
        "BQQACwAjQBEACQGcBwMFAhUEABcKBhUIDBDc/Dz8POwxAC/UPPw8xDABESEV"
        "IREjESE1IREDrgIt/dOo/dMCLQUE/dOq/dMCLaoCLQABAJ7/EgHDAP4ABQAZ"
        "QAwDngCDBgMEARkAGAYQ/OzUzDEAEPzsMDczFQMjE/DTpIFS/qz+wAFAAAEA"
        "ZAHfAn8CgwADABG2AJwCBAEABBDczDEAENTsMBMhFSFkAhv95QKDpAAAAQDb"
        "AAABrgD+AAMAEbcAgwIBGQAYBBD87DEAL+wwNzMVI9vT0/7+AAEAAP9CArIF"
        "1QADAC1AFAAaAQIBAhoDAANCAp8AgQQCAAEDL8Q5OTEAEPTsMEtTWAcQBe0H"
        "EAXtWSIBMwEjAgiq/fiqBdX5bQAAAgCH/+MEjwXwAAsAFwAjQBMGoBIAoAyR"
        "EowYCRwPHgMcFRsYEPzs9OwxABDk9OwQ7jABIgIREBIzMhIREAInMgAREAAj"
        "IgAREAACi5ydnZydnZ2d+wEJ/vf7+/73AQkFUP7N/sz+zf7NATMBMwE0ATOg"
        "/nP+hv6H/nMBjQF5AXoBjQAAAQDhAAAEWgXVAAoAS0AVQgOgBAKgBYEHAKAJ"
        "CB8GHAMAHwELENTsxPzsMQAv7DL07NTsMEtTWFkiAUuwD1RYvQAL/8AAAQAL"
        "AAsAQDgRNzhZtA8DDwQCXTchEQU1JTMRIRUh/gFK/pkBZcoBSvykqgRzSLhI"
        "+tWqAAABAJYAAARKBfAAHAClQCcZGhsDGBwRBQQAEQUFBEIQoRGUDaAUkQQA"
        "oAIAEAoCAQocFxADBh0Q/MTU7MDAERI5MQAv7DL07PTsMEtTWAcQBe0HBe0R"
        "FzlZIgFLsBVUS7AWVFtLsBRUW1i9AB0AQAABAB0AHf/AOBE3OFlAMlUEVgVW"
        "B3oEegV2G4cZBwQABBkEGgQbBRx0AHYGdRpzG3QcggCGGYIaghuCHKgAqBsR"
        "XQBdJSEVITU2ADc+ATU0JiMiBgc1PgEzMgQVFAYHBgABiQLB/ExzAY0zYU2n"
        "hl/TeHrUWOgBFEVbGf70qqqqdwGROm2XSXeWQkPMMTLowlylcB3+6wAAAQCc"
        "/+MEcwXwACgAe0AuABUTCoYJH4YgE6AVDaAJkwYcoCCTI5EGjBWjKRYcEwAD"
        "FBkcJiAQHAMUHwkGKRD8xMTU7PTsERc5OTEAEOzk9OTsEObuEO4Q7hDuERI5"
        "MAFLsBZUS7AUVFtYvQApAEAAAQApACn/wDgRNzhZQAlkHmEfYSBkIQQAXQEe"
        "ARUUBCEiJic1HgEzMjY1NCYrATUzMjY1NCYjIgYHNT4BMzIEFRQGAz+Ro/7Q"
        "/uhex2pUyG2+x7mlrraVnqOYU75yc8lZ5gEMjgMlH8SQ3fIlJcMxMpaPhJWm"
        "d3BzeyQmtCAg0bJ8qwACAGQAAASkBdUAAgANAIxAHQENAw0AAwMNQgADCweg"
        "BQEDgQkBDAoAHAYIBAwOENzUPMTsMhE5MQAv5NQ87DISOTBLU1gHEATJBxAF"
        "yVkiAUuwC1RLsA1UW1i9AA4AQAABAA4ADv/AOBE3OFlAKgsAKgBIAFkAaQB3"
        "AIoABxYBKwAmASsDNgFOAU8MTw1WAWYBdQF6A4UBDV0AXQkBIQMzETMVIxEj"
        "ESE1Awb+AgH+Nf7V1cn9XgUl/OMDzfwzqP6gAWDDAAABAJ7/4wRkBdUAHQB1"
        "QCMEGgcRhhAdGqAHFKAQiQ0CoACBDYwHpB4XHAEKAxwAChAGHhD8xNTsEMTu"
        "MQAQ5OT07BDm7hD+xBDuERI5MAFLsBZUS7AUVFtYvQAeAEAAAQAeAB7/wDgR"
        "NzhZAUuwD1RYvQAe/8AAAQAeAB4AQDgRNzhZEyEVIRE+ATMyABUUACEiJic1"
        "HgEzMjY1NCYjIgYH3QMZ/aAsWCz6AST+1P7vXsNoWsBrrcrKrVGhVAXVqv6S"
        "Dw/+7urx/vUgIMsxMLacnLYkJgAAAgCP/+MElgXwAAsAJABYQCQTBgANhgwA"
        "oBYGoBwWpRCgDIkikRyMJQwiCRwZHhMcAyEfGyUQ/Ozs9OzkMQAQ5PTk/OQQ"
        "7hDuEO4REjkwQBTLAMsBzQLNA80EywXLBgekHrIeAl0BXQEiBhUUFjMyNjU0"
        "JgEVLgEjIgIDPgEzMgAVFAAjIAAREAAhMhYCpIifn4iIn58BCUybTMjTDzuy"
        "a+EBBf7w4v79/u4BUAEbTJsDO7qiobu7oaK6Anm4JCb+8v7vV13+7+vm/uoB"
        "jQF5AWIBpR4AAAEAqAAABGgF1QAGAGNAGAURAgMCAxEEBQRCBaAAgQMFAwEE"
        "AQAGBxD8zMQROTkxAC/07DBLU1gHEAXtBxAF7VkiAUuwFlRYvQAHAEAAAQAH"
        "AAf/wDgRNzhZQBJYAgEGAxoFOQVIBWcDsACwBgddAF0TIRUBIwEhqAPA/eLT"
        "Af79MwXVVvqBBSsAAAMAi//jBIsF8AALACMALwBDQCUYDACgJwagHi2gEpEe"
        "jCejMBgMJCocFSQcDwkcFRseAxwPIRswEPzE7PTE7BDuEO4ROTkxABDs5PTs"
        "EO4Q7jk5MAEiBhUUFjMyNjU0JiUmJjU0NjMyFhUUBgcWFhUUBCMiJDU0NhMU"
        "FjMyNjU0JiMiBgKLkKWlkJCmpf6lgpH/3t/+kYGSo/739/f+96RIkYOCk5OC"
        "g5ECxZqHh5qbhoeaViCygLPQ0LOAsiAixo/Z6OjZj8YBYXSCgnR0goIAAAIA"
        "gf/jBIcF8AAYACQAWEAjBx8ZAYYAGaAKpQSgAIkWH6AQkRaMJQccHCETHgAi"
        "IhwNGyUQ/Ozk9OzsMQAQ5PTsEOb+9e4Q7hESOTBAFsQZwhrAG8AcwB3CHsQf"
        "B6oSvBLpEgNdAV03NR4BMzISEw4BIyIANTQAMyAAERAAISImATI2NTQmIyIG"
        "FRQW4UycS8jTDzqybOD++wEQ4gEDARH+sf7lTJwBPoifn4iIn58fuCQmAQ0B"
        "ElZcAQ/r5gEW/nP+hv6f/lseApe6oqG7u6GiugAAAgDwAAABwwQjAAMABwAc"
        "QA4GgwSmAIMCBQEDBAAYCBD8POwyMQAv7PTsMDczFSMRMxUj8NPT09P+/gQj"
        "/gACAJ7/EgHDBCMAAwAJACVAEwKDAAeeBIMApgoHCAUBGQQAGAoQ/DzsMtTM"
        "MQAQ5PzsEO4wEzMVIxEzFQMjE/DT09OkgVIEI/792az+wAFAAAEA2QBeBdsE"
        "pgAGAE1AKgKcAwQDAZwAAQQEAwGcAgEFBgUAnAYFQgUEAgEABQOoBqcHAQIA"
        "JAQjBxD87DI5MQAQ9OwXOTBLU1gHBO0HEAjtBxAI7QcQBO1ZIgkCFQE1AQXb"
        "+/gECPr+BQID8P6R/pO2AdGmAdEAAAIA2QFgBdsDogADAAcAHEANAJwCBpwE"
        "CAUBBAAjCBD8PMQyMQAQ1OzU7DATIRUhFSEVIdkFAvr+BQL6/gOiqPCqAAEA"
        "2QBeBdsEpgAGAE9AKwacAAYDBAMFnAQEAwCcAQIBBpwFBgICAUIGBQMCAAUE"
        "qAGnBwYCJAQAIwcQ/DzsOTEAEPTsFzkwS1NYBxAI7QcQBO0HEATtBxAI7Vki"
        "EzUBFQE1AdkFAvr+BAYD8Lb+L6b+L7YBbQACAJMAAAOwBfAAAwAkAHBAKyQe"
        "CQYECh0TBAAUhhOIEJUXkQCDAh0aDQkFBAoeAQ0cGgQcBQEDACYaEyUQ3MT8"
        "7NTsEO4ROTkREjkREjkxAC/u9v707hDNETk5FzkwAUuwDFRYvQAlAEAAAQAl"
        "ACX/wDgRNzhZtnkJegp6IANdJTMVIxMjNTQ2PwE+ATU0JiMiBgc1PgEzMhYV"
        "FAYPAQ4BBw4BFQGHy8vFvzhaWjkzg2xPs2FewWe430haWC8nCAYG/v4BkZpl"
        "glZZNV4xWW5GQ7w5OMKfTIlWVi81GRU8NAAAAgCH/pwHcQWiAAsATACVQDIY"
        "DAMJqRkVGwOpTA80Mw+sMKk3FawkqTdDTTM0HhoAKBIGGAwoGiseKEkSKyoo"
        "SSw9TRDc7PzsEP79/jzGEO4REjk5MQAQ1MT87BD+7dTGEMXuMhDE7hE5OTAA"
        "S7AJVEuwDFRbS7AQVFtLsBNUW0uwFFRbWL0ATf/AAAEATQBNAEA4ETc4WUAJ"
        "D04fTi9OP04EAV0BFBYzMjY1NCYjIgYBDgEjIiY1NDYzMhYXNTMRPgE1NCYn"
        "JiQjIgYHBgIVFBIXFgQzMjY3FwYEIyIkJyYCNTQSNzYkMzIEFx4BFRAABQL6"
        "jnx7jZB6eY8CITybZ6zX2KtnnDuPkqU/QGj+1bB74mCdsXNtaQEUnYH5aFp9"
        "/tmYuf64gICGiH6BAVK91AFre0tP/sL+6AIZj6OkjoylpP5ITUn5yMj6S0yD"
        "/SAW37FrvFCDi0FAZv61wZ/+6mpobVdRb2Fng319AUm9tgFKfX+HrqBi5nv+"
        "+f7QBgAAAgAQAAAFaAXVAAIACgC6QEEAEQEABAUEAhEFBQQBEQoDCgARAgAD"
        "AwoHEQUEBhEFBQQJEQMKCBEKAwpCAAMHlQEDgQkFCQgHBgQDAgEACQUKCxDU"
        "xBc5MQAvPOTU7BI5MEtTWAcQBe0HBe0HEAXtBwXtBxAI7QcQBe0HEAXtBxAI"
        "7VkisiAMAQFdQDoPAFgAdgBwAIwABQcBCAIGAwkEFgEZAlYBWAJQDGcBaAJ4"
        "AXYCfANyBHcHeAiHAYgCgAyYApkDlgQXXQBdCQEhATMBIwMhAyMCvP7uAiX+"
        "e+UCOdKI/V+I1QUO/RkDrvorAX/+gQADAMkAAATsBdUACAARACAAQ0AjGQCV"
        "CgmVEoEBlQqtHxELCAITGR8FAA4cFgUZHC4JABwSBCEQ/Owy/OzU7BEXOTk5"
        "MQAv7Oz07BDuOTCyDyIBAV0BESEyNjU0JiMBESEyNjU0JiMlITIWFRQGBx4B"
        "FRQEIyEBkwFEo52do/68ASuUkZGU/gsCBOf6gHyVpf7w+/3oAsn93YeLjIUC"
        "Zv4+b3JxcKbAsYmiFCDLmMjaAAEAc//jBScF8AAZADZAGg2hDq4KlREBoQCu"
        "BJUXkRGMGgcZDQAwFBAaEPzsMuwxABDk9Oz07BDu9u4wtA8bHxsCAV0BFS4B"
        "IyAAERAAITI2NxUOASMgABEQACEyFgUnZueC/wD+8AEQAQCC52Zq7YT+rf56"
        "AYYBU4btBWLVX17+x/7Y/tn+x15f00hIAZ8BZwFoAZ9HAAIAyQAABbAF1QAI"
        "ABEALkAVAJUJgQGVEAgCEAoABRkNMgAcCQQSEPzs9OwROTk5OTEAL+z07DCy"
        "YBMBAV0BETMgABEQACElISAAERAAKQEBk/QBNQEf/uH+y/5CAZ8BsgGW/mj+"
        "UP5hBS/7dwEYAS4BLAEXpv6X/oD+fv6WAAEAyQAABIsF1QALAC5AFQaVBAKV"
        "AIEIlQStCgUBCQcDHAAEDBD87DLUxMQxAC/s7PTsEO4wsh8NAQFdEyEVIREh"
        "FSERIRUhyQOw/RoCx/05Avj8PgXVqv5Gqv3jqgABAMkAAAQjBdUACQApQBIG"
        "lQQClQCBBK0IBQEHAxwABAoQ/Owy1MQxAC/s9OwQ7jCyDwsBAV0TIRUhESEV"
        "IREjyQNa/XACUP2wygXVqv5Iqv03AAABAHP/4wWLBfAAHQA5QCAABRsBlQMb"
        "lQgSoRGuFZUOkQiMHgIAHBE0BDMYGQsQHhD87Pzk/MQxABDk9Oz07BD+1O4R"
        "OTkwJREhNSERBgQjIAAREAAhMgQXFSYmIyAAERAAITI2BMP+tgISdf7moP6i"
        "/nUBiwFekgEHb3D8i/7u/u0BEwESa6jVAZGm/X9TVQGZAW0BbgGZSEbXX2D+"
        "zv7R/tL+ziUAAQDJAAAFOwXVAAsALEAUCJUCrQQAgQoGBwMcBTgJARwABAwQ"
        "/Owy/OwyMQAvPOQy/OwwslANAQFdEzMRIREzESMRIREjycoC3srK/SLKBdX9"
        "nAJk+isCx/05AAABAMkAAAGTBdUAAwA5twCvAgEcAAQEEPzsMQAv7DABS7AQ"
        "VFi9AAT/wAABAAQABABAOBE3OFlADTAFQAVQBWAFjwWfBQZdEzMRI8nKygXV"
        "+isAAf+W/mYBkwXVAAsATUATCwIAB5UFsACBDAUIBjkBHAAEDBD87OQ5OTEA"
        "EOT87BE5OTABS7AQVFi9AAz/wAABAAwADABAOBE3OFlADTANQA1QDWANjw2f"
        "DQZdEzMREAYrATUzMjY1ycrN400/hm4F1fqT/vL0qpbCAAABAMkAAAVqBdUA"
        "CgDvQCgIEQUGBQcRBgYFAxEEBQQCEQUFBEIIBQIDAwCvCQYFAQQGCAEcAAQL"
        "EPzsMtTEETkxAC887DIXOTBLU1gHEATtBxAF7QcQBe0HEATtWSKyCAMBAV1A"
        "khQCAQQCCQgWAigFKAg3AjYFNAhHAkYFQwhVAmcCdgJ3BYMCiAWPCJQCmwjn"
        "AhUGAwkFCQYbAxkHBQoDCgcYAygFKwYqBzYENgU2BjUHMAxBA0AERQVABkAH"
        "QAxiA2AEaAVnB3cFcAyLA4sFjgaPB48MmgOdBp0HtgO1B8UDxQfXA9YH6APp"
        "BOgF6gb3A/gF+QYsXXEAXXETMxEBIQkBIQERI8nKAp4BBP0bAxr+9v0zygXV"
        "/YkCd/1I/OMCz/0xAAABAMkAAARqBdUABQAlQAwClQCBBAEcAzoABAYQ/Ozs"
        "MQAv5OwwQAkwB1AHgAOABAQBXRMzESEVIcnKAtf8XwXV+tWqAAEAyQAABh8F"
        "1QAMAL9ANAMRBwgHAhEBAggIBwIRAwIJCgkBEQoKCUIKBwIDCAMArwgLBQkI"
        "AwIBBQoGHAQ+ChwABA0Q/Oz87BEXOTEALzzE7DIRFzkwS1NYBxAF7QcQCO0H"
        "EAjtBxAF7VkisnAOAQFdQFYDBw8IDwkCChUCFAcTCiYCJgcgByYKIAo0BzUK"
        "aQJ8AnsHeQqAAoIHggqQAhYEAQsDEwEbAyMBLAMnCCgJNAE8A1YIWQllCGoJ"
        "dgh5CYEBjQOVAZsDFF0AXRMhCQEhESMRASMBESPJAS0BfQF/AS3F/n/L/n/E"
        "BdX8CAP4+isFH/wABAD64QABAMkAAAUzBdUACQB5QB4HEQECAQIRBgcGQgcC"
        "AwCvCAUGAQcCHAQ2BxwABAoQ/Oz87BE5OTEALzzsMjk5MEtTWAcQBO0HEATt"
        "WSKyHwsBAV1AMDYCOAdIAkcHaQJmB4ACBwYBCQYVARoGRgFJBlcBWAZlAWkG"
        "eQaFAYoGlQGaBp8LEF0AXRMhAREzESEBESPJARAClsT+8P1qxAXV+x8E4for"
        "BOH7HwACAHP/4wXZBfAACwAXACNAEwaVEgCVDJESjBgJGQ8zAxkVEBgQ/Oz8"
        "7DEAEOT07BDuMAEiABEQADMyABEQACcgABEQACEgABEQAAMn3P79AQPc3AEB"
        "/v/cAToBeP6I/sb+xf6HAXkFTP64/uX+5v64AUgBGgEbAUik/lv+nv6f/lsB"
        "pAFiAWIBpQACAMkAAASNBdUACAATADpAGAGVEACVCYESEAoIAgQABRkNPxEA"
        "HAkEFBD87DL87BEXOTEAL/Ts1OwwQAsPFR8VPxVfFa8VBQFdAREzMjY1NCYj"
        "JSEyBBUUBCsBESMBk/6NmpqN/jgByPsBAf7/+/7KBS/9z5KHhpKm49vd4v2o"
        "AAIAc/74BdkF8AALAB0AUkAqERACDwEMDQwOAQ0NDEIPHgwGlRIAlRiREowN"
        "Hg0bDwwDCRkbMwMZFRAeEPzs/OwROTkROTEAEMTk9OwQ7jkSOTBLU1gHEAXt"
        "BxAF7Rc5WSIBIgAREAAzMgAREAATASMnBgYjIAAREAAhIAAREAIDJ9z+/QED"
        "3NwBAf7/PwEK9N0hIxD+xf6HAXkBOwE6AXjRBUz+uP7l/ub+uAFIARoBGwFI"
        "+s/+3e8CAgGlAWEBYgGl/lv+nv78/o4AAAIAyQAABVQF1QATABwAsUA1CQgH"
        "AwoGEQMEAwURBAQDQgYEABUDBBWVCRSVDYELBAUGAxEJABwWDgUKGRkEET8U"
        "ChwMBB0Q/Owy/MTsERc5ETk5OTEALzz07NTsEjkSORI5MEtTWAcQBe0HEAXt"
        "ERc5WSKyQB4BAV1AQnoTAQUABQEFAgYDBwQVABUBFAIWAxcEJQAlASUCJgMn"
        "BiYHJggmCSAeNgE2AkYBRgJoBXUEdQV3E4gGiAeYBpgHH10AXQEeARcTIwMu"
        "ASsBESMRISAWFRQGAREzMjY1NCYjA41Bez7N2b9Ki3jcygHIAQD8g/2J/pKV"
        "lZICvBaQfv5oAX+WYv2JBdXW2I26Ak/97oeDg4UAAAEAh//jBKIF8AAnAH5A"
        "PA0MAg4LAh4fHggJAgcKAh8fHkIKCx4fBBUBABWhFJQYlREElQCUJZERjCge"
        "CgsfGwcAIhsZDi0HGRQiKBDcxOz87OQREjk5OTkxABDk9OTsEO727hDGERc5"
        "MEtTWAcQDu0RFzkHEA7tERc5WSKyDykBAV22HykvKU8pA10BFS4BIyIGFRQW"
        "HwEeARUUBCEiJic1HgEzMjY1NCYvAS4BNTQkMzIWBEhzzF+ls3emeuLX/t3+"
        "52rvgHvscq28h5p74soBF/Vp2gWkxTc2gHZjZR8ZK9m22eAwL9BFRoh+bnwf"
        "GC3Aq8bkJgAAAf/6AAAE6QXVAAcASkAOBgKVAIEEAUADHABABQgQ1OT85DEA"
        "L/TsMjABS7AKVFi9AAgAQAABAAgACP/AOBE3OFlAEwAJHwAQARACHwcQCUAJ"
        "cAmfCQldAyEVIREjESEGBO/97sv97gXVqvrVBSsAAAEAsv/jBSkF1QARAEtA"
        "FggCEQsABZUOjAkAgRIIHAo4ARwAQRIQ/Oz87DEAEOQy9OwROTk5OTABS7AQ"
        "VFi9ABIAQAABABIAEv/AOBE3OFm2HxOPE58TA10TMxEUFjMyNjURMxEQACEg"
        "ABGyy67Dwq7L/t/+5v7l/t8F1fx18NPT8AOL/Fz+3P7WASoBJAABABAAAAVo"
        "BdUABgC3QCcEEQUGBQMRAgMGBgUDEQQDAAEAAhEBAQBCAwQBrwAGBAMCAAUF"
        "AQcQ1MQXOTEAL+wyOTBLU1gHEAXtBxAI7QcQCO0HEAXtWSKyUAgBAV1AYgAD"
        "KgNHBEcFWgN9A4MDBwYABwIIBAkGFQEUAhoEGgUqACYBJgIpBCkFJQYgCDgA"
        "MwEzAjwEPAU3BkgARQFFAkkESQVHBlkAVgZmAmkEaQV6AHYBdgJ5BHkFdQaA"
        "CJgAlwYpXQBdIQEzCQEzAQJK/cbTAdkB2tL9xwXV+xcE6forAAEARAAAB6YF"
        "1QAMAXtASQUaBgUJCgkEGgoJAxoKCwoCGgECCwsKBhEHCAcFEQQFCAgHAhED"
        "AgwADAERAAAMQgoFAgMGAwCvCwgMCwoJCAYFBAMCAQsHAA0Q1MwXOTEALzzs"
        "MjIXOTBLU1gHEAXtBxAI7QcQCO0HEAXtBxAI7QcQBe0HBe0HEAjtWSKyAA4B"
        "AV1A8gYCBgUCCgAKAAoSCigFJAogCj4CPgU0CjAKTAJNBUIKQApZAmoCawVn"
        "CmAKewJ/AnwFfwWACpYClQUdBwAJAggDAAQGBQAFAAYBBwQIAAgHCQAJBAoK"
        "DAAOGgMVBBUIGQwQDiAEIQUgBiAHIAgjCSQKJQsgDiAOPAI6AzUEMwUwCDYJ"
        "OQs/DDAORgBGAUoCQARFBUAFQgZCB0IIQAhACUQKTQxADkAOWAJWCFkMUA5m"
        "AmcDYQRiBWAGYAdgCGQJZApkC3cAdgF7AngDdwR0BXkGeQd3CHAIeAx/DH8O"
        "hgKHA4gEiQWFCYoLjw6XBJ8Orw5bXQBdEzMJATMJATMBIwkBI0TMAToBOeMB"
        "OgE5zf6J/v7F/sL+BdX7EgTu+xIE7vorBRD68AABAD0AAAU7BdUACwFdQEYJ"
        "EQoLCggRBwgLCwoIEQkIBQYFBxEGBgUDEQQFBAIRAQIFBQQCEQMCCwALAREA"
        "AAtCCwgFAgQDAK8JBgsIBQIEAAQGAAoMENTE3MQRFzkxAC887DIXOTBLU1gH"
        "EAXtBxAI7QcQCO0HEAXtBxAF7QcQCO0HEAjtBxAF7VkiAUuwDFRLsA1UW0uw"
        "DlRbWL0ADP/AAAEADAAMAEA4ETc4WUC4BwIICBYCGQgXCycIJws0AjgINgtL"
        "CFgFWwhmAmsIdwJ3C4YCgAKHBYsIhQuUApAClwWdCJYLGwYBCQMIBwcJFgEZ"
        "AxkHFwkQDSYBKAIpAyYFKAcnCSkLIA01ADQBPAM7BDoGOwc0CTQKOAs/DUgJ"
        "Tw1YC18NZQBlAWoDagRoBWkGaQdsCWwKeAN5BnkHeAh9CX8KeAuAAIABgwKI"
        "A4UFhAiDC48NkACQAZQClwWXBpUIkwufDa8NQF0AXRMzCQEzCQEjCQEjAYHZ"
        "AXMBddn+IAIA2f5c/lnaAhUF1f3VAiv9M/z4Anv9hQMdAAH//AAABOcF1QAI"
        "AJRAKAMRBAUEAhEBAgUFBAIRAwIIAAgBEQAACEICAwCvBgIHBEAFHABABwkQ"
        "1OT85BI5MQAv7DI5MEtTWAcQBe0HEAjtBxAI7QcQBe1ZIrIACgEBXUA8BQIU"
        "AjUCMAIwBTAIRgJAAkAFQAhRAlEFUQhlAoQCkwIQFgEaAx8KJgEpAzcBOANA"
        "CmcBaAN4A3AKnwoNXQBdAzMJATMBESMRBNkBngGb2f3wywXV/ZoCZvzy/TkC"
        "xwAAAQBcAAAFHwXVAAkAm0AbAxEHCAcIEQIDAkIIlQCBA5UFCAMAAUIEAAYK"
        "ENzE1OQROTkxAC/s9OwwS1NYBxAF7QcQBe1ZIgFLsAlUS7AKVFtYvQAKAEAA"
        "AQAKAAr/wDgRNzhZQEAFAgoHGAcpAiYHOAdIAkcHSAgJBQMLCAALFgMaCBAL"
        "Lws1AzkIPwtHA0oITwtVA1kIZgNpCG8LdwN4CH8LnwsWXQBdEyEVASEVITUB"
        "IXMElfxQA8f7PQOw/GcF1Zr7b6qaBJEAAAEAsP7yAlgGFAAHAFNADwSpBrIC"
        "qQCxCAUBA0MACBDc/MwyMQAQ/Oz07DABS7AMVFi9AAj/wAABAAgACABAOBE3"
        "OFkBS7ASVEuwE1RbWL0ACABAAAEACAAI/8A4ETc4WRMhFSMRMxUhsAGo8PD+"
        "WAYUj/n8jwABAAD/QgKyBdUAAwAtQBQCGgEBAAAaAwMCQgGfAIEEAgABAy/E"
        "OTkxABD07DBLU1gHEAXtBxAF7VkiEwEjAaoCCKr9+AXV+W0GkwAAAQDH/vIC"
        "bwYUAAcAPEAQA6kBsgWpALEIAEMEBgIECBD8PNzsMQAQ/Oz07DABS7APVEuw"
        "EFRbWL0ACP/AAAEACAAIAEA4ETc4WQERITUzESM1Am/+WO/vBhT43o8GBI8A"
        "AQDZA6gF2wXVAAYAGEAKAwQBAIEHAwEFBxDczDkxABD0zDI5MAEBIwEBIwED"
        "vAIfyf5I/kjJAh8F1f3TAYv+dQItAAAB/+z+HQQU/qwAAwAPtQCpAQACBBDE"
        "xDEA1OwwARUhNQQU+9j+rI+PAAABAKoE8AKJBmYAAwAxQAkBtACzBANEAQQQ"
        "3OwxABD07DAAS7AJVEuwDlRbWL0ABP/AAAEABAAEAEA4ETc4WQkBIwEBbwEa"
        "mf66Bmb+igF2AAIAe//jBC0EewAKACUAvEAnGR8LFwkOAKkXBrkOESCGH7oc"
        "uSO4EYwXDAAXAxgNCQgLHwMIFEUmEPzszNTsMjIROTkxAC/E5PT89OwQxu4Q"
        "7hE5ETkSOTBAbjAdMB4wHzAgMCEwIj8nQB1AHkAfQCBAIUAiUB1QHlAfUCBQ"
        "IVAiUCdwJ4Udhx6HH4cghyGFIpAnoCfwJx4wHjAfMCAwIUAeQB9AIEAhUB5Q"
        "H1AgUCFgHmAfYCBgIXAecB9wIHAhgB6AH4AggCEYXQFdASIGFRQWMzI2PQE3"
        "ESM1DgEjIiY1NDYzITU0JiMiBgc1PgEzMhYCvt+sgW+Zubi4P7yIrMv9+wEC"
        "p5dgtlRlvlrz8AIzZntic9m0KUz9gapmYcGivcASf4suLqonJ/wAAAIAuv/j"
        "BKQGFAALABwAOEAZA7kMDwm5GBWMD7gblxkAEhJHGAwGCBpGHRD87DIy9Owx"
        "AC/s5PTE7BDG7jC2YB6AHqAeAwFdATQmIyIGFRQWMzI2AT4BMzISERACIyIm"
        "JxUjETMD5aeSkqenkpKn/Y46sXvM///Me7E6ubkCL8vn58vL5+cCUmRh/rz+"
        "+P74/rxhZKgGFAAAAQBx/+MD5wR7ABkAP0AbAIYBiAQOhg2ICrkRBLkXuBGM"
        "GgcSDQBIFEUaEPzkMuwxABDk9OwQ/vTuEPXuMEALDxsQG4AbkBugGwUBXQEV"
        "LgEjIgYVFBYzMjY3FQ4BIyIAERAAITIWA+dOnVCzxsazUJ1OTaVd/f7WAS0B"
        "BlWiBDWsKyvjzc3jKyuqJCQBPgEOARIBOiMAAgBx/+MEWgYUABAAHAA4QBka"
        "uQAOFLkFCIwOuAGXAxcEAAgCRxESC0UdEPzs9OwyMjEAL+zk9MTsEMTuMLZg"
        "HoAeoB4DAV0BETMRIzUOASMiAhEQEjMyFgEUFjMyNjU0JiMiBgOiuLg6sXzL"
        "///LfLH9x6eSkqiokpKnA7YCXvnsqGRhAUQBCAEIAURh/hXL5+fLy+fnAAAC"
        "AHH/4wR/BHsAFAAbAHBAJAAVAQmGCIgFFakBBbkMAbsYuRK4DIwcGxUCCBUI"
        "AEsCEg9FHBD87PTsxBESOTEAEOT07OQQ7hDuEPTuERI5MEApPx1wHaAd0B3w"
        "HQU/AD8BPwI/FT8bBSwHLwgvCSwKbwBvAW8CbxVvGwldcQFdARUhHgEzMjY3"
        "FQ4BIyAAERAAMzIABy4BIyIGBwR//LIMzbdqx2Jj0Gv+9P7HASn84gEHuAKl"
        "iJq5DgJeWr7HNDSuKiwBOAEKARMBQ/7dxJe0rp4AAAEALwAAAvgGFAATAHBA"
        "HAUQAQwIqQYBhwCXDga8CgITBwAHCQUIDQ8LTBQQ/DzE/DzExBI5OTEAL+Qy"
        "/OwQ7jISOTkwAUuwClRYvQAU/8AAAQAUABQAQDgRNzhZAUuwDlRYvQAUAEAA"
        "AQAUABT/wDgRNzhZtkAVUBWgFQNdARUjIgYdASEVIREjESM1MzU0NjMC+LBj"
        "TQEv/tG5sLCuvQYUmVBoY4/8LwPRj067qwAAAgBx/lYEWgR7AAsAKABKQCMZ"
        "DB0JEoYTFrkPA7kmI7gnvAm5D70aHSYZAAgMRwYSEiBFKRD8xOz07DIyMQAv"
        "xOTs5PTE7BD+1e4REjk5MLZgKoAqoCoDAV0BNCYjIgYVFBYzMjYXEAIhIiYn"
        "NR4BMzI2PQEOASMiAhEQEjMyFhc1MwOipZWUpaWUlaW4/v76YaxRUZ5StbQ5"
        "snzO/PzOfLI5uAI9yNzcyMfc3Ov+4v7pHR6zLCq9v1tjYgE6AQMBBAE6YmOq"
        "AAABALoAAARkBhQAEwA0QBkDCQADDgEGhw4RuAyXCgECCABODQkIC0YUEPzs"
        "MvTsMQAvPOz0xOwREhc5MLJgFQEBXQERIxE0JiMiBhURIxEzET4BMzIWBGS4"
        "fHyVrLm5QrN1wcYCpP1cAp6fnr6k/YcGFP2eZWTvAAACAMEAAAF5BhQAAwAH"
        "ACtADga+BLEAvAIFAQgEAEYIEPw87DIxAC/k/OwwQAsQCUAJUAlgCXAJBQFd"
        "EzMRIxEzFSPBuLi4uARg+6AGFOkAAAL/2/5WAXkGFAALAA8AREAcCwIHAA6+"
        "DAeHBb0AvAyxEAgQBQZPDQEIDABGEBD8POwy5DkSOTEAEOzk9OwQ7hESOTkw"
        "QAsQEUARUBFgEXARBQFdEzMRFAYrATUzMjY1ETMVI8G4o7VGMWlMuLgEYPuM"
        "1sCcYZkGKOkAAQC6AAAEnAYUAAoAvEApCBEFBgUHEQYGBQMRBAUEAhEFBQRC"
        "CAUCAwO8AJcJBgUBBAYIAQgARgsQ/Owy1MQROTEALzzs5Bc5MEtTWAcQBO0H"
        "EAXtBxAF7QcQBO1ZIrIQDAEBXUBfBAIKCBYCJwIpBSsIVgJmAmcIcwJ3BYIC"
        "iQWOCJMClgWXCKMCEgkFCQYCCwMKBygDJwQoBSsGKwdADGgDYAyJA4UEiQWN"
        "Bo8HmgOXB6oDpwW2B8UH1gf3A/AD9wTwBBpdcQBdEzMRATMJASMBESO6uQIl"
        "6/2uAmvw/ce5BhT8aQHj/fT9rAIj/d0AAQDBAAABeQYUAAMAIrcAlwIBCABG"
        "BBD87DEAL+wwQA0QBUAFUAVgBXAF8AUGAV0TMxEjwbi4BhT57AAAAQC6AAAH"
        "HQR7ACIAWkAmBhIJGA8ABh0HFQyHHSADuBu8GRAHABEPCAgGUBEID1AcGAga"
        "RiMQ/Owy/Pz87BESOTEALzw85PQ8xOwyERIXOTBAEzAkUCRwJJAkoCSgJL8k"
        "3yT/JAkBXQE+ATMyFhURIxE0JiMiBhURIxE0JiMiBhURIxEzFT4BMzIWBClF"
        "wIKvvrlydY+muXJ3jaa5uT+weXqrA4l8dvXi/VwCnqGcvqT9hwKeopu/o/2H"
        "BGCuZ2J8AAABALoAAARkBHsAEwA2QBkDCQADDgEGhw4RuAy8CgECCABODQkI"
        "C0YUEPzsMvTsMQAvPOT0xOwREhc5MLRgFc8VAgFdAREjETQmIyIGFREjETMV"
        "PgEzMhYEZLh8fJWsublCs3XBxgKk/VwCnp+evqT9hwRgrmVk7wACAHH/4wR1"
        "BHsACwAXAEpAEwa5EgC5DLgSjBgJEg9RAxIVRRgQ/Oz07DEAEOT07BDuMEAj"
        "Pxl7AHsGfwd/CH8Jfwp/C3sMfw1/Dn8PfxB/EXsSoBnwGREBXQEiBhUUFjMy"
        "NjU0JicyABEQACMiABEQAAJzlKyrlZOsrJPwARL+7vDx/u8BEQPf58nJ5+jI"
        "x+mc/sj+7P7t/scBOQETARQBOAACALr+VgSkBHsAEAAcAD5AGxq5AA4UuQUI"
        "uA6MAb0DvB0REgtHFwQACAJGHRD87DIy9OwxABDk5OT0xOwQxO4wQAlgHoAe"
        "oB7gHgQBXSURIxEzFT4BMzISERACIyImATQmIyIGFRQWMzI2AXO5uTqxe8z/"
        "/8x7sQI4p5KSp6eSkqeo/a4GCqpkYf68/vj++P68YQHry+fny8vn5wACAHH+"
        "VgRaBHsACwAcAD5AGwO5DA8JuRgVuA+MG70ZvB0YDAYIGkcAEhJFHRD87PTs"
        "MjIxABDk5OT0xOwQxu4wQAlgHoAeoB7gHgQBXQEUFjMyNjU0JiMiBgEOASMi"
        "AhEQEjMyFhc1MxEjAS+nkpKoqJKSpwJzOrF8y///y3yxOri4Ai/L5+fLy+fn"
        "/a5kYQFEAQgBCAFEYWSq+fYAAAEAugAAA0oEewARADBAFAYLBwARCwOHDrgJ"
        "vAcKBggACEYSEPzE7DIxAC/k9OzE1MwREjkwtFATnxMCAV0BLgEjIgYVESMR"
        "MxU+ATMyFhcDSh9JLJynubk6uoUTLhwDtBIRy779sgRgrmZjBQUAAQBv/+MD"
        "xwR7ACcA50A8DQwCDgtTHx4ICQIHClMeHx5CCgseHwQVAIYBiQQUhhWJGLkR"
        "BLkluBGMKB4KCx8bBwBSGwgOBwgUIkUoEPzE7NTs5BESOTk5OTEAEOT07BD+"
        "9e4Q9e4SFzkwS1NYBxAO7REXOQcO7REXOVkisgAnAQFdQG0cChwLHAwuCSwK"
        "LAssDDsJOwo7CzsMCyAAIAEkAigKKAsqEy8ULxUqFigeKB8pICkhJCeGCoYL"
        "hgyGDRIAAAABAgIGCgYLAwwDDQMOAw8DEAMZAxoDGwMcBB0JJy8pPylfKX8p"
        "gCmQKaAp8CkYXQBdcQEVLgEjIgYVFBYfAR4BFRQGIyImJzUeATMyNjU0Ji8B"
        "LgE1NDYzMhYDi06oWomJYpQ/xKX32FrDbGbGYYKMZatAq5jgzma0BD+uKChU"
        "VEBJIQ4qmYmctiMjvjU1WVFLUCUPJJWCnqweAAABADcAAALyBZ4AEwA4QBkO"
        "BQgPA6kAEQG8CIcKCwgJAgQACBASDkYUEPw8xPw8xDI5OTEAL+z0PMTsMhE5"
        "OTCyrxUBAV0BESEVIREUFjsBFSMiJjURIzUzEQF3AXv+hUtzvb3VooeHBZ7+"
        "wo/9oIlOmp/SAmCPAT4AAAEArv/jBFgEYAATADZAGQMJAAMOAQaHDhGMCgG8"
        "DA0JCAtOAggARhQQ/Oz07DIxAC/kMvTE7BESFzkwtGAVzxUCAV0TETMRFBYz"
        "MjY1ETMRIzUOASMiJq64fHyVrbi4Q7F1wcgBugKm/WGfn76kAnv7oKxmY/AA"
        "AAEAPQAABH8EYAAGARJAJwMRBAUEAhEBAgUFBAIRAwIGAAYBEQAABkICAwC/"
        "BQYFAwIBBQQABxDUxBc5MQAv7DI5MEtTWAcQBe0HEAjtBxAI7QcQBe1ZIgFL"
        "sApUWL0AB//AAAEABwAHAEA4ETc4WQFLsBRUS7AVVFtYvQAHAEAAAQAHAAf/"
        "wDgRNzhZQI5IAmoCewJ/AoYCgAKRAqQCCAYABgEJAwkEFQAVARoDGgQmACYB"
        "KQMpBCAINQA1AToDOgQwCEYARgFJA0kERgVIBkAIVgBWAVkDWQRQCGYAZgFp"
        "A2kEZwVoBmAIdQB0AXsDewR1BXoGhQCFAYkDiQSJBYYGlgCWAZcCmgOYBJgF"
        "lwaoBacGsAjACN8I/wg+XQBdEzMJATMBIz3DAV4BXsP+XPoEYPxUA6z7oAAA"
        "AQBWAAAGNQRgAAwCAUBJBVUGBQkKCQRVCgkDVQoLCgJVAQILCwoGEQcIBwUR"
        "BAUICAcCEQMCDAAMAREAAAxCCgUCAwYDAL8LCAwLCgkIBgUEAwIBCwcADRDU"
        "zBc5MQAvPOwyMhc5MEtTWAcQBe0HEAjtBxAI7QcQBe0HEAjtBxAF7QcF7QcQ"
        "CO1ZIgFLsApUS7ARVFtLsBJUW0uwE1RbS7ALVFtYvQAN/8AAAQANAA0AQDgR"
        "NzhZAUuwDFRLsA1UW0uwEFRbWL0ADQBAAAEADQAN/8A4ETc4WUD/BQIWAhYF"
        "Igo1CkkCSQVGCkAKWwJbBVUKUApuAm4FZgp5An8CeQV/BYcCmQKYBZQKvAK8"
        "Bc4CxwPPBR0FAgkDBgQLBQoICwkECwUMFQIZAxYEGgUbCBsJFAsVDCUAJQEj"
        "AicDIQQlBSIGIgclCCcJJAohCyMMOQM2BDYIOQwwDkYCSANGBEAEQgVABkAH"
        "QAhECUQKRAtADkAOVgBWAVYCUARRBVIGUgdQCFMJVApVC2MAZAFlAmoDZQRq"
        "BWoGagduCWELZwxvDnUAdQF5An0DeAR9BXoGfwZ6B38HeAh5CX8Jewp2C30M"
        "hwKIBY8OlwCXAZQCkwOcBJsFmAaYB5kIQC+WDJ8OpgCmAaQCpAOrBKsFqQap"
        "B6sIpAyvDrUCsQO9BLsFuAm/DsQCwwPMBMoFeV0AXRMzGwEzGwEzASMLASNW"
        "uObl2ebluP7b2fHy2QRg/JYDavyWA2r7oAOW/GoAAQA7AAAEeQRgAAsBWkBG"
        "BREGBwYEEQMEBwcGBBEFBAECAQMRAgIBCxEAAQAKEQkKAQEAChELCgcIBwkR"
        "CAgHQgoHBAEECAC/BQIKBwQBBAgAAggGDBDUxNTEERc5MQAvPOwyFzkwS1NY"
        "BxAF7QcQCO0HEAjtBxAF7QcQBe0HEAjtBxAI7QcQBe1ZIgFLsApUS7APVFtL"
        "sBBUW0uwEVRbWL0ADP/AAAEADAAMAEA4ETc4WQFLsBRUWL0ADABAAAEADAAM"
        "/8A4ETc4WUCYCgQEChoEFQomCj0EMQpVBFcHWApmCnYBegR2B3QKjQSCCpkE"
        "nwSXB5IKkAqmAakErwSlB6MKoAocCgMEBQUJCgsaAxUFFQkaCykDJgUlCSoL"
        "IA06ATkDNwU0BzYJOQswDUkDRgVFCUoLQA1ZAFYBWQJZA1cFVgZZB1YIVglZ"
        "C1ANbw14AX8NmwGUB6sBpAewDc8N3w3/DS9dAF0JAiMJASMJATMJAQRk/msB"
        "qtn+uv662QGz/nLZASkBKQRg/d/9wQG4/kgCSgIW/nEBjwABAD3+VgR/BGAA"
        "DwGiQEMHCAIJEQAPChELCgAADw4RDwAPDREMDQAADw0RDg0KCwoMEQsLCkIN"
        "CwkQAAsFhwO9Dgu8EA4NDAoJBgMACA8EDwsQENTExBEXOTEAEOQy9OwRORE5"
        "EjkwS1NYBxAF7QcQCO0HEAjtBxAF7QcQCO0HBe0XMlkiAUuwClRLsAhUW1i9"
        "ABD/wAABABAAEABAOBE3OFkBS7AUVFi9ABAAQAABABAAEP/AOBE3OFlA8AYA"
        "BQgGCQMNFgoXDRANIw01DUkKTwpODVoJWgpqCocNgA2TDRIKAAoJBgsFDAsO"
        "Cw8XARUCEAQQBRcKFAsUDBoOGg8nACQBJAIgBCAFKQgoCSUKJAskDCcNKg4q"
        "DyARNwA1ATUCMAQwBTgKNgs2DDgNOQ45DzARQQBAAUACQANABEAFQAZAB0AI"
        "QglFCkcNSQ5JD0ARVABRAVECVQNQBFAFVgZVB1YIVwlXClULVQxZDlkPUBFm"
        "AWYCaAppDmkPYBF7CHgOeA+JAIoJhQuFDIkNiQ6JD5kJlQuVDJoOmg+kC6QM"
        "qw6rD7ARzxHfEf8RZV0AXQUOASsBNTMyNj8BATMJATMCk06UfJNsTFQzIf47"
        "wwFeAV7DaMh6mkiGVARO/JQDbAABAFgAAAPbBGAACQC0QBoIEQIDAgMRBwgH"
        "QgipALwDqQUIAwEABAEGChDcxDLEETk5MQAv7PTsMEtTWAcQBe0HEAXtWSIB"
        "S7ALVEuwDFRbWL0ACgBAAAEACgAK/8A4ETc4WQFLsBNUWL0ACv/AAAEACgAK"
        "AEA4ETc4WUBCBQIWAiYCRwJJBwULCA8LGAMbCCsIIAs2AzkIMAtAAUACRQNA"
        "BEAFQwhXA1kIXwtgAWACZgNgBGAFYgh/C4ALrwsbXQBdEyEVASEVITUBIXED"
        "av1MArT8fQK0/WUEYKj825OoAyUAAQEA/rIEFwYUACQAgkA0GQ8VCwYlCRoQ"
        "FR0LBSAhAwALqQkAqQHACRWpE7ElDAkKBSQWGQAdCgUTAhQAIBlDCg8FJRDU"
        "PMT8PMQyOTkREjkREjk5ERI5OTEAEPzsxPTsEO4SFzkSORE5ORESORESOTkw"
        "AUuwDFRYvQAl/8AAAQAlACUAQDgRNzhZsgAmAV0FFSMiJj0BNCYrATUzMjY9"
        "ATQ2OwEVIyIGHQEUBgceAR0BFBYzBBc++alsjj09j2up+T5EjVZbbm9aVo2+"
        "kJTd75d0j3OV8N2Tj1iN+J2OGRuOnPiNWAABAQT+HQGuBh0AAwAStwEAsQQA"
        "BQIEENTsMQAQ/MwwAREjEQGuqgYd+AAIAAAAAQEA/rIEFwYUACQAnkA2HyUb"
        "FgwPCBsLFRkPBAUgAwAZqRsAqSPAGw+pEbElHBkaFQ8BBAAIGhUjEgQAGh8V"
        "QxAACwQlENQ8xDL8PMQREjk5ERI5ERI5ORESOTkxABD87MT07BDuEhc5ERI5"
        "ORE5ETk5ERI5MAFLsApUWL0AJQBAAAEAJQAl/8A4ETc4WQFLsA5UWL0AJf/A"
        "AAEAJQAlAEA4ETc4WbIAJgFdBTMyNj0BNDY3LgE9ATQmKwE1MzIWHQEUFjsB"
        "FSMiBh0BFAYrAQEARoxVWm9vWlWMRj/5p2yOPj6ObKf5P75Wj/icjhsZjp34"
        "jlePk93wlXOPdJfv3ZQAAQDZAdMF2wMxAB0AI0AQARAbDAATBJwbE5wMHgAP"
        "HhDUxDEAENT81OwQwBESOTkwARUGBiMiJyYnJicmIyIGBzU2NjMyFxYXFhcW"
        "MzI2Bdtps2FukgsFBw+bXlisYmmzYW6TCgUIDpteVqkDMbJPRDsEAgMFPk1T"
        "sk9FPAQCAwU+TAD//wAQAAAFaAdOAicAJAAAAAAABwEDALwBdQADABAAAAVo"
        "B20ACwAOACEAy0BUDBENDBscGw4RHBseERwbHREcHBsNESEPIQwRDgwPDyEg"
        "EQ8hHxEhDyFCDBsPDQkDwRUJHpUNCY4gHB4dHBggHyENEgYOGAwGGwBWGBwP"
        "BlYSHCEiENTE1OwyENTuMhE5ETkREjkROTkREjk5MQAvPObW7hDU7hESOTk5"
        "MEtTWAcQBe0HBe0HEAjtBxAF7QcQBe0HBe0HBe0HEAjtWSKyICMBAV1AIBoM"
        "cwybDAMHDwgbUCNmDWkOdQ17DnkceR12IHYhgCMMXQBdATQmIyIGFRQWMzI2";

    const char VERA_TTF_PART_2[] =
        "AwEhAS4BNTQ2MzIWFRQGBwEjAyEDIwNUWT9AV1g/P1mY/vACIf5YPT6fc3Kh"
        "PzwCFNKI/V+I1QZaP1lXQT9YWP7z/RkDTilzSXOgoXJGdin6iwF//oEA//8A"
        "c/51BScF8AInACYAAAAAAAcA3QEtAAD//wDJAAAEiwdrAicAKAAAAAAABwEE"
        "AJ4Bdf//AMkAAAUzB14CJwAxAAAAAAAHAQUA/gF1//8Ac//jBdkHTgInADIA"
        "AAAAAAcBAwEnAXX//wCy/+MFKQdOAicAOAAAAAAABwEDAO4Bdf//AHv/4wQt"
        "BmYCJwBEAAAAAAAHAI0AUgAA//8Ae//jBC0GZgInAEQAAAAAAAcAQwBSAAD/"
        "/wB7/+MELQZmAicARAAAAAAABwDXAFIAAP//AHv/4wQtBhACJwBEAAAAAAAH"
        "AI4AUgAA//8Ae//jBC0GNwInAEQAAAAAAAcA2ABSAAD//wB7/+MELQcGAicA"
        "RAAAAAAABwDcAFIAAP//AHH+dQPnBHsCJwBGAAAAAAAHAN0AjwAA//8Acf/j"
        "BH8GZgInAEgAAAAAAAcAjQCLAAD//wBx/+MEfwZmAicASAAAAAAABwBDAIsA"
        "AP//AHH/4wR/BmYCJwBIAAAAAAAHANcAiwAA//8Acf/jBH8GEAInAEgAAAAA"
        "AAcAjgCLAAD//wCQAAACbwZmAicA1gAAAAAABwCN/x0AAP///8cAAAGmBmYC"
        "JwDWAAAAAAAHAEP/HQAA////3gAAAlwGZgInANYAAAAAAAcA1/8dAAD////0"
        "AAACRgYQAicA1gAAAAAABwCO/x0AAP//ALoAAARkBjcCJwBRAAAAAAAHANgA"
        "mAAA//8Acf/jBHUGZgInAFIAAAAAAAcAjQBzAAD//wBx/+MEdQZmAicAUgAA"
        "AAAABwBDAHMAAP//AHH/4wR1BmYCJwBSAAAAAAAHANcAcwAA//8Acf/jBHUG"
        "EAInAFIAAAAAAAcAjgBzAAD//wBx/+MEdQY3AicAUgAAAAAABwDYAHMAAP//"
        "AK7/4wRYBmYCJwBYAAAAAAAHAI0AewAA//8Arv/jBFgGZgInAFgAAAAAAAcA"
        "QwB7AAD//wCu/+MEWAZmAicAWAAAAAAABwDXAHsAAP//AK7/4wRYBhACJwBY"
        "AAAAAAAHAI4AewAAAAEAOf87A8cF1QALACdAFAgEuQoCAIEGwgwDWQUBVwlZ"
        "BwAMENQ87Pw87DEAEOT01DzsMjABMxEhFSERIxEhNSEBqLABb/6RsP6RAW8F"
        "1f5cmfujBF2ZAAACAMMDdQM9BfAACwAaACBAEQbDFcQAwwyRGwlaElsDWhgb"
        "ENzs/OwxABD07PzsMAEiBhUUFjMyNjU0JicyFhcWFhUUBiMiJjU0NgIAUG5u"
        "UFBub09AdisuLrmGh7S4BW9vUE9tbU9PcIExLi1yQoS3tIeGugAAAgCs/scE"
        "IwWYAAYAIQBRQCsTFhQADwwBCweGCIgLEIYPiAy5FBYLuR0fHLgWjCIcFQAJ"
        "HhMLDwcEEhkiENzs1DzUPDzsMjIxABDk9DzE7BDE/vTuEPXuEjkREjkREjkw"
        "JREGBhUUFgEVJiYnAzY2NxUGBgcRIxEmABEQADcRMxMWFgKmk6SkAhBKiEQB"
        "RolIQYlNZvH+9wEJ8WYBSYmDA1gS4ri54gOhrCkqA/ygBSonqh4jB/7kASAU"
        "ATMBAQECATIWAR/+4QQhAAEAgQAABGIF8AAbAGBAIQcWCAGGABIKqRQIDASg"
        "AJQZkRAMoA4ADQkLBxwTDxURHBDcPMzM/DzE1MQxAC/sMvTk7BDUPO4yEO4R"
        "OTkwAUuwDFRYvQAc/8AAAQAcABwAQDgRNzhZtDYBNgICAF0BFS4BIyIGHQEh"
        "FSERIRUhNTMRIzUzNRA2MzIWBE5MiD2UdAGH/nkCLfwf7MfH1ug9lwW0tikp"
        "m9TXj/4vqqoB0Y/uAQXzHwAAAgBc/z0DogXwAAsAPgCRQDwvMCoGABcdMDYE"
        "DSeKJg2KDCrGJsUjEMYMxTyRIz8vBgAXMAQTHS0JNgMTVzktVyAJVwwiGjkm"
        "IgNXMz8Q3OzkxNTk7NTsEO4RORESOREXOTkxABDE9OTsEObuEO4Q7hEXOTk5"
        "ERI5MAFLsApUS7ALVFtLsAxUW0uwDlRbWL0APwBAAAEAPwA//8A4ETc4WQEO"
        "ARUUFhc+ATU0JhMVLgEjIgYVFBcWFx4BFRQGBx4BFRQGIyImJzUeATMyNjU0"
        "LwEuATU0NjcuATU0NjMyFgF7Pz6L+j8+j8xTjzhhbM4aDtODXF0+OcytSZpY"
        "V5Q6ZnHdGdaAXVs7O8imSZkDqC5aLkyFhy1bLkuIApOkJydQR1pzDwh3mmVa"
        "jDU0bUCOqB0dpCcnVExmew54mWZbjzEscEWCnx0AAAEBMwHRA4UEIQALABK3"
        "CccDDAZcAAwQ1OwxABDU7DABNDYzMhYVFAYjIiYBM61+fKusfX2sAvp8q6t8"
        "faysAAABAJ7/OwQ5BdUADQAlQBIIAgTBAIEGAg4AB10FA10BCw4Q1NT83Ow5"
        "MQAQxDL07BE5MAEhESMRIxEjESYmNTQkAnkBwI2+jtfrAQQF1flmBh/54QNO"
        "Ed24vugAAAEAuv/jBKwGFAAvAJpAMC0nIQwEBg0gAAQqFoYXGrkTKrkDlxOM"
        "LgwJDR0gIScJCCQnCAYdCCQQFi0IEABGMBD8xPzMEMbu1O4Q7hE5ORI5Ejkx"
        "AC/k/u4Q/tXuEhc5FzkwQEAPBQ8GDwcPJw8oigyKDQcKBgoHCgsKDAoNCh8N"
        "IAohDCIEJhkNGR8ZIDogOiFNH00gSSFJImofaiClBqUHpiAYXQFdEzQ2MzIW"
        "Fw4BFRQWHwEeARUUBiMiJic1HgEzMjY1NCYvAS4BNTQ2Ny4BIyIGFREjuu/a"
        "0NsDl6g6QTmmYOHTQIhJUIxBdHg7ZVxgV6eXCINxgoi7BHHI2+jgCHNgL1Eq"
        "JWqOZKy3GRikHh1fWz9UPjc7h1t/rB1ncIuD+5MAAAQBGwAABuUFzQAXAC8A"
        "OABMAGBANkVCQz8yyUgwyTlKQ8oMOcoAyRjIDMkkSEUzMAQxQjw/OTZJMWBL"
        "NmBDPF4SCR5LXgYJHl8qTRDc5PzsEP79xO4Q7jIROTkSORIXOTEAL+72/u0Q"
        "7TIQ7tbuORI5OTABIgYHBgYVFBYXFhYzMjY3NjY1NCYnJiYnMgQXFhIVFAIH"
        "BgQjIiQnJgI1NBI3NiQTIxEzMjY1NCYnMhYVFAYHFhYXFyMnJiYjIxEjEQQA"
        "g+JeXmBgXl7ig4TjXl1dXlxe44SYAQdtbWxsbW3++ZiY/vltbWxsbW0BB317"
        "e25XWGawrmlgGEMuiayBO0k2QpsFZl5eXuWCgeNeXl9fXl3ig4XjXV5eZ25t"
        "bf76mpj++21tbm5tbQEFmJoBBm1tbv5i/uw+S0w/Z3d5VnARCE1J39FgM/6c"
        "A0QAAwEbAAAG5QXNABcALwBJAENAJj3LPjrMQcokMcswNMxHyhjJAMgkyQw3"
        "YUQ9MF4qCQZEXh4JBhJKENzM/OwQ/u0yEO4xAC/u9v797tbuEP3u1u4wATIE"
        "FxYSFRQCBwYEIyIkJyYCNTQSNzYkFyIGBwYGFRQWFxYWMzI2NzY2NTQmJyYm"
        "FxUmJiMiBhUUFjMyNjcVBgYjIiY1NDYzMhYEAJgBB21tbGxtbf75mJj++W1t"
        "bGxtbQEHmIPiXl5gYF5e4oOE415dXV5cXuOnQoJClaerm0B6QkOJRtj7+9hJ"
        "iAXNbm1t/vqamP77bW1ubm1tAQWYmgEGbW1uZ15eXuWCgeNeXl9fXl3ig4Xj"
        "XV5e9YEhIK+dn64fIn8dHPTQ0fIcAAACAScDkwZGBdUADAAUAD5AIQEGBxAK"
        "BBIOCQMGyQ0CAIEVAQkFYgMJYgsNYw9iE2MRFRDU5Pzk1OzU7BE5MQAQ9Dw8"
        "7Bcy1Dw8xBE5MAETEzMRIxEDIwMRIxEjFSMRIxEjNQRKrqSqccM3y3Jxy3LJ"
        "BdX/AAEA/b4B5P7RAS/+HAJCXv4cAeReAAABAXME7gNSBmYAAwAxQAkCtACz"
        "BANEAQQQ1OwxABD07DAAS7AJVEuwDlRbWL0ABP/AAAEABAAEAEA4ETc4WQEz"
        "ASMCi8f+upkGZv6IAAACANcFRgMpBhAAAwAHAJJADgYCzgQAzQgBZAAFZAQI"
        "ENz81OwxABD8POwyMABLsApUS7ANVFtYvQAIAEAAAQAIAAj/wDgRNzhZAUuw"
        "DFRLsA1UW0uwDlRbS7AXVFtYvQAI/8AAAQAIAAgAQDgRNzhZAUuwD1RLsBlU"
        "W1i9AAgAQAABAAgACP/AOBE3OFlAEWABYAJgBWAGcAFwAnAFcAYIXQEzFSMl"
        "MxUjAl7Ly/55y8sGEMrKygAAAQDZACcF2wTdABMAPkAiDQwKAwLPBACcBgzP"
        "DgqcEgYQCBQSDg0MCAQDAggJBQ8AFBDcPMQyFzkxABDUPMwy/DzsEP487DkR"
        "EjkwEyEBFwchFSEHIRUhASc3ITUhNyHZAwQBAH2uAS/+SMMCe/z6/v59rv7V"
        "AbbD/YcDogE7ZtWo8Kr+x2bTqvAAAgAIAAAHSAXVAA8AEwCHQDkREQ4PDhAR"
        "Dw8ODREPDgwRDg8OQgWVAwuVEQGVEJUAgREHlQOtDQkREA8NDAUOCgAECAYC"
        "HBIKDhQQ1NQ87DLUxMQREhc5MQAvPOzsxPTs7BDuEO4wS1NYBxAF7QcF7QcQ"
        "Be0HEAXtWSKygBUBAV1AE2cRdxB3EYYMhRCWEZAVoBW/FQldARUhESEVIREh"
        "FSERIQMjARcBIREHNf0bAsf9OQL4/D398KDNAnGL/rYBywXVqv5Gqv3jqgF/"
        "/oEF1Z788AMQAAMAZv+6BeUGFwAJABMAKwCeQDwdHxoNKywTCgEABA0pJiAU"
        "DQQqJh4aBJUmDZUakSaMLCssKhQXECAeIxMKAQAEHSkQBx8HGSMzEBkXECwQ"
        "/Oz87MAREjk5FzkSOTkREjk5ETkxABDk9OwQ7hDAEMAREjk5EjkSFzkSORES"
        "OTkwQCpXAFoVVxlVIWoVZSF7FXYcdSEJRhNZAFYTagBkE2Qcaih8AHMTdhx6"
        "KAtdAV0JAR4BMzIAETQmJy4BIyIAERQWFwcmAjUQACEyFhc3FwcWEhUQACEi"
        "JicHJwS2/TM+oV/cAQEneT2hX9z+/Scnhk5PAXkBO4LdV6Jmqk5Q/oj+xoDd"
        "W6JnBFj8skBDAUgBGnC4uEBD/rj+5XC8RJ5mAQigAWIBpU1Lv1nGZ/72nv6f"
        "/ltLS79YAAMA3QDdBc8D7gALABcALwD/QB0tGxUJIQMAJBgEFQ8nIRUbDyEw"
        "DAAkGBIGKhIeMBDUxNTEETk5OTkxABDUxNTEEMAREhc5EjkREjkwQL4FAgUD"
        "BQQABQAGAAcFCAUJBQoKEA8RDxIPEwoUFQIVAxUEEAUQBhAHFQgVCRUKGg4a"
        "DxoQHxEfEh8TGhQaFRoWJAIkAyQEIAUgBiAHJAgkCSQKKg4qDyoQLxEvEi8T"
        "KhQqFSoWNQI1AzUEMAUwBjAHNQg1CTUKOg46DzoQPxE/Ej8TOhQ6FToWRQJF"
        "A0UEQAVABkAHRQhFCUUKSg5KD0oQTxFPEk8TShRKFUoWVrQfsCCwIbAisCaw"
        "J7AotCkIXQFdAR4BMzI2NTQmIyIGBy4BIyIGFRQWMzI2Fw4BIyImNTQ2MzIW"
        "Fz4BMzIWFRQGIyImA5MxhlRlgHZZUoXEMYVVZn92WVKGkEadXoi6p4ZfmUhE"
        "nmGGvKeGXpUCL1hah2llhoc3WFiEamWGiBaHf9+mr9h+ioqD4aev1ncAAAIA"
        "2QAABdsFBAALAA8ALkAYBdAHA5wA0AkBDJwODQIVBAAXDAgVCgYQENQ87DL8"
        "POwyMQAv7NQ87Pw87DABESEVIREjESE1IREBIRUhA64CLf3TqP3TAi390wUC"
        "+v4FBP59qv59AYOqAYP7pqoAAgDZAAAF2wSoAAYACgBUQC4CnAMEAwGcAAEE"
        "BAMBnAIBBQYFAJwGBUIFBAIBAAUD0QanB5wJAQgCACQHBCMLEPw87DIyOTEA"
        "L+z07Bc5MEtTWAcE7QcQCO0HEAjtBxAE7VkiCQIVATUBASEVIQXb/EADwPr+"
        "BQL6/gUC+v4D+P7r/u6yAXCqAW/8AqoAAAIA2QAABdsEqAAGAAoAVkAvBpwA"
        "BgMEAwWcBAQDAJwBAgEGnAUGAgIBQgYFAwIABQTRAacHnAgGBwIkCQQAIwsQ"
        "/Dw87DI5MQAv7PTsFzkwS1NYBxAI7QcQBO0HEATtBxAI7VkiEzUBFQE1AQEV"
        "ITXZBQL6/gPBAUH6/gP4sP6Rqv6QsgES/ceqqgAAAQBSAAAEwwXVABgAxkBG"
        "EAIRFhEPAg4PFhYRDwIQDwgNCA4CDQ0IQg8LCQQA0xcGEgvTFAkQDYECDAkO"
        "AwUWDwMVEhADABFmEwBlARwNZgoFZQcDGRDUPOwy7PzsMuwSFzkSOTkRFzkx"
        "AC/kMtQ87DLUPOwyERI5MEtTWAcQBe0HEAjtBxAI7QcQBe1ZIgFLsAxUWL0A"
        "Gf/AAAEAGQAZAEA4ETc4WUAohg+QD6YPoA+1DwUnDCcNJw4pECgRKBI3DjkQ"
        "hwyIEqYNpQ6qEKkRDl0AXQEhESMRITUhNSchNSEBMwkBMwEhFSEHFSEEjf5j"
        "yf5gAaBU/rQBCP7DvgF7AXm//sIBCP61VAGfAcf+OQHHezObewJK/UQCvP22"
        "e5szAAABAK7+VgTlBGAAIABNQCUTGR8DFgYDCQwDARIPBoccFowKAbwAvSEZ"
        "CRIJCAtOHwIIAEYhEPzsMvTsxBI5MQAQ5OQy9Dzs3MQRFzkREhc5MLYfImAi"
        "zyIDAV0TETMRFBYzMjY1ETMRFBYzMjY3FQ4BIyImJw4BIyImJxGuuIqHlJW4"
        "IyUJIBwpSSNFUg8ykWJmjyr+VgYK/UiRlKioAo38ojw5CwyUFxZOUE9PTk79"
        "1wAAAgBo/+cDwQUtAB0AKQBiQBkAJyEJGwYnFQYPIRsPFdUqDCQDAB4SJBgq"
        "ENTM3Mw5ORE5MQAQ5MzczBDOEM4REjkREjkwAUuwDFRLsAtUW0uwDlRbS7AQ"
        "VFtLsBRUW1i9ACoAQAABACoAKv/AOBE3OFkBPgE1NCYjIgYjIiY1NDYzMhIR"
        "EAAjIiY1NBIzMhYHNCYjIgIVFBYzMhIC9A8PSUg3kCQkMJBltNb+39WYy92i"
        "ZYILV09tjVZQbY0CbVejS4GDdCwfPmL+yv75/rH+RtijxgEBW+B0ff7+z3R7"
        "AQQAAAEAGf53BTsFwQALAF1AFAoEDAIFBwIABwwKBQQDAQAGBggMENTEFzkx"
        "ABDE1MwQzhESOTkwQDBRA1YFUAVaCnMDcAN2BHUFcAV6CoADgAUMWgl/An8D"
        "cAVwBnsJdAuPAo8DgAWABgtdAV0TIRUhCQEhFSE1CQE3BOr8QQKg/UoD7/re"
        "AtX9SQXBwf0z/QTAlQMhAuMAAAEAnP53BXEFwQAHAB5ADwYC1wTWAK8IA2cB"
        "BWcACBDU7NTsMQAQ/OzsMjATIREjESERI5wE1fD9Cu8Fwfi2Bn35gwAAAf/h"
        "//AEqgQvACMAy0AxCwIVHx4DAAjaDxoWANki2A/VGAweGxoZGBcGJBIBAAsC"
        "BCMWFQUiHxIMIxJoBSMfJBDU1NTsEjkREjkSOTkRFzkREhc5MQAvPOT07DIy"
        "EO4RFzk5OTABS7AKVFi9ACT/wAABACQAJABAOBE3OFlAVhgeGB8CCQAJAQ0C"
        "DQMPBA8FDwYPBw8IDwkPCg8LDwwPDQ8ODw8PEA8RDxIPEw4UDRUJFgsXCBgP"
        "GA0ZCBoJIxEAEQEWAhYDFxQWFREWFxccGBwZESMoXQBdASMDDgEVFBYzMjY3"
        "Bw4BIyImNTQ2NxMhAyMTIyIGByM+ATMhBIe2aQ8PLzcRLiUeHjcadnkVIlD+"
        "usK1wyk2PAmgHI+lA3kDkf4ZSlwWOjEFBY0ICGZkLpChAXj8bwORQEWmfQAA"
        "AQAv/o0D+gYOACUAJkAUINsAGg3bExrcB7EmCmkXah1pBCYQ3Oz87DEAEPzs"
        "3OQQ3uQwATITNjcSEjMyFhUUBiMiJicmJiMiAwYHAgIjIiY1NDYzMhYXFhYB"
        "N2oOAgEMvspQZEA3KjgMBgkQaw4EBBG9xE9lRD0hMA8KCv76ArBsOQIDAbxU"
        "QTY/JiMPSP2VwW7+If5iU0E4Px0cElMAAAMAcwHVAzsF8AADAB4AKQBfQDMo"
        "ByUEHxIYEALjAB/dEADhJd0FChnfGN4V3QrgHJEqABgNHxAiBgIBKBEGawRs"
        "GCJrDSoQ3OzM/OwyMsDAERI5ORESOTEAEPTk/PTsEMTu7dbuEO4REjkSORE5"
        "OTATIRUhAREjNQYGIyImNTQ2MzM1NCYjIgYHNTY2MzIWBSIGFRQWMzI2NTWL"
        "ArD9UAKulSyQXYCYv7y2dXU+iERJkUW3s/7soX5iUmiCAlB7Arj+QHA/RIdx"
        "h4oEW1siIn8cHLDwQ09ATZByHQADAGAB1QNkBfAAAwAPABsALkAZAuMA4Rbd"
        "CuAQ3QSRHAATDQEZawdsE2sNHBDc7PzsORESOTEAEPTs9Oz87DATIRUhATIW"
        "FRQGIyImNTQ2FyIGFRQWMzI2NTQmiwKw/VABWLPOzrOz0NCzaX5/aGl9fAJQ"
        "ewQb3b+/29y+v91zoYiFoKCFiaAAAQBOAAAFzwXnAB8AQEAiCeUZkRIPAwMA"
        "5RABESAWEw8MHwYCAQACbQYcHA9tDBwWIBDU7OzU7OzAwBESORESORESOTEA"
        "LzzsFzL07DAlFSE1NhI1NAAjIgAVFBIXFSE1ISYCNRAAISAAERQCBwXP/aix"
        "xv742Nj+98ey/agBP56RAX8BMQEvAYGOobKysmEBTMrwASL+3e/K/rRhsrKL"
        "ASq4AT4Biv53/svC/tiNAAADAHv/4wdvBHsABgAzAD4BA0BDJy0lPQ4NADSp"
        "JRaGFYgSAKkOOhK5HBkuhi26KgO5DrsHMQq4HxmMJT80NyYGDwAlNxwHJg8V"
        "AAgNPSYIDy03CCJFPxD87MzU/DzU7MQREjk5ETkREjkREjkxABDE5DL0PMTk"
        "/Dz07BDE7jIQ7hD07hDuETkRORESOTBAgTArMCwwLTAuMC8wMEArQCxALUAu"
        "QC9AMFArUCxQLVAuUC9QMIUrhTCAQJBAoECwQMBA0EDgQOBA8EAdPwA/Bj8N"
        "Pw4/DwUwLDAtMC4wL0AsQC1ALkAvUCxQLVAuUC9vAG8Gbw1vDm8PYCxgLWAu"
        "YC9wLHAtcC5wL4AsgC2ALoAvHV1xAV0BLgEjIgYHAz4BMzIAHQEhHgEzMjY3"
        "FQ4BIyImJw4BIyImNTQ2MyE1NCYjIgYHNT4BMzIWAyIGFRQWMzI2PQEGtgGl"
        "iZm5DkRK1ITiAQj8sgzMt2jIZGTQaqf4TUnYj73S/fsBAqeXYLZUZb5ajtXv"
        "36yBb5m5ApSXtK6eATBaXv7d+lq/yDU1rioseXd4eLuovcASf4suLqonJ2D+"
        "GGZ7YnPZtCkAAwBI/6IEnAS8AAkAEwArAORAPCssJh8dGhMKAQAEDSkmIBQN"
        "BComHhoEuSYNuRq4JowsKywqFBcQIB4jEwoBAAQQBx8dBxIjUSkQEhdFLBD8"
        "7DL07DLAERIXORI5ORESOTkROTEAEOT07BDuEMAQwBESOTkSORIXORE5ORES"
        "OTBAcCgBPy1ZFFYcVR1WIGoVZiF/AHsEfwV/Bn8Hfwh/CX8Kfwt/DHsNehV7"
        "Gn8bfxx/HX8efx9/IHshfyJ/I38kfyV7JpsZlSWoGaAt8C0mWQBWE1UdWihp"
        "AGYTZRxqKHoAdBN2HHooiR6VGJokohitJBFdAV0JAR4BMzI2NTQmJy4BIyIG"
        "FRQWFwcuATUQADMyFhc3FwceARUQACMiJicHJwOJ/hkpZ0GTrBRcKmc+l6kT"
        "FH02NgER8V2fQ4tfkjU2/u7wYKE/i2ADIf2wKijoyE91mikp69NIbi6XTcV3"
        "ARQBODM0qE+zTcZ4/u3+xzQzqE4AAgCP/+MDrAXVACAAJACGQC8gGgUCBAYZ"
        "ABCGD4gMACGDIwyVE4wjgSUGIhkWCQUBABoiCQAcASIcISYPCRwWJRDc7NT8"
        "7NTsERI5ERI5ERI5EjkxABDk9OwQ/s0Q9O4SOTkXOTABS7AQVEuwElRbS7AT"
        "VFtYvQAl/8AAAQAlACUAQDgRNzhZQAt0BHQFdAZ0B3YcBV0BMxUUBg8BDgEV"
        "FBYzMjY3FQ4BIyImNTQ2PwE+ATc+ATUTIzUzAfS+N1paOjODbU60YF7AZ7jg"
        "SVlYMCYIBwbEysoERJxlgldYNV4xWW5GQ7w5OMKfTIlWVi81GRU8NgEO/gAC"
        "ATUAAAIABdUAAwAJAGJADwcAgwKBBAgHBAADBQEAChD8POwyOTkxAC/0/Mww"
        "AUuwC1RYvQAKAEAAAQAKAAr/wDgRNzhZAUuwD1RLsBBUW0uwE1RbWL0ACv/A"
        "AAEACgAKAEA4ETc4WbYACyALUAsDXQEjNTMRIxETMxMCAMvLyxWiFATX/vor"
        "Ao8BZf6bAAABANkBHwXbA14ABQAXQAoEnAIABgMXAQAGENzU7DEAENTE7DAT"
        "IREjESHZBQKo+6YDXv3BAZUAAAEAPf/XBRkGfQAKACpAGAoJCAcGBQsCBAIA"
        "CwoJBwYFBAMACAEICxDUzBc5MQAQ1MzEERIXOTABMxUjASMBByclAQRcvXP9"
        "rkL+wX0ZARsBAAZ9YPm6A3MtUGL9OwABAB/+VgUCBhQAIwCKQEAODQIPDBEZ"
        "HhkLCgkIBAcRHh4ZQhkMEwoHHgEaCqkIE4oS5hapDwGKAOYEqSGXDxwIJB4d"
        "GxoZDAsJCAcKABIkENTMFzkxABDEMsT87PTsEO727hDuMhI5ORESOTkwS1NY"
        "BxAF7RcyBxAF7REXOVkiAUuwDFRYvQAk/8AAAQAkACQAQDgRNzhZARUuASMi"
        "BgcDIRUhAwIGIyImJzUeATMyNjcTIzUhEz4BMzIWBQImUCxgchk8AR/+w386"
        "vLo6ZC80YS9hbSKJ+AEXPyTGlzVkBfCkHRx6hP7Jj/2F/uPTFRamISGJpgKt"
        "jwFKt8MSAAIA2QEQBdsD9AAdADsAP0AfLh85KgAtIhMBEBsMHiqcMTmcIgSc"
        "GwycEzweAC0PPBDUPMQyMQAQ1OzU7Nz81OzAERI5ORESOTkREjk5MAEVBgYj"
        "IicmJyYnJiMiBgc1NjYzMhcWFxYXFjMyNhMVBgYjIicmJyYnJiMiBgc1NjYz"
        "MhcWFxYXFjMyNgXbabNhbpIKBwYPm15YrGJps2FukwsFBg+bXlapZ2mzYW6S"
        "CgcGD5teWKxiabNhbpMKBQcPm15WqQJvs05FOwQDAgY9TFSzTkU7BQICBj1L"
        "AdqyT0U7BAMCBj1MU7JORTsEAgMGPUsAAv/6AAAFYAXBAAIABgA4QA8AAwED"
        "BQQDAgEABQcFBgcQ1MwRFzkxAC/EzBE5MEAUYwFtAnABeAJ/AnkFdgYHbgB/"
        "AAJdAV0JASEBMwEhAqz+XgNE/e/gAkP6mgTu+8QFD/o/AAIAngCNBCUEIwAG"
        "AA0AhkBJA+gEBQQC6AECBQUEAugDAgYABgHoAAAGCugLDAsJ6AgJDAwLCegK"
        "CQ0HDQjoBwcNQgkCCwTnBwCmDgkMBQIHAwBvBQoHbwxuDhD8/DzU7DIRORES"
        "OTEAEPQ87DI5OTBLU1gHEATtBxAI7QcQCO0HEATtBxAE7QcQCO0HEAjtBxAE"
        "7VkiARUBARUBNRMVAQEVATUEJf7TAS3+KyP+0wEt/isEI7/+9P70vwGiUgGi"
        "v/70/vS/AaJSAAACAMEAjQRIBCMABgANAIZASQzoDQwJCgkL6AoKCQ3oBwgH"
        "DOgLDAgIBwXoBgUCAwIE6AMDAgboAAEABegEBQEBAEIMBQoD5wcApg4MCAEF"
        "AAhvCgcBbwMAcA4Q/Dz81DzsEjkREjkxABD0POwyOTkwS1NYBxAI7QcQBO0H"
        "EATtBxAI7QcQCO0HEATtBxAE7QcQCO1ZIhMBFQE1AQElARUBNQEBwQHV/isB"
        "Lf7TAbIB1f4rAS3+0wQj/l5S/l6/AQwBDL/+XlL+Xr8BDAEMAAMA7AAABxQA"
        "/gADAAcACwAjQBEIBACDCgYCBBkFABkBCRkIDBDU/NTs1OwxAC88POwyMjAl"
        "MxUjJTMVIyUzFSMDltTUAqnV1fqt1dX+/v7+/v4A//8AEAAABWgHawInACQA"
        "AAAAAAcBBgC8AXX//wAQAAAFaAdeAicAJAAAAAAABwEFALwBdf//AHP/4wXZ"
        "B14CJwAyAAAAAAAHAQUBJwF1AAIAcwAACAwF1QAQABkAO0AfBZUDEQGVAIEY"
        "B5UDrQkYEhAKFQYCHBEABAgVGQ0QGhD87NTExNTsMhI5OTk5MQAv7Owy9Owy"
        "EO4wARUhESEVIREhFSEgABEQACEXIyAAERAAITMH+v0aAsf9OQL4+9f+T/5B"
        "Ab8BsWeB/r/+wAFAAUGBBdWq/kaq/eOqAXwBcAFtAXyq/uH+4P7f/t8AAwBx"
        "/+MHwwR7AAYAJwAzAIRAMQcIABCGD4gMAKkILgy5FhMoA7kIuyIlH7gZE4w0"
        "BgAWIjEJDwAIB0sxEglRKxIcRTQQ/Oz0/PTsxBESOTkSOTEAEOQy9DzE5Owy"
        "EMTuMhDuEPTuERI5MEAlPzVfNXA1nzXPNdA18DUHPwA/Bj8HPwg/CQVvAG8G"
        "bwdvCG8JBV1xAV0BLgEjIgYHBRUhHgEzMjY3FQ4BIyImJw4BIyIAERAAMzIW"
        "Fz4BMzIAJSIGFRQWMzI2NTQmBwoCpImZuQ4DSPyyDMy3ashiZNBqoPJRR9GM"
        "8f7vARHxjNNCTuiP4gEI+rCUrKuVk6ysApSYs66eNVq+xzQ0riosbm1ubQE5"
        "ARMBFAE4b2xrcP7dh+fJyefoyMfpAAABAAAB6QQAAnkAAwAQtgKpAOkEAQAv"
        "xjEAEPzsMBEhFSEEAPwAAnmQAAABAAAB6QgAAnkAAwAPtQKpAAQBAC/MMQAQ"
        "1OwwESEVIQgA+AACeZAAAgCuA+kDbQXVAAUACwAnQBMGAJ4JA4EMCQoGGQcD"
        "BAcAGQEMENz8zNTMEP7UzjEAEPQ87DIwASM1EzMDBSM1EzMDAYHTpIFSAZrT"
        "pIFSA+mtAT/+wa2tAT/+wQACAK4D6QNtBdUABQALACdAEwkDngYAgQwJCgcZ"
        "BgEDBAEZAAwQ3OzUzBDc7tTOMQAQ9DzsMjABMxUDIxMlMxUDIxMBANOkgVIB"
        "mtOkgVIF1az+wAFArKz+wAFAAAEArgPpAdMF1QAFABhACwCeA4EGAwQAGQEG"
        "ENz81MwxABD07DABIzUTMwMBgdOkgVID6a0BP/7BAAABALID/gHXBdUABQAY"
        "QAsDngCBBgMEAXEABhDc7NTMMQAQ9OwwATMVAyMTAQTTpIFSBdWY/sEBPwAA"
        "AwDZAJYF2wRvAAMABwALAClAFADqAgbqBAIInAQKDAkFAXIEAAgMENzUPPw8"
        "xDEAENTE/MQQ7hDuMAEzFSMRMxUjASEVIQLf9vb29v36BQL6/gRv9v4S9QJB"
        "qgACAAb+IwPuBnUAAwAHACJAEQIGAAgEBggGBAMCAQAGBQcIENTMFzkxABDU"
        "zBE5EjkwCQcB+v5/AYEBgf5/AfT+DP4MBYH8z/zHAzkEJfvb+9MELf//AD3+"
        "VgR/BhACJwBcAAAAAAAHAI4AXgAA/////AAABOcHTgInADwAAAAAAAcBAwBz"
        "AXUAAf6J/+MCzQXwAAMAK0ATAA8BAgECDwMAA0ICjACRBAEDBBDUzDEAEOTk"
        "MEtTWAcQBe0HEAXtWSIBMwEjAi2g/FygBfD58wAAAgBeAFIEvASyACMALwCD"
        "QEkDCRsVBC0eACccAiEdDBItFAsKAxMPAR0tuRPrD+wnuR3rITAeDAASBCok"
        "FDAcFRsqHRMcGAkDJAsKAQMCJCgCcwZ0KigccxgwENzk7PTk7BIXORI5ORES"
        "OTkSOTkREjkREhc5MQAQ1OTs9OTsEMAREhc5Ejk5ERI5ORE5ORIXOTABNxcH"
        "FhYVFAYHFwcnBgYjIiYnByc3JiY1NDY3JzcXNjYzMhYTNCYjIgYVFBYzMjYD"
        "e89yziUkJijRcs87dD06eD3Pcc8lJSYmz3PPN3RAPHVcm3Jwnp1xcZwD4dFz"
        "zjt3Pj9zOc9xzygmJSXPc84+djpAdDjOc88nJST+fHCamnBynJ0AAAEAngCN"
        "AnMEIwAGAEdAJQPoBAUEAugBAgUFBALoAwIGAAYB6AAGQgIE5wCmBwIDAG8F"
        "bgcQ/OwyOTEAEPTsOTBLU1gHBO0HEAjtBxAI7QcQBO1ZIgEVAQEVATUCc/7T"
        "AS3+KwQjv/70/vS/AaJSAAABAMEAjQKWBCMABgBJQCYF6AYFAgMCBOgDAwIG"
        "6AABAAXoBAUBAQBCBQPnAKYHBQFvAwBwBxD8POw5MQAQ9Ow5MEtTWAcQCO0H"
        "EATtBxAE7QcQCO1ZIhMBFQE1AQHBAdX+KwEt/tMEI/5eUv5evwEMAQwAAAIA"
        "LwAABEoGFAAVABkAm0AoCxQYBwOpABCHDhi+FrEOlwkAvAUBEQ4PBBYCCAAP"
        "FAQICBcACgZMGhD8PMQyxPw8xBDuMhESOTkxAC885jLu/u4Q7hDuMhI5OTAB"
        "S7AKVFi9ABr/wAABABoAGgBAOBE3OFkBS7AOVFi9ABoAQAABABoAGv/AOBE3"
        "OFlAGhAWEBcQGBAZBDAbUBuAD4AQgBugG9Ab7xsIXQBdAREjESERIxEjNTM1"
        "NDY7ARUjIgYdAQEzFSMESrn+B7mwsK2zubBjTQH5ubkEYPugA9H8LwPRj063"
        "r5lQaGMBsukAAQAvAAAESgYUABUAhEAhCBMEDwupCQSHAJcRCbwNAgUACggD"
        "CAEKDAgIEAESDkwWEPw8xMT8PMQQ7hESOTkxAC885jL+7hDuMhI5OTABS7AK"
        "VFi9ABb/wAABABYAFgBAOBE3OFkBS7AOVFi9ABYAQAABABYAFv/AOBE3OFlA"
        "ETAXUBeACoALgBegF9AX7xcIXQEhESMRISIGHQEhFSERIxEjNTM1NDYCSgIA"
        "uf63Y00BL/7RubCwrgYU+ewFe1BoY4/8LwPRj067qwAAAQA5/zsDxwXVABMA"
        "PkAgEga5ABAIuQoEAAIOCgyBAsIUDwBZEQ0BVwkFWQsHAxQQ1Dw87DL8PDzs"
        "MjEAEOT0xDIQxDIQ7jIQ7jIwJSERIxEhNSERITUhETMRIRUhESEDx/6RsP6R"
        "AW/+kQFvsAFv/pEBb9/+XAGkmgIfmQGk/lyZ/eEAAQDbAkgBrgNGAAMAErcC"
        "gwAEARkABBDU7DEAENTsMBMzFSPb09MDRv4AAQCu/xIB0wD+AAUAGEALA54A"
        "gwYDBAEZAAYQ1OzUzDEAEPzsMCUzFQMjEwEA06SBUv6s/sABQAACAK7/EgNt"
        "AP4ABQALACdAEwkDngYAgwwDBAEZAAcJCgcZBgwQ3OzUzBDc7tTOMQAQ/Dzs"
        "MjAlMxUDIxMlMxUDIxMCmtOkgVL+ZtOkgVL+rP7AAUCsrP7AAUAAAAcAcf/j"
        "CkwF8AALABcAIwAnADMAPwBLAK5ARCQPJSYlJg8nJCdCQACSDC6SHo0okhhG"
        "BpI0DI06JhKMJBiRTCVJQycxK0MNPQkNDw4DDRUxDRs9DkkNFTcrDRsOIQtM"
        "EPzk7NTE7OQQ7hDu9u4Q7hESORESOTEAEOQy9Dw85DLsMhDu9u4Q7jIwS1NY"
        "BxAF7QcQBe1ZIgFLsBRUS7AJVFtLsAtUW0uwDFRbS7ANVFtLsA5UW1i9AEwA"
        "QAABAEwATP/AOBE3OFkBIgYVFBYzMjY1NCYnMhYVFAYjIiY1NDYBMhYVFAYj"
        "IiY1NDYhMwEjEyIGFRQWMzI2NTQmATIWFRQGIyImNTQ2FyIGFRQWMzI2NTQm"
        "CPRXZGRXVWNjVZ66u52gurv5dJ68u5+fuboEJaD8WqAfVmNiV1djZAOynrq7"
        "naC6u59XY2NXVWNjApGUhIKVlYKDlX/cu7vb27u82wLg27u92tu8utz58wWO"
        "lYKElJSEgZb9n9y7u9vbu7zbf5SEgpWVgoOVAP//ABAAAAVoB20CJwAkAAAA"
        "AAAHAQcAvAF1//8AyQAABIsHbQInACgAAAAAAAcBBwCeAXX//wAQAAAFaAdr"
        "AicAJAAAAAAABwEEALwBdf//AMkAAASLB04CJwAoAAAAAAAHAQMAngF1//8A"
        "yQAABIsHawInACgAAAAAAAcBBgCeAXX//wCiAAACHwdrAicALAAAAAAABwEE"
        "/y8Bdf////4AAAJgB20CJwAsAAAAAAAHAQf/LwF1//8ABgAAAlgHTgInACwA"
        "AAAAAAcBA/8vAXX//wA7AAABugdrAicALAAAAAAABwEG/y8Bdf//AHP/4wXZ"
        "B2sCJwAyAAAAAAAHAQQBJwF1//8Ac//jBdkHbQInADIAAAAAAAcBBwEnAXX/"
        "/wBz/+MF2QdrAicAMgAAAAAABwEGAScBdf//ALL/4wUpB2sCJwA4AAAAAAAH"
        "AQQA7gF1//8Asv/jBSkHbQInADgAAAAAAAcBBwDuAXX//wCy/+MFKQdrAicA"
        "OAAAAAAABwEGAO4BdQABAMEAAAF5BGAAAwAgtwC/AgEIAEYEEPzsMQAv7DBA"
        "CxAFQAVQBWAFcAUFAV0TMxEjwbi4BGD7oAAAAQDBBO4DPwZmAAYAN0AMBAUC"
        "tACzBwQCdQYHENzsOTEAEPTsMjkwAEuwCVRLsA5UW1i9AAf/wAABAAcABwBA"
        "OBE3OFkBMxMjJwcjAbaU9Yu0tIsGZv6I9fUAAQC2BR0DSgY3ABsAY0AkABIH"
        "DgsEARIHDwsEEsMZBwTDFQvtHA8BDgAHFVYWdwdWCHYcEPTs/OwROTk5OTEA"
        "EPw8/NQ87BESORESORESORESOTAAS7AJVEuwDFRbWL0AHP/AAAEAHAAcAEA4"
        "ETc4WQEnLgEjIgYHIz4BMzIWHwEeATMyNjczDgEjIiYB/DkWIQ0mJAJ9AmZb"
        "JkAlORYhDSYkAn0CZlsmQAVaNxQTSVKHkxwhNxQTSVKHkxwAAAEA1QViAysF"
        "9gADAC+3Au8A7gQBAAQQ1MwxABD87DAAS7AJVEuwDlRbWL0ABP/AAAEABAAE"
        "AEA4ETc4WRMhFSHVAlb9qgX2lAAAAQDHBSkDOQZIAA0AV0AOC/AEBwCzDgdW"
        "CAFWAA4Q3OzU7DEAEPQ81OwwAEuwCVRYvQAO/8AAAQAOAA4AQDgRNzhZAEuw"
        "D1RLsBBUW0uwEVRbWL0ADgBAAAEADgAO/8A4ETc4WRMzHgEzMjY3Mw4BIyIm"
        "x3YLYVdWYA12Cp6RkZ4GSEtLSkyPkJAAAQGaBUQCZgYQAAMALEAJAs4AzQQB"
        "ZAAEENTsMQAQ/OwwAEuwCVRYvQAE/8AAAQAEAAQAQDgRNzhZATMVIwGazMwG"
        "EMwAAAIA7gThAxIHBgALABcAX0ARA8EV8gnBD/EYAFYMeAZWEhgQ1Oz07DEA"
        "EPTs9OwwAEuwCVRLsAxUW1i9ABj/wAABABgAGABAOBE3OFkBS7AKVEuwC1Rb"
        "S7AMVFtYvQAY/8AAAQAYABgAQDgRNzhZATQmIyIGFRQWMzI2NxQGIyImNTQ2"
        "MzIWAphYQEFXV0FAWHqfc3Ofn3NznwX0P1hXQEFXWEBzoKBzc5+fAAABASP+"
        "dQLBAAAAEwAfQA4JBgoN8wYAEwAQJwMJFBDc1OzUzDEAL9T8xBI5MCEWFhUU"
        "BiMiJic1FhYzMjY1NCYnAlQ3Nnh2LlcrIkovOzwrLT5pMFlbDAyDEQ8wLh5X"
        "PQACAPAE7gOuBmYAAwAHAEJAEQYCtAQAswgEBwMABQEDBQcIENTc1MwRORES"
        "OTEAEPQ87DIwAEuwCVRLsA5UW1i9AAj/wAABAAgACABAOBE3OFkBMwMjAzMD"
        "IwL8sviHgarfiQZm/ogBeP6IAAABAUz+dQLBAAAAEwAgQA8LDgoH8w70AAEA"
        "CgQnERQQ1OzE1MwxAC/8/MQSOTAhMwYGFRQWMzI2NxUGBiMiJjU0NgG4dy0r"
        "NzYgPh8mRB56czU9WB8uLg8PhQoKV10waQABAMEE7gM/BmYABgA3QAwDALQE"
        "AbMHAwV1AQcQ3Ow5MQAQ9DzsOTAAS7AJVEuwDlRbWL0AB//AAAEABwAHAEA4"
        "ETc4WQEDMxc3MwMBtvWLtLSL9QTuAXj19f6IAAH/8gAABHUF1QANAD9AHgwL"
        "CgQDAgYABpUAgQgDBAELDgAEBQEcDAc6CQB5DhD0POzE/DzEERI5ERI5MQAv"
        "5OwRFzkwtDAPUA8CAV0TMxElFwERIRUhEQcnN9PLATlQ/ncC1/xelE3hBdX9"
        "mNtv/u7946oCO2pungAAAQACAAACSAYUAAsAXkAaCgkIBAMCBgCXBgMEAQkK"
        "AAR6BQEICnoHAAwQ1Dzk/DzkERI5ERI5MQAv7Bc5MAFLsBBUWL0ADABAAAEA"
        "DAAM/8A4ETc4WUATEA1ADVANYA1zBHoKcA3gDfANCV0TMxE3FwcRIxEHJzfH"
        "uH1Mybh7SsUGFP2mWmqN/OMCmlhqjQD//wCH/+MEogdtAicANgAAAAAABwEI"
        "AIsBdf//AG//4wPHBmYCJwBWAAAAAAAHAOAAFwAA//8AXAAABR8HbQInAD0A"
        "AAAAAAcBCAC+AXX//wBYAAAD2wZmAicAXQAAAAAABwDgABsAAAACAQT+ogGu"
        "BZgAAwAHABxADQH1AAT1BQgEAAUGAggQ3DzsMjEAENTs1OwwAREjERMRIxEB"
        "rqqqqgGY/QoC9gQA/QoC9gAAAgAKAAAFugXVAAwAGQBnQCAQCakLDZUAgRKV"
        "DgsHBwEZEwQPDRYZBDIKEQ0cCAB5GhD0POwyxPTsEMQXOTEAL8Yy7vbuEO4y"
        "MEAoIBt/G7AbA58JnwqfC58Mnw6fD58QnxG/Cb8Kvwu/DL8Ovw+/EL8REF0B"
        "XRMhIAAREAApAREjNTMTESEVIREzIAAREAAh0wGgAbEBlv5p/lD+YMnJywFQ"
        "/rDzATUBH/7h/ssF1f6X/oD+fv6WAryQAeP+HZD96gEYAS4BLAEXAAIAcf/j"
        "BHUGFAAOACgBJ0BeJXsmJR4jHiR7IyMeD3sjHih7JygeIx4mJygnJSQlKCgn"
        "IiMiHyAfISAgH0IoJyYlIiEgHwgjHgMPIwO5Gwm5FYwbI7EpJicSDCEgGCgl"
        "IyIfBR4PBgwSElEGEhhFKRD87PTsETk5FzkSOTkREjk5MQAQ7MT07BDuEjkS"
        "ORIXOTBLU1gHEA7JBxAIyQcQCMkHEA7JBxAI7QcO7QcQBe0HEAjtWSKyPyoB"
        "AV1AdhYlKx8oIi8jLyQpJS0mLScqKDYlRiVYIFghYCBgIWYidSB1IXUiEyUj"
        "JSQmJiYnJyg2JDYlRiRFJVogWiFiIGIhfwB/AX8CegN7CX8Kfwt/DH8Nfw5/"
        "D38QfxF/En8TfxR7FXobehx/HX8ediB2IXgioCrwKiddAF0BLgEjIgYVFBYz"
        "MjY1NCYTFhIVFAAjIgA1NAAzMhYXJwUnJSczFyUXBQNGMlgpp7mukpGuNgl+"
        "cv7k5uf+5QEU3RI0Kp/+wSEBGbXkfwFNIf7ZA5MRENjDvN7evHq8ASaP/uCt"
        "//7JATf/+gE3BQW0a2NczJFvYWIA/////AAABOcHawInADwAAAAAAAcBBABz"
        "AXX//wA9/lYEfwZmAicAXAAAAAAABwCNAF4AAAACAMkAAASNBdUADAAVAD1A"
        "Gw6VCQ2VAvYAgQsVDwkDBAESGQY/DQoBHAAEFhD87DIy/OwRFzkxAC/0/OzU"
        "7DBACQ8XHxc/F18XBAFdEzMRMzIEFRQEKwERIxMRMzI2NTQmI8nK/vsBAf7/"
        "+/7Kyv6NmpmOBdX++OHc3OL+rgQn/dGShoaRAAIAuv5WBKQGFAAQABwAPkAb"
        "FLkFCBq5AA6MCLgBvQOXHRESC0cXBAAIAkYdEPzsMjL07DEAEOzk5PTE7BDG"
        "7jBACWAegB6gHuAeBAFdJREjETMRPgEzMhIREAIjIiYBNCYjIgYVFBYzMjYB"
        "c7m5OrF7zP//zHuxAjinkpKnp5KSp6j9rge+/aJkYf68/vj++P68YQHry+fn"
        "y8vn5wAAAQDZAi0F2wLXAAMAEbYAnAIEAQAEENTEMQAQ1OwwEyEVIdkFAvr+"
        "AteqAAABARkAPwWcBMUACwCFQE0KnAsKBwgHCZwICAcEnAMEBwcGBZwGBwYE"
        "nAUEAQIBA5wCAgELnAABAAqcCQoBAQBCCggHBgQCAQAIBQMLCQwLCgkHBQQD"
        "AQgCAAgGDBDUPMwyFzkxABDUPMwyFzkwS1NYBxAI7QcQBe0HEAXtBxAI7QcQ"
        "Be0HEAjtBxAF7QcQCO1ZIgkCBwEBJwEBNwEBBZz+NwHJd/41/jV2Acj+OHYB"
        "ywHLBEz+Nf43eQHL/jV5AckBy3n+NQHLAAEAiQKcAsUF3wAKACxAGAcA3QkD"
        "3QQC3Qn3BZELCHwGXQN8AXwACxDc9OT85DEAEPTs7NTsEO4yMBMzEQc1NzMR"
        "MxUhnMzf5onN/dcDCgJjKXQn/StuAAABAF4CnAK0BfAAGABKQCQAfQYEABd9"
        "BgYEQgQCAA7dDwDdAvcL3Q8SkRkADgh+ARUOAxkQ3MTUxOwROTEAEPTE7Pzs"
        "EO4REjkwS1NYBxAF7RcyBwXtWSIBIRUhNTY3ADU0JiMiBgc1NjYzMhYVFAEG"
        "AQwBqP2qIj8BWGhVNHpITYU5ka7+tTgDDnJuHzgBMV5CUSMjexwchGyL/uQw"
        "AAEAYgKNAs0F8AAoAEhAJwAVEwrdCR/dIBPdFQ3dCfgG9xzdIPgjkSkWEwAU"
        "GX4mEH4DFB8JKRDcxMTU7NTsETk5OTEAEPTk7Pzk7NTsEO4Q7hESOTABFhYV"
        "FAYjIiYnNRYWMzI2NTQmIyM1MzI2NTQmIyIGBzU2NjMyFhUUBgIMXGW+sTl9"
        "RjR3Q214b2xWXl5hZF8oZlFJgDeQqVoEYBJtUnyGFRR5GxpPRkpMbD88Oj0S"
        "F3MREnZjRWD//wCJ/+MHfwXwACcA8AAAAAAAJwC8AzUAAAAHAQkEi/1k//8A"
        "if/jBz8F8AAnAPAAAAAAACcAvAM1AAAABwDxBIv9ZP//AGL/4wd/BfAAJwDy"
        "AAAAAAAnALwDNQAAAAcBCQSL/WT//wBz/+MFiwdtAicAKgAAAAAABwEKARsB"
        "df//AHH+VgRaBkgCJwBKAAAAAAAHANoAiwAA//8AyQAAAZUHUAInACwAAAAA"
        "AAcBC/8vAXX//wCH/nUEogXwAicANgAAAAAABwDdAIsAAP//AG/+dQPHBHsC"
        "JwBWAAAAAAAHAN0AFwAA//8Ac//jBScHawInACYAAAAAAAcBBAEtAXX//wBx"
        "/+MD5wZmAicARgAAAAAABwCNAIkAAP//AHP/4wUnB20CJwAmAAAAAAAHAQgB"
        "LQF1//8Acf/jA+cGZgInAEYAAAAAAAcA4ACJAAAAAgBx/+ME9AYUABgAJABK"
        "QCQHA9MJAfkiuQAWHLkNEIwWuAWXCwIfDAQDAAgICgZHGRITRSUQ/Oz0PMT8"
        "FzzEMQAv7OT0xOwQxO79PO4yMLZgJoAmoCYDAV0BESE1ITUzFTMVIxEjNQ4B"
        "IyICERASMzIWARQWMzI2NTQmIyIGA6L+ugFGuJqauDqxfMv//8t8sf3Hp5KS"
        "qKiSkqcDtgFOfZOTffr8qGRhAUQBCAEIAURh/hXL5+fLy+fnAAABAGQB3wJ/"
        "AoMAAwARtgCcAgQBAAQQ3MwxABDU7DATIRUhZAIb/eUCg6QAAAEA2wJIAa4D"
        "RgADABK3AoMABAEZAAQQ1OwxABDU7DATMxUj29PTA0b+AAEAAP/jBI8F8AAx"
        "ARxAOiAS0yIQKwfTCRmhGq4WlR0BoQCuBJUvkR2MKQkyKyIhKSMmEhAKAw0J"
        "EQgsICYTBxEIEQ0cGQAmKiEvPNTEMvzExBI5ORI5ORESOREXORI5ORE5OTEA"
        "EMQy5PTs9OwQ7vbuEO4y3TzuMjABS7AJVEuwDFRbS7ANVFtLsA9UW0uwF1Rb"
        "S7AYVFtYvQAy/8AAAQAyADIAQDgRNzhZQHoOAA4BCwILMVQUaQxsDm4PbxBv"
        "EW8SbxNpFGsfbyBvIW8ibyNuJGwlaSdpLZ8HnwifCZ8KnwufDJ8Nnw6fD58Q"
        "nxGfEp8Tlh+fIJ8hnyKfI58knyWfJp8nnyifKZ8qnyufLJ0tMgAIAAkQCBAJ"
        "IAggCVUVUx5qFWcfCl0AXQEVLgEjIgYHIQchDgEVFBYXIQchHgEzMjY3FQ4B"
        "IyIAAyM3MzQmNTQ2NSM3MxIAMzIWBI9bqWadyiACQTf95gIBAQIBvjj+iiDK"
        "nWapW1m5YO3+yyjTN4sBAcI3nCgBNuxiuQVi1WlayLt7GC4jIC4Ye7vKWmnT"
        "SEgBIgEDexcvICMvF3sBAQEiRwACANcFDgMpBdkAAwAHAKVADQQAzgYCCAFk"
        "AAVkBAgQ1Pzc7DEAENQ87DIwAEuwDlRLsBFUW1i9AAgAQAABAAgACP/AOBE3"
        "OFkBS7AOVEuwDVRbS7AXVFtYvQAI/8AAAQAIAAgAQDgRNzhZAUuwEVRLsBlU"
        "W1i9AAgAQAABAAgACP/AOBE3OFkAS7AYVFi9AAj/wAABAAgACABAOBE3OFlA"
        "EWABYAJgBWAGcAFwAnAFcAYIAV0BMxUjJTMVIwJey8v+ecvLBdnLy8sAAQFz"
        "BO4C8AX2AAMAf0ARAgMAAwEAAANCAAL6BAEDAwQQxBDAMQAQ9MwwS1NYBxAF"
        "yQcQBclZIgBLsAxUWL0ABP/AAAEABAAEAEA4ETc4WQBLsA5UWL0ABABAAAEA"
        "BAAE/8A4ETc4WUAgBgIVAiUBJQI2AkYCVgJqAWcCCQ8ADwEfAB8BLwAvAQZd"
        "AV0BMwMjAje55JkF9v74AAEAtgUOA0oF6QAdAHVAIRYQDwMTDAcBAAMIFwzD"
        "BBPDGwj6HhABDwAHFlYYB1YJHhDU7NTsETk5OTkxABD0POzU7DISFzkREhc5"
        "MABLsAxUWL0AHv/AAAEAHgAeAEA4ETc4WQBLsA5UWL0AHgBAAAEAHgAe/8A4"
        "ETc4WbQQCx8aAl0BJy4BIyIGHQEjNDYzMhYfAR4BMzI2PQEzDgEjIiYB/DkZ"
        "HwwkKH1nViQ9MDkXIg8gKH0CZ1QiOwU5IQ4LMi0GZXYQGx4NDDMpBmR3EAAB"
        "AQwE7gKLBfYAAwCJQBEBAgMCAAMDAkIAAfoEAQMDBBDEEMAxABD0zDBLU1gH"
        "EAXJBxAFyVkiAEuwDFRYvQAE/8AAAQAEAAQAQDgRNzhZAEuwDlRYvQAEAEAA"
        "AQAEAAT/wDgRNzhZQCoGAAYBFgASASQAJAE1AUMBVQBVAZ8AnwGvAK8BDg8A"
        "DwMfAB8DLwAvAwZdAV0BEyMDAcfEmeYF9v74AQgAAQDPBO4DMQX4AAYAd0AK"
        "BAAFAvoHBAIGBxDUxDkxABD0PMQ5MABLsAxUWL0AB//AAAEABwAHAEA4ETc4"
        "WQBLsA5UWL0ABwBAAAEABwAH/8A4ETc4WQFLsA5UWL0AB//AAAEABwAHAEA4"
        "ETc4WUATDwAPAQwEHwAfAR0ELwAvAS0ECQBdATMTIycHIwGivNOLpqaLBfj+"
        "9rKyAAEAzwTuAzEF+AAGAIZACgMEAQD6BwMFAQcQ1MQ5MQAQ9MQyOTAAS7AM"
        "VEuwCVRbS7AKVFtLsAtUW1i9AAf/wAABAAcABwBAOBE3OFkAS7AOVFi9AAcA"
        "QAABAAcAB//AOBE3OFkBS7AOVFi9AAf/wAABAAcABwBAOBE3OFlAEwAAAwMA"
        "BhAAEgMQBiAAIgMgBgkAXQEDMxc3MwMBotOLpqaL0wTuAQqysv72AAACAD8C"
        "nAL0Bd8AAgANANRAFgADCwfdBQEJ9wORDgEMCgBdBggEDA4Q3NQ8xOwyETkx"
        "ABD0/NQ87DISOTABS7AOVEuwD1RbS7AQVFtLsBFUW0uwC1RbS7AKVFtYvQAO"
        "AEAAAQAOAA7/wDgRNzhZAEuwEVRLsA5UW1i9AA7/wAABAA4ADgBAOBE3OFlA"
        "VAsBHQEvATkBSQFGA1kDaQOLA6sDuwMLAQAPAQ8CDwUPBg8HDwgPCw8MDw0T"
        "AB8BHwIfBR8GHwcfCB8LHwwfDSIANQBHAEsNUwBbDWUAhAClALUAHl0BXQkB"
        "IQMzETMVIxUjNSE1Ad3+ywE1FqaHh5D+YgVm/l0CHP3kbbq6eQAAAQDHBQYD"
        "OQX4AA0AakAOBwAEwwv6DgdWCAFWAA4Q1OzU7DEAEPT8xDIwAEuwDFRYvQAO"
        "/8AAAQAOAA4AQDgRNzhZAEuwDlRYvQAOAEAAAQAOAA7/wDgRNzhZAUuwDlRL"
        "sA9UW1i9AA7/wAABAA4ADgBAOBE3OFkTMx4BMzI2NzMOASMiJsd2DWNTUmEQ"
        "dgqgj5CfBfg2OTc4d3t6AAABAZoFDgJmBdsAAwARtgAC+gQBAAQQ1MwxABD0"
        "zDABMxUjAZrMzAXbzQAAAAAAAgABAAAAAAAUAAMAAQAAARoAAAEGAAABAAAA"
        "AAAAAAEDAAAAAgAAAAAAAAAAAAAAAAAAAAEAAAMEBQYHCAkKCwwNDg8QERIT"
        "FBUWFxgZGhscHR4fICEiIyQlJicoKSorLC0uLzAxMjM0NTY3ODk6Ozw9Pj9A"
        "QUJDREVGR0hJSktMTU5PUFFSU1RVVldYWVpbXF1eX2BhAGJjZGVmZ2hpamts"
        "bW5vcHFyc3R1dnd4eXp7fH1+f4CBgoOEhYaHiImKi4yNjo+QkZKTlJWWl5iZ"
        "mpucnZ6foKGio6SlpqeoqaqrrK2ur7CxsrO0tba3uLm6u7y9vr/AwcLDxMXG"
        "x8jJysvMzc7P0NEA0tPU1dbX2Nna29zd3t/gAAQCPgAAADoAIAAEABoAfgD/"
        "AQcBEQEfATEBQgFTAWEBeAF+AZICxwLdA6kDwCAmIDAgOiCsISIiBiIeIisi"
        "SCJlJcr7Av//AAAAIACgAQYBDAEeATABQQFSAV4BeAF9AZICxgLYA6kDwCAT"
        "IDAgOSCsISIiAiIPIisiSCJgJcr7Af///+MAAP/1AAD/2AAA/6D/XgAA/0P/"
        "aP8UAAAAAPz2/NsAAOCW4IXgVt9qAAAAAN5x3l8AANrvBb8AAQAAADgAAAD0"
        "AAAA/AAAAAAA+gAAAAAAAAD6APwAAAAAAQIAAAAAAAAAAAEgASgAAAAAAUIA"
        "AAAAAAAArACjAIQAhQC9AJYA5wCGAI4AiwCdAKkApAEAAIoA2QCDAJMA8QDy"
        "AI0AlwCIAMMA3QDwAJ4AqgDzAPQA9QCiAK0AyQDHAK4AYgBjAJAAZADLAGUA"
        "yADKAM8AzADNAM4A6ABmANIA0ADRAK8AZwDvAJEA1QDTANQAaADqAOwAiQBq"
        "AGkAawBtAGwAbgCgAG8AcQBwAHIAcwB1AHQAdgB3AOkAeAB6AHkAewB9AHwA"
        "uAChAH8AfgCAAIEA6wDtALoA/QD+AAAAAAAAAP8A+ADWAPkA+gDjAOQA1wDg"
        "ANoA2wDcAN8A2ADeALIAswAAAAAAAAC2ALcAxAAAALQAtQDFAAAAggDCAIcA"
        "AAAAAAAAqwCYAAAAAAAAAKgAmgAAAJkA7gAAAAAAvAAAAAAAAAEBAKUAAAAA"
        "AAAAkgCPAAAAAAAAAJQAlQTNAGYAAAAAAosAAAKLAAADNQE1A64AxQa0AJ4F"
        "FwCqB5oAcQY9AIECMwDFAx8AsAMfAKQEAAA9BrQA2QKLAJ4C4wBkAosA2wKy"
        "AAAFFwCHBRcA4QUXAJYFFwCcBRcAZAUXAJ4FFwCPBRcAqAUXAIsFFwCBArIA"
        "8AKyAJ4GtADZBrQA2Qa0ANkEPwCTCAAAhwV5ABAFfQDJBZYAcwYpAMkFDgDJ"
        "BJoAyQYzAHMGBADJAlwAyQJc/5YFPwDJBHUAyQbnAMkF/ADJBkwAcwTTAMkG"
        "TABzBY8AyQUUAIcE4//6BdsAsgV5ABAH6QBEBXsAPQTj//wFewBcAx8AsAKy"
        "AAADHwDHBrQA2QQA/+wEAACqBOcAewUUALoEZgBxBRQAcQTsAHEC0QAvBRQA"
        "cQUSALoCOQDBAjn/2wSiALoCOQDBB8sAugUSALoE5QBxBRQAugUUAHEDSgC6"
        "BCsAbwMjADcFEgCuBLwAPQaLAFYEvAA7BLwAPQQzAFgFFwEAArIBBAUXAQAG"
        "tADZBXkAEAV5ABAFlgBzBQ4AyQX8AMkGTABzBdsAsgTnAHsE5wB7BOcAewTn"
        "AHsE5wB7BOcAewRmAHEE7ABxBOwAcQTsAHEE7ABxAjkAkAI5/8cCOf/eAjn/"
        "9AUSALoE5QBxBOUAcQTlAHEE5QBxBOUAcQUSAK4FEgCuBRIArgUSAK4EAAA5"
        "BAAAwwUXAKwFFwCBBAAAXAS4ATMFFwCeBQoAuggAARsIAAEbCAABJwQAAXME"
        "AADXBrQA2QfLAAgGTABmBqoA3Qa0ANkGtADZBrQA2QUXAFIFFwCuBCMAaAVk"
        "ABkGDgCcBLb/4QQrAC8DxQBzA8UAYAYdAE4H2wB7BOUASAQ/AI8DNQE1BrQA"
        "2QUZAD0FFwAfBrQA2QVa//oE5QCeBOUAwQgAAOwFFwAABXkAEAV5ABAGTABz"
        "CI8AcwgvAHEEAAAACAAAAAQlAK4EJQCuAosArgKLALIGtADZA/QABgS8AD0E"
        "4//8AVb+iQUXAF4DMwCeAzMAwQUKAC8FCgAvBAAAOQKLANsCiwCuBCUArgq8"
        "AHEFeQAQBQ4AyQV5ABAFDgDJBQ4AyQJcAKICXP/+AlwABgJcADsGTABzBkwA"
        "cwZMAHMF2wCyBdsAsgXbALICOQDBBAAAwQQAALYEAADVBAAAxwQAAZoEAADu"
        "BAABIwQAAPAEAAFMBAAAwQR///ICRgACBRQAhwQrAG8FewBcBDMAWAKyAQQG"
        "MwAKBOUAcQTj//wEvAA9BNcAyQUUALoGtADZBrQBGQM1AIkDNQBeAzUAYgfB"
        "AIkHwQCJB8EAYgYzAHMFFABxAlwAyQUUAIcEKwBvBZYAcwRmAHEFlgBzBGYA"
        "cQUUAHEC4wBkAosA2wUXAAAEAADXBAABcwQAALYEAAEMBAAAzwQAAM8DNQA/"
        "BAAAxwQAAZoAAgAAAAAAAP8rAI8AAAAAAAAAAAAAAAAAAAAAAAAAAAEMAAAA"
        "AQACAAMABAAFAAYABwAIAAkACgALAAwADQAOAA8AEAARABIAEwAUABUAFgAX"
        "ABgAGQAaABsAHAAdAB4AHwAgACEAIgAjACQAJQAmACcAKAApACoAKwAsAC0A"
        "LgAvADAAMQAyADMANAA1ADYANwA4ADkAOgA7ADwAPQA+AD8AQABBAEIAQwBE"
        "AEUARgBHAEgASQBKAEsATABNAE4ATwBQAFEAUgBTAFQAVQBWAFcAWABZAFoA"
        "WwBcAF0AXgBfAGAAYQBiAGMAZABlAGYAZwBoAGkAagBrAGwAbQBuAG8AcABx"
        "AHIAcwB0AHUAdgB3AHgAeQB6AHsAfAB9AH4AfwCAAIEAggCDAIQAhQCGAIcA"
        "iACJAIoAiwCMAI0AjgCPAJAAkQCSAJMAlACVAJYAlwCYAJkAmgCbAJwAnQCe"
        "AJ8AoAChAKIAowCkAKUApgCnAKgAqQCqAKsArACtAK4ArwCwALEAsgCzALQA"
        "tQC2ALcAuAC5ALoAuwC8AL0AvgC/AMAAwQDCAMMAxADFAMYAxwDIAMkAygDL"
        "AMwAzQDOAM8A0ADRANMA1ADVANYA1wDYANkA2gDbANwA3QDeAN8A4ADhAOIA"
        "4wDkAOUA5gDnAOgA6QDqAOsA7ADtAO4A7wDwAPEA8gDzAPUA9AD2APgA+QD6"
        "APsA/AD9AP4A/wEAAQEBAgEDAQQBBQEGAQcBCAEJAQoBCwEMAQ0Jc2Z0aHlw"
        "aGVuDnBlcmlvZGNlbnRlcmVkBEV1cm8FYzY0NTkFYzY0NjAFYzY0NjEFYzY0"
        "NjIFYzY0NjMFYzY0NjYFYzY0NjcFYzY0NjgFYzY0NjkAAAAAACIAIgAiACIA"
        "WACTAP8BtgJPA4MDsgP2BCEEbgSYBLQEygTeBQMFRQWCBgUGfgbhB0wHtwf8"
        "CGUIzwjuCRYJUglzCa4KHwrfC1gLsAv7DDoMaQyTDOYNFA09DXoODQ4vDqwP"
        "AA9FD4UP6xB1EPERKRFwEeASvROKE+sUUBSLFLEU4RUDFRgVQBXWFiIWbRa5"
        "FyMXexffGBsYQxiAGPgZFhl3GbMaBBpSGqEa2BuHG8QcABydHboehx93H+gg"
        "WyByIPMhNSFCIeIh7yH8IgkiFiIjIjAiPSJKIlciZCJxIn4iiyKYIqUisiK/"
        "Iswi2SLmIvMjACMNIxojJyM0I0EjTiNbI2gjlCPPJDQkjyUzJVMlgSYUJron"
        "SyeQJ7coEyhYKMMpXyolKlwqoyrpK3or0yxELI8ssS1QLaAuDi5SLqovhzBB"
        "ML0xBTEhMVAxzzJIMnoy3zNGM3AzcDN9M4ozlzPmNHo0jzSjNNE0/zUcNTk1"
        "ZzWRNZ41qzXPNls2kzbNN0M3qTfrOAA4HDhKOQ85HDkpOTY5QzlQOV05ajl3"
        "OYQ5kTmeOas5uDnFOdI57zocOns6oDrlOwg7XjuOO8Q79DwiPF88pzy0PME8"
        "zjzbPP49Yz47Pkg+VT6YPuc+/T9hP40/3EA6QEtAXEBtQHpAh0CUQKFArkC7"
        "QMhA1UDiQUBBVkFrQkVCqkL3Q19DskP/RFVE20UqRT8AAAAAAAEAAC2GAAEH"
        "lBgAAAoVeAAQACT/0wAQACX/twAQACYAAAAQACcAAAAQACkAAAAQACoASwAQ"
        "ACsAAAAQAC0AcgAQAC4AAAAQAC8AAAAQADIAOQAQADMAAAAQADQASwAQADUA"
        "AAAQADf/RAAQADn/iAAQADr/rQAQADv/mgAQADz/DQAQAD0AAAAQAEkAAAAQ"
        "AFEAAAAQAFIAJgAQAFUAAAAQAFn/yQAQAFoAAAAQAFz/3AAQAGL/0wAQAGQA"
        "AAAQAGcAOQAQAHgAAAAQAHkAJgAQAHoAJgAQAHsAJgAQAHwAJgAQAH0AJgAQ"
        "AIkAAAAQAJAAAAAQAK3/0wAQAK7/0wAQAK8AOQAQALr/3AAQALv/DQAQAMf/"
        "0wAQAMn/0wAQANAAOQAQANEAOQAQANIAOQAQAOUAAAAQAOkAAAAQAOr/DQAQ"
        "AOv/3AAQAOwAAAAQAPYASwAQAPsAAAAQAP0AAAAkABD/0wAkABH/3AAkAB3/"
        "3AAkACQAOQAkACb/3AAkACr/3AAkADL/3AAkADT/3AAkADYAAAAkADf/YQAk"
        "ADgAAAAkADn/fQAkADr/kAAkADsAAAAkADz/YQAkAEb/3AAkAEf/3AAkAEj/"
        "3AAkAEn/twAkAFL/3AAkAFT/3AAkAFf/3AAkAFgAAAAkAFn/iAAkAFr/rQAk"
        "AFz/dQAkAGIAOQAkAGT/3AAkAGf/3AAkAGgAAAAkAG//3AAkAHD/3AAkAHH/"
        "3AAkAHL/3AAkAHP/3AAkAHn/3AAkAHr/3AAkAHv/3AAkAHz/3AAkAH3/3AAk"
        "AH4AAAAkAH8AAAAkAIAAAAAkAIEAAAAkAKn/twAkAKoAAAAkAK0AOQAkAK4A"
        "OQAkAK//3AAkALT++AAkALX/AwAkALr/dQAkALv/YQAkAMUALwAkAMcAOQAk"
        "AMkAOQAkAND/3AAkANH/3AAkANL/3AAkANMAAAAkANQAAAAkANUAAAAkAOMA"
        "AAAkAOr/YQAkAOv/dQAkAPb/3AAkAPkAAAAkAPv/3AAkAPz/3AAkAP3/3AAk"
        "AP7/3AAlABAAAAAlACb/3AAlACr/3AAlADL/3AAlADb/3AAlADn/wQAlADr/"
        "twAlADz/kAAlAGT/3AAlAGf/3AAlAKn/wQAlAKr/3AAlAK//3AAlALT/kAAl"
        "ALX/kAAlALv/kAAlAMX/rQAlAND/3AAlANH/3AAlANL/3AAlAOP/3AAlAOr/"
        "kAAlAPb/3AAlAPn/3AAlAPv/3AAlAP3/3AAmABAAAAAmACQAAAAmADYAAAAm"
        "ADz/3AAmAGIAAAAmAKn/3AAmAKr/3AAmAK0AAAAmAK4AAAAmALQAAAAmALUA"
        "JgAmALv/3AAmAMUAAAAmAMcAAAAmAMkAAAAmAOMAAAAmAOr/3AAmAPkAAAAn"
        "ABAAAAAnACT/3AAnADn/3AAnADoAAAAnADz/kAAnAGL/3AAnAKn/3AAnAKr/"
        "3AAnAK3/3AAnAK7/3AAnALT/0wAnALX/yQAnALv/kAAnAMX/RAAnAMf/3AAn"
        "AMn/3AAnAOr/kAApABAAAAApABH+twApAB3/YQApACT/RAApADb/3AApADf/"
        "3AApAET/RAApAEj/kAApAEz/awApAFL/twApAFX/awApAFj/kAApAFz/RAAp"
        "AGL/RAApAGn/RAApAGr/RAApAGv/RAApAGz/RAApAG3/RAApAG7/RAApAHD/"
        "kAApAHH/kAApAHL/kAApAHP/kAApAHn/twApAHr/twApAHv/twApAHz/twAp"
        "AH3/twApAH7/kAApAH//kAApAID/kAApAIH/kAApAKkAAAApAKoAAAApAK3/"
        "RAApAK7/RAApALT/0wApALUAAAApALr/RAApAMX+iAApAMf/RAApAMn/RAAp"
        "AOP/3AApAOv/RAApAPn/3AAqABAAAAAqACQAAAAqADf/twAqADoAAAAqADz/"
        "mgAqAGIAAAAqAKn/3AAqAKr/3AAqAK0AAAAqAK4AAAAqALT/0wAqALX/0wAq"
        "ALv/mgAqAMX/yQAqAMcAAAAqAMkAAAAqAOr/mgArABAAAAArABH/3AArAB0A"
        "AAArAKkAAAArAKoAAAArALT/twArALX/wQArAMX/twAtABD/twAtACT/3AAt"
        "AGL/3AAtAKn/3AAtAKr/3AAtAK3/3AAtAK7/3AAtALT/twAtALX/wQAtAMX/"
        "kAAtAMf/3AAtAMn/3AAuABD/KQAuACT/3AAuACb/kAAuADL/kAAuADf/YQAu"
        "ADj/yQAuADr/twAuADz/twAuAET/3AAuAEj/mgAuAFL/mgAuAFj/mgAuAFz/"
        "awAuAGL/3AAuAGT/kAAuAGf/kAAuAGj/yQAuAGn/3AAuAGr/3AAuAGv/3AAu"
        "AGz/3AAuAG3/3AAuAG7/3AAuAHD/mgAuAHH/mgAuAHL/mgAuAHP/mgAuAHn/"
        "mgAuAHr/mgAuAHv/mgAuAHz/mgAuAH3/mgAuAH7/mgAuAH//mgAuAID/mgAu"
        "AIH/mgAuAKn/fQAuAKoAAAAuAK3/3AAuAK7/3AAuAK//kAAuALT/wQAuALX/"
        "wQAuALr/awAuALv/twAuAMUAAAAuAMf/3AAuAMn/3AAuAND/kAAuANH/kAAu"
        "ANL/kAAuANP/yQAuANT/yQAuANX/yQAuAOr/twAuAOv/awAuAPv/kAAuAP3/"
        "kAAvABD/3AAvACQALwAvADL/twAvADf+5gAvADj/mgAvADn/HwAvADr/RAAv"
        "ADz+8AAvAEQAAAAvAEj/3AAvAFL/3AAvAFj/3AAvAFz/RAAvAGIALwAvAGf/"
        "twAvAGj/mgAvAGkAAAAvAGoAAAAvAGsAAAAvAGwAAAAvAG0AAAAvAG4AAAAv"
        "AHD/3AAvAHH/3AAvAHL/3AAvAHP/3AAvAHn/3AAvAHr/3AAvAHv/3AAvAHz/"
        "3AAvAH3/3AAvAH7/3AAvAH//3AAvAID/3AAvAIH/3AAvAKkAAAAvAKoAAAAv"
        "AK0ALwAvAK4ALwAvAK//twAvALT+YQAvALX95gAvALr/RAAvALv+8AAvAMUA"
        "AAAvAMcALwAvAMkALwAvAND/twAvANH/twAvANL/twAvANP/mgAvANT/mgAv"
        "ANX/mgAvAOr+8AAvAOv/RAAyABAAOQAyABH/rQAyAB3/3AAyACT/3AAyADn/"
        "3AAyADv/fQAyADz/kAAyAGL/3AAyAKn/3AAyAKoAAAAyAK3/3AAyAK7/3AAy"
        "ALT/0wAyALX/3AAyALv/kAAyAMX/RAAyAMf/3AAyAMn/3AAyAOr/kAAzABD/"
        "0wAzABH+wQAzAB0AAAAzACT/fQAzADgAAAAzADoAAAAzADz/0wAzAET/pAAz"
        "AEj/twAzAEz/0wAzAFH/3AAzAFL/twAzAFX/3AAzAFb/3AAzAFj/3AAzAFwA"
        "AAAzAGL/fQAzAGgAAAAzAGn/pAAzAGr/pAAzAGv/pAAzAGz/pAAzAG3/pAAz"
        "AG7/pAAzAHD/twAzAHH/twAzAHL/twAzAHP/twAzAHj/3AAzAHn/twAzAHr/"
        "twAzAHv/twAzAHz/twAzAH3/twAzAH7/3AAzAH//3AAzAID/3AAzAIH/3AAz"
        "AKn/3AAzAKoAAAAzAK3/fQAzAK7/fQAzALQAJgAzALUAJgAzALoAAAAzALv/"
        "0wAzAMX+twAzAMf/fQAzAMn/fQAzANMAAAAzANQAAAAzANUAAAAzAOT/3AAz"
        "AOr/0wAzAOsAAAAzAPr/3AA0ABAAOQA0AKkAAAA0AKoAAAA0ALT/0wA0ALX/"
        "3AA0AMX/fQA1ABD/rQA1ABH/twA1AB3/wQA1ACT/rQA1ACb/mgA1ADf/awA1"
        "ADn/kAA1ADr/rQA1ADz/fQA1AET/0wA1AEj/pAA1AFL/pAA1AFj/pAA1AFz/"
        "kAA1AGL/rQA1AGT/mgA1AGn/0wA1AGr/0wA1AGv/0wA1AGz/0wA1AG3/0wA1"
        "AG7/0wA1AHD/pAA1AHH/pAA1AHL/pAA1AHP/pAA1AHn/pAA1AHr/pAA1AHv/"
        "pAA1AHz/pAA1AH3/pAA1AH7/pAA1AH//pAA1AID/pAA1AIH/pAA1AKn/kAA1"
        "AKr/3AA1AK3/rQA1AK7/rQA1ALT/awA1ALX/fQA1ALr/kAA1ALv/fQA1AMX/"
        "3AA1AMf/rQA1AMn/rQA1AOr/fQA1AOv/kAA1APv/mgA1AP3/mgA2ACQAJgA2"
        "ACYAAAA2ACoAAAA2ADIAAAA2ADQAAAA2ADYAAAA2AGIAJgA2AGQAAAA2AGcA"
        "AAA2AK0AJgA2AK4AJgA2AK8AAAA2AMcAJgA2AMkAJgA2ANAAAAA2ANEAAAA2"
        "ANIAAAA2AOMAAAA2APYAAAA2APkAAAA2APsAAAA2AP0AAAA3ABD/RAA3ABH/"
        "DQA3AB3/HwA3ACT/YQA3ACb/iAA3ADf/3AA3AET+rQA3AEb+pAA3AEj+pAA3"
        "AEz/wQA3AFL+pAA3AFX+0wA3AFb+rQA3AFj+yQA3AFr+rQA3AFz+wQA3AGL/"
        "YQA3AGT/iAA3AGn+rQA3AGr+rQA3AGv+rQA3AGz+rQA3AG3+rQA3AG7+rQA3"
        "AG/+pAA3AHD+pAA3AHH+pAA3AHL+pAA3AHP+pAA3AHn+pAA3AHr+pAA3AHv+"
        "pAA3AHz+pAA3AH3+pAA3AH7+yQA3AH/+yQA3AID+yQA3AIH+yQA3AKn/RAA3"
        "AKr/kAA3AK3/YQA3AK7/YQA3ALQAAAA3ALX/0wA3ALr+wQA3AMX++AA3AMf/"
        "YQA3AMn/YQA3AOT+rQA3AOv+wQA3APr+rQA3APv/iAA3APz+pAA3AP3/iAA3"
        "AP7+pAA4ACQAAAA4AC0AAAA4AD3/3AA4AGIAAAA4AK0AAAA4AK4AAAA4AMcA"
        "AAA4AMkAAAA4AOX/3AA5ABD/iAA5ABH++AA5AB3/WQA5ACT/fQA5ADL/3AA5"
        "AET/YQA5AEj/YQA5AEz/0wA5AFL/YQA5AFj/dQA5AFz/yQA5AGL/fQA5AGf/"
        "3AA5AGn/YQA5AGr/YQA5AGv/YQA5AGz/YQA5AG3/YQA5AG7/YQA5AHD/YQA5"
        "AHH/YQA5AHL/YQA5AHP/YQA5AHn/YQA5AHr/YQA5AHv/YQA5AHz/YQA5AH3/"
        "YQA5AH7/dQA5AH//dQA5AID/dQA5AIH/dQA5AKn/TgA5AKr/kAA5AK3/fQA5"
        "AK7/fQA5AK//3AA5ALQAAAA5ALUAAAA5ALr/yQA5AMX+5gA5AMf/fQA5AMn/"
        "fQA5AND/3AA5ANH/3AA5ANL/3AA5AOv/yQA6ABD/rQA6ABH/FQA6AB3/iAA6"
        "ACT/kAA6AET/fQA6AEj/iAA6AEz/0wA6AFL/iAA6AFX/pAA6AFj/twA6AFz/"
        "3AA6AGL/kAA6AGn/fQA6AGr/fQA6AGv/fQA6AGz/fQA6AG3/fQA6AG7/fQA6"
        "AHD/iAA6AHH/iAA6AHL/iAA6AHP/iAA6AHn/iAA6AHr/iAA6AHv/iAA6AHz/"
        "iAA6AH3/iAA6AH7/twA6AH//twA6AID/twA6AIH/twA6AKn/kAA6AKr/3AA6"
        "AK3/kAA6AK7/kAA6ALT/3AA6ALUAAAA6ALr/3AA6AMX++AA6AMf/kAA6AMn/"
        "kAA6AOv/3AA7ABD/mgA7ACQAAAA7ACb/awA7ADL/fQA7ADf/3AA7AEj/pAA7"
        "AGIAAAA7AGT/awA7AGf/fQA7AHD/pAA7AHH/pAA7AHL/pAA7AHP/pAA7AKn/"
        "kAA7AKoAAAA7AK0AAAA7AK4AAAA7AK//fQA7ALT/YQA7ALX/rQA7AMX/0wA7"
        "AMcAAAA7AMkAAAA7AND/fQA7ANH/fQA7ANL/fQA7APv/awA7AP3/awA8ABD/"
        "DQA8ABH+YQA8AB3+8AA8ACT/YQA8ACb/kAA8ADL/kAA8AET+5gA8AEj+8AA8"
        "AEz/twA8AFL+8AA8AFj/FQA8AGL/YQA8AGT/kAA8AGf/kAA8AGn+5gA8AGr+"
        "5gA8AGv+5gA8AGz+5gA8AG3+5gA8AG7+5gA8AHD+8AA8AHH+8AA8AHL+8AA8"
        "AHP+8AA8AHn+8AA8AHr+8AA8AHv+8AA8AHz+8AA8AH3+8AA8AH7/FQA8AH//"
        "FQA8AID/FQA8AIH/FQA8AKn/HwA8AKr/awA8AK3/YQA8AK7/YQA8AK//kAA8"
        "ALT/kAA8ALX/3AA8AMX++AA8AMf/YQA8AMn/YQA8AND/kAA8ANH/kAA8ANL/"
        "kAA8APv/kAA8AP3/kAA9ABD/3AA9AKkAAAA9AKoAAAA9ALT/3AA9ALX/3AA9"
        "AMX/3ABIAFv/3ABJABD/kABJABH/awBJAB3/twBJAFf/3ABJAFr/3ABJAFz/"
        "3ABJAKn/twBJAKr/3ABJALQAQQBJALUAAABJALr/3ABJAMX/FQBJAOv/3ABO"
        "AET/3ABOAEj/twBOAFL/twBOAFj/wQBOAFz/twBOAGn/3ABOAGr/3ABOAGv/"
        "3ABOAGz/3ABOAG3/3ABOAG7/3ABOAHD/twBOAHH/twBOAHL/twBOAHP/twBO"
        "AHn/twBOAHr/twBOAHv/twBOAHz/twBOAH3/twBOAH7/wQBOAH//wQBOAID/"
        "wQBOAIH/wQBOALr/twBOAOv/twBRABAAAABRABEAAABRAB0AAABRAKkAAABR"
        "AKoAAABRALT/awBRALX/kABRAMX/pABSABAAJgBSABH/3ABSAB0AAABSAFv/"
        "wQBSAKkAAABSAKoAAABSALT/awBSALX/twBSAMX/fQBVABD/fQBVABH/RABV"
        "AB3/3ABVAEb/0wBVAEf/3ABVAEj/0wBVAEkAAABVAEr/3ABVAEv/3ABVAFD/"
        "3ABVAFH/3ABVAFL/0wBVAFT/3ABVAFX/3ABVAFgAAABVAFkAAABVAFoAAABV"
        "AFv/yQBVAFwAAABVAF0AAABVAG//0wBVAHD/0wBVAHH/0wBVAHL/0wBVAHP/"
        "0wBVAHj/3ABVAHn/0wBVAHr/0wBVAHv/0wBVAHz/0wBVAH3/0wBVAH4AAABV"
        "AH8AAABVAIAAAABVAIEAAABVAKn/twBVAKoAAABVALQAAABVALUAVgBVALoA"
        "AABVAMX+yQBVAOYAAABVAOsAAABVAPf/3ABVAPz/0wBVAP7/0wBZABD/yQBZ"
        "ABH/YQBZAB3/kABZAKn/3ABZAKr/3ABZALQAAABZALX/3ABZAMX+8ABaABAA"
        "AABaABH/RABaAB3/kABaAKn/3ABaAKr/3ABaALQAAABaALUAAABaAMX/KQBb"
        "AEb/3ABbAEj/wQBbAFL/wQBbAG//3ABbAHD/wQBbAHH/wQBbAHL/wQBbAHP/"
        "wQBbAHn/wQBbAHr/wQBbAHv/wQBbAHz/wQBbAH3/wQBbAPz/3ABbAP7/3ABc"
        "ABD/3ABcABH+3ABcAB3/awBcAKn/3ABcAKr/3ABcALQAAABcALUAAABcAMX+"
        "0wBiABD/0wBiABH/3ABiAB3/3ABiACQAOQBiACb/3ABiACr/3ABiADL/3ABi"
        "ADT/3ABiADYAAABiADf/YQBiADgAAABiADn/fQBiADr/kABiADsAAABiADz/"
        "YQBiAEb/3ABiAEf/3ABiAEj/3ABiAEn/twBiAFL/3ABiAFT/3ABiAFf/3ABi"
        "AFgAAABiAFn/iABiAFr/rQBiAFz/dQBiAGIAOQBiAGT/3ABiAGf/3ABiAGgA"
        "AABiAG//3ABiAHD/3ABiAHH/3ABiAHL/3ABiAHP/3ABiAHn/3ABiAHr/3ABi"
        "AHv/3ABiAHz/3ABiAH3/3ABiAH4AAABiAH8AAABiAIAAAABiAIEAAABiAKn/"
        "twBiAKoAAABiAK0AOQBiAK4AOQBiAK//3ABiALT++ABiALX/AwBiALr/dQBi"
        "ALv/YQBiAMUALwBiAMcAOQBiAMkAOQBiAND/3ABiANH/3ABiANL/3ABiANMA"
        "AABiANQAAABiANUAAABiAOMAAABiAOr/YQBiAOv/dQBiAPb/3ABiAPkAAABi"
        "APv/3ABiAPz/3ABiAP3/3ABiAP7/3ABkABAAAABkACQAAABkADYAAABkADz/"
        "3ABkAGIAAABkAKn/3ABkAKr/3ABkAK0AAABkAK4AAABkALQAAABkALUAJgBk"
        "ALv/3ABkAMUAAABkAMcAAABkAMkAAABkAOMAAABkAOr/3ABkAPkAAABnABAA"
        "OQBnABH/rQBnAB3/3ABnACT/3ABnADn/3ABnADv/fQBnADz/kABnAGL/3ABn"
        "AKn/3ABnAKoAAABnAK3/3ABnAK7/3ABnALT/0wBnALX/3ABnALv/kABnAMX/"
        "RABnAMf/3ABnAMn/3ABnAOr/kABoACQAAABoAC0AAABoAD3/3ABoAGIAAABo"
        "AK0AAABoAK4AAABoAMcAAABoAMkAAABoAOX/3ABwAFv/3ABxAFv/3AByAFv/"
        "3ABzAFv/3AB4ABAAAAB4ABEAAAB4AB0AAAB4AKkAAAB4AKoAAAB4ALT/awB4"
        "ALX/kAB4AMX/pAB5ABAAJgB5ABH/3AB5AB0AAAB5AFv/wQB5AKkAAAB5AKoA"
        "AAB5ALT/awB5ALX/twB5AMX/fQB6ABAAJgB6ABH/3AB6AB0AAAB6AFv/wQB6"
        "AKkAAAB6AKoAAAB6ALT/awB6ALX/twB6AMX/fQB7ABAAJgB7ABH/3AB7AB0A"
        "AAB7AFv/wQB7AKkAAAB7AKoAAAB7ALT/awB7ALX/twB7AMX/fQB8ABAAJgB8"
        "ABH/3AB8AB0AAAB8AFv/wQB8AKkAAAB8AKoAAAB8ALT/awB8ALX/twB8AMX/"
        "fQB9ABAAJgB9ABH/3AB9AB0AAAB9AFv/wQB9AKkAAAB9AKoAAAB9ALT/awB9"
        "ALX/twB9AMX/fQCJABAAJgCJAKkAAACJAKoAAACJALT/kACJALX/kACJAMX/"
        "rQCQABAAAACQAKkAAACQAKoAAACQALT/rQCQALX/pACQAMX/kACpACQAAACp"
        "ACX/3ACpACb/3ACpACf/3ACpACkAAACpACr/3ACpACsAAACpAC3/3ACpAC4A"
        "AACpAC8AAACpADIAAACpADMAAACpADQAAACpADUAAACpADf/kACpADn/kACp"
        "ADr/3ACpADsAAACpADz/awCpAD0AAACpAEkAAACpAFEAAACpAFIAAACpAFUA"
        "AACpAFn/3ACpAFr/3ACpAFz/3ACpAGIAAACpAGT/3ACpAGcAAACpAHgAAACp"
        "AHkAAACpAHoAAACpAHsAAACpAHwAAACpAH0AAACpAIkAAACpAJAAlwCpAK0A"
        "AACpAK4AAACpAK8AAACpALr/3ACpALv/awCpAMcAAACpAMkAAACpANAAAACp"
        "ANEAAACpANIAAACpAOUAAACpAOkAAACpAOr/awCpAOv/3ACpAOwAAACpAPb/"
        "3ACpAPv/3ACpAP3/3ACqACT/twCqACX/twCqACb/3ACqACf/3ACqACkAAACq"
        "ACoAAACqACsAAACqAC3/3ACqAC4AAACqAC8AAACqADL/3ACqADMAAACqADQA"
        "AACqADUAAACqADf/RACqADn/TgCqADr/kACqADv/kACqADz/HwCqAD0AAACq"
        "AEkAAACqAFEAAACqAFIAAACqAFUAAACqAFn/3ACqAFr/3ACqAFz/3ACqAGL/"
        "twCqAGT/3ACqAGf/3ACqAHgAAACqAHkAAACqAHoAAACqAHsAAACqAHwAAACq"
        "AH0AAACqAIkAAACqAJAAAACqAK3/twCqAK7/twCqAK//3ACqALr/3ACqALv/"
        "HwCqAMf/twCqAMn/twCqAND/3ACqANH/3ACqANL/3ACqAOUAAACqAOkAAACq"
        "AOr/HwCqAOv/3ACqAOwAAACqAPYAAACqAPv/3ACqAP3/3ACtABD/0wCtABH/"
        "3ACtAB3/3ACtACQAOQCtACb/3ACtACr/3ACtADL/3ACtADT/3ACtADYAAACt"
        "ADf/YQCtADgAAACtADn/fQCtADr/kACtADsAAACtADz/YQCtAEb/3ACtAEf/"
        "3ACtAEj/3ACtAEn/twCtAFL/3ACtAFT/3ACtAFf/3ACtAFgAAACtAFn/iACt"
        "AFr/rQCtAFz/dQCtAGIAOQCtAGT/3ACtAGf/3ACtAGgAAACtAG//3ACtAHD/"
        "3ACtAHH/3ACtAHL/3ACtAHP/3ACtAHn/3ACtAHr/3ACtAHv/3ACtAHz/3ACt"
        "AH3/3ACtAH4AAACtAH8AAACtAIAAAACtAIEAAACtAKn/twCtAKoAAACtAK0A"
        "OQCtAK4AOQCtAK//3ACtALT++ACtALX/AwCtALr/dQCtALv/YQCtAMUALwCt"
        "AMcAOQCtAMkAOQCtAND/3ACtANH/3ACtANL/3ACtANMAAACtANQAAACtANUA"
        "AACtAOMAAACtAOr/YQCtAOv/dQCtAPb/3ACtAPkAAACtAPv/3ACtAPz/3ACt"
        "AP3/3ACtAP7/3ACuABD/0wCuABH/3ACuAB3/3ACuACQAOQCuACb/3ACuACr/"
        "3ACuADL/3ACuADT/3ACuADYAAACuADf/YQCuADgAAACuADn/fQCuADr/kACu"
        "ADsAAACuADz/YQCuAEb/3ACuAEf/3ACuAEj/3ACuAEn/twCuAFL/3ACuAFT/"
        "3ACuAFf/3ACuAFgAAACuAFn/iACuAFr/rQCuAFz/dQCuAGIAOQCuAGT/3ACu"
        "AGf/3ACuAGgAAACuAG//3ACuAHD/3ACuAHH/3ACuAHL/3ACuAHP/3ACuAHn/"
        "3ACuAHr/3ACuAHv/3ACuAHz/3ACuAH3/3ACuAH4AAACuAH8AAACuAIAAAACu"
        "AIEAAACuAKn/twCuAKoAAACuAK0AOQCuAK4AOQCuAK//3ACuALT++ACuALX/"
        "AwCuALr/dQCuALv/YQCuAMUALwCuAMcAOQCuAMkAOQCuAND/3ACuANH/3ACu"
        "ANL/3ACuANMAAACuANQAAACuANUAAACuAOMAAACuAOr/YQCuAOv/dQCuAPb/"
        "3ACuAPkAAACuAPv/3ACuAPz/3ACuAP3/3ACuAP7/3ACvABAAOQCvABH/rQCv"
        "AB3/3ACvACT/3ACvADn/3ACvADv/fQCvADz/kACvAGL/3ACvAKn/3ACvAKoA"
        "AACvAK3/3ACvAK7/3ACvALT/0wCvALX/3ACvALv/kACvAMX/RACvAMf/3ACv"
        "AMn/3ACvAOr/kAC0ACT++AC0ACX/wQC0ACb/twC0ACf/wQC0ACn/wQC0ACr/"
        "twC0ACv/wQC0AC3/wQC0AC7/wQC0AC//wQC0ADL/twC0ADP/wQC0ADT/twC0"
        "ADX/wQC0ADcAAAC0ADkAAAC0ADoAAAC0ADv/iAC0ADwAAAC0AD3/3AC0AEn/"
        "twC0AFH/kAC0AFL/awC0AFX/kAC0AFn/twC0AFr/twC0AFz/twC0AGL++AC0"
        "AGT/twC0AGf/twC0AHj/kAC0AHn/awC0AHr/awC0AHv/awC0AHz/awC0AH3/"
        "awC0AIn/wQC0AJD+fQC0AK3++AC0AK7++AC0AK//twC0ALr/twC0ALsAAAC0"
        "AMf++AC0AMn++AC0AND/twC0ANH/twC0ANL/twC0AOX/3AC0AOn/twC0AOoA"
        "AAC0AOv/twC0AOz/wQC0APb/twC0APv/twC0AP3/twC6ABD/3AC6ABH+3AC6"
        "AB3/awC6AKn/3AC6AKr/3AC6ALQAAAC6ALUAAAC6AMX+0wC7ABD/DQC7ABH+"
        "YQC7AB3+8AC7ACT/YQC7ACb/kAC7ADL/kAC7AET+5gC7AEj+8AC7AEz/twC7"
        "AFL+8AC7AFj/FQC7AGL/YQC7AGT/kAC7AGf/kAC7AGn+5gC7AGr+5gC7AGv+"
        "5gC7AGz+5gC7AG3+5gC7AG7+5gC7AHD+8AC7AHH+8AC7AHL+8AC7AHP+8AC7"
        "AHn+8AC7AHr+8AC7AHv+8AC7AHz+8AC7AH3+8AC7AH7/FQC7AH//FQC7AID/"
        "FQC7AIH/FQC7AKn/HwC7AKr/awC7AK3/YQC7AK7/YQC7AK//kAC7ALT/kAC7"
        "ALX/3AC7AMX++AC7AMf/YQC7AMn/YQC7AND/kAC7ANH/kAC7ANL/kAC7APv/"
        "kAC7AP3/kADFACQAJgDFACX/twDFACb/kADFACf/twDFACn/twDFACr/twDF"
        "ACv/twDFAC0ALwDFAC7/twDFAC//twDFADL/kADFADP/twDFADT/kADFADX/"
        "twDFADf+5gDFADn+iADFADr/AwDFADv/twDFADz+iADFAD0AAADFAEn/3ADF"
        "AFH/twDFAFL/twDFAFX/twDFAFn/FQDFAFr/PADFAFz/kADFAGIAJgDFAGT/"
        "kADFAGf/kADFAHj/twDFAHn/twDFAHr/twDFAHv/twDFAHz/twDFAH3/twDF"
        "AIn/twDFAJAAJgDFAK0AJgDFAK4AJgDFAK//kADFALr/kADFALv+iADFAMcA"
        "JgDFAMkAJgDFAND/kADFANH/kADFANL/kADFAOUAAADFAOn/twDFAOr+iADF"
        "AOv/kADFAOz/twDFAPb/twDFAPv/kADFAP3/kADHABD/0wDHABH/3ADHAB3/"
        "3ADHACQAOQDHACb/3ADHACr/3ADHADL/3ADHADT/3ADHADYAAADHADf/YQDH"
        "ADgAAADHADn/fQDHADr/kADHADsAAADHADz/YQDHAEb/3ADHAEf/3ADHAEj/"
        "3ADHAEn/twDHAFL/3ADHAFT/3ADHAFf/3ADHAFgAAADHAFn/iADHAFr/rQDH"
        "AFz/dQDHAGIAOQDHAGT/3ADHAGf/3ADHAGgAAADHAG//3ADHAHD/3ADHAHH/"
        "3ADHAHL/3ADHAHP/3ADHAHn/3ADHAHr/3ADHAHv/3ADHAHz/3ADHAH3/3ADH"
        "AH4AAADHAH8AAADHAIAAAADHAIEAAADHAKn/twDHAKoAAADHAK0AOQDHAK4A"
        "OQDHAK//3ADHALT++ADHALX/AwDHALr/dQDHALv/YQDHAMUALwDHAMcAOQDH"
        "AMkAOQDHAND/3ADHANH/3ADHANL/3ADHANMAAADHANQAAADHANUAAADHAOMA"
        "AADHAOr/YQDHAOv/dQDHAPb/3ADHAPkAAADHAPv/3ADHAPz/3ADHAP3/3ADH"
        "AP7/3ADJABD/0wDJABH/3ADJAB3/3ADJACQAOQDJACb/3ADJACr/3ADJADL/"
        "3ADJADT/3ADJADYAAADJADf/YQDJADgAAADJADn/fQDJADr/kADJADsAAADJ"
        "ADz/YQDJAEb/3ADJAEf/3ADJAEj/3ADJAEn/twDJAFL/3ADJAFT/3ADJAFf/"
        "3ADJAFgAAADJAFn/iADJAFr/rQDJAFz/dQDJAGIAOQDJAGT/3ADJAGf/3ADJ"
        "AGgAAADJAG//3ADJAHD/3ADJAHH/3ADJAHL/3ADJAHP/3ADJAHn/3ADJAHr/"
        "3ADJAHv/3ADJAHz/3ADJAH3/3ADJAH4AAADJAH8AAADJAIAAAADJAIEAAADJ"
        "AKn/twDJAKoAAADJAK0AOQDJAK4AOQDJAK//3ADJALT++ADJALX/AwDJALr/"
        "dQDJALv/YQDJAMUALwDJAMcAOQDJAMkAOQDJAND/3ADJANH/3ADJANL/3ADJ"
        "ANMAAADJANQAAADJANUAAADJAOMAAADJAOr/YQDJAOv/dQDJAPb/3ADJAPkA"
        "AADJAPv/3ADJAPz/3ADJAP3/3ADJAP7/3ADQABAAOQDQABH/rQDQAB3/3ADQ"
        "ACT/3ADQADn/3ADQADv/fQDQADz/kADQAGL/3ADQAKn/3ADQAKoAAADQAK3/"
        "3ADQAK7/3ADQALT/0wDQALX/3ADQALv/kADQAMX/RADQAMf/3ADQAMn/3ADQ"
        "AOr/kADRABAAOQDRABH/rQDRAB3/3ADRACT/3ADRADn/3ADRADv/fQDRADz/"
        "kADRAGL/3ADRAKn/3ADRAKoAAADRAK3/3ADRAK7/3ADRALT/0wDRALX/3ADR"
        "ALv/kADRAMX/RADRAMf/3ADRAMn/3ADRAOr/kADSABAAOQDSABH/rQDSAB3/"
        "3ADSACT/3ADSADn/3ADSADv/fQDSADz/kADSAGL/3ADSAKn/3ADSAKoAAADS"
        "AK3/3ADSAK7/3ADSALT/0wDSALX/3ADSALv/kADSAMX/RADSAMf/3ADSAMn/"
        "3ADSAOr/kADTACQAAADTAC0AAADTAD3/3ADTAGIAAADTAK0AAADTAK4AAADT"
        "AMcAAADTAMkAAADTAOX/3ADUACQAAADUAC0AAADUAD3/3ADUAGIAAADUAK0A"
        "AADUAK4AAADUAMcAAADUAMkAAADUAOX/3ADVACQAAADVAC0AAADVAD3/3ADV"
        "AGIAAADVAK0AAADVAK4AAADVAMcAAADVAMkAAADVAOX/3ADjACQAJgDjACYA"
        "AADjACoAAADjADIAAADjADQAAADjADYAAADjAGIAJgDjAGQAAADjAGcAAADj"
        "AK0AJgDjAK4AJgDjAK8AAADjAMcAJgDjAMkAJgDjANAAAADjANEAAADjANIA"
        "AADjAOMAAADjAPYAAADjAPkAAADjAPsAAADjAP0AAADlABD/3ADlAKkAAADl"
        "AKoAAADlALT/3ADlALX/3ADlAMX/3ADpABAAAADpAKkAAADpAKoAAADpALT/"
        "pADpALX/kADpAMX/twDqABD/DQDqABH+YQDqAB3+8ADqACT/YQDqACb/kADq"
        "ADL/kADqAET+5gDqAEj+8ADqAEz/twDqAFL+8ADqAFj/FQDqAGL/YQDqAGT/"
        "kADqAGf/kADqAGn+5gDqAGr+5gDqAGv+5gDqAGz+5gDqAG3+5gDqAG7+5gDq"
        "AHD+8ADqAHH+8ADqAHL+8ADqAHP+8ADqAHn+8ADqAHr+8ADqAHv+8ADqAHz+"
        "8ADqAH3+8ADqAH7/FQDqAH//FQDqAID/FQDqAIH/FQDqAKn/HwDqAKr/awDq"
        "AK3/YQDqAK7/YQDqAK//kADqALT/kADqALX/3ADqAMX++ADqAMf/YQDqAMn/"
        "YQDqAND/kADqANH/kADqANL/kADqAPv/kADqAP3/kADrABD/3ADrABH+3ADr"
        "AB3/awDrAKn/3ADrAKr/3ADrALQAAADrALUAAADrAMX+0wDsABAAAADsABH/"
        "awDsAB3/twDsAKkAAADsAKoAAADsALT/3ADsALUAAADsAMX/RAD2ABAAAAD2"
        "ACQAAAD2ADf/twD2ADoAAAD2ADz/mgD2AGIAAAD2AKn/3AD2AKr/3AD2AK0A"
        "AAD2AK4AAAD2ALT/0wD2ALX/0wD2ALv/mgD2AMX/yQD2AMcAAAD2AMkAAAD2"
        "AOr/mgD5ACQAJgD5ACYAAAD5ACoAAAD5ADIAAAD5ADQAAAD5ADYAAAD5AGIA"
        "JgD5AGQAAAD5AGcAAAD5AK0AJgD5AK4AJgD5AK8AAAD5AMcAJgD5AMkAJgD5"
        "ANAAAAD5ANEAAAD5ANIAAAD5AOMAAAD5APYAAAD5APkAAAD5APsAAAD5AP0A"
        "AAD7ABAAAAD7ACQAAAD7ADYAAAD7ADz/3AD7AGIAAAD7AKn/3AD7AKr/3AD7"
        "AK0AAAD7AK4AAAD7ALQAAAD7ALUAJgD7ALv/3AD7AMUAAAD7AMcAAAD7AMkA"
        "AAD7AOMAAAD7AOr/3AD7APkAAAD9ABAAAAD9ACQAAAD9ADYAAAD9ADz/3AD9"
        "AGIAAAD9AKn/3AD9AKr/3AD9AK0AAAD9AK4AAAD9ALQAAAD9ALUAJgD9ALv/"
        "3AD9AMUAAAD9AMcAAAD9AMkAAAD9AOMAAAD9AOr/3AD9APkAAAAAAAEAAAEM"
        "AE0ABwBCAAQAAgAQAEAABwAABBUFaAADAAEAAQAAB23+HQAACrz+if6JCkwA"
        "AQAAAAAAAAAAAAAAAAAAAQwAAQQOAZAABQAEBUcEzAAA/kIFRwTMAAACUwCP"
        "AmYIAgILBgMDCAQCAgSAAACvEAAgSgAAAAAAAAAAQml0cwBAACD7AgYU/hQB"
        "mgdtAeMAAAABAAAAAAAAAAEAAEIAHbECiwRgAAAjYwXVAABWZXJhU2FucwAA"
        "AAAAAAAA/////zb///42MjhSMDAAAEAAAAAAAAAUAAABEAkMBQADAwMECAYJ"
        "CAIEBAUIAwMDAwYGBgYGBgYGBgYDAwgICAUKBgcHBwYGBwcDAwYFCAcHBgcG"
        "BwUHBgcGBQUEAwQIBQUGBgYGBgMGBgMDBQMJBgYGBgQFBAYFBwUFBgUDBQgG"
        "BgcGBwcHBgYGBgYGBgYGBgYDAwMDBgYGBgYGBgYGBgUFBgYFBQYGCQkJBQUI"
        "CQcICAgIBgYFBgcFBQQEBwkGBQMIBgYIBgYGCQYGBgcKCQUJBQUDAwgEBQUC"
        "BgQEBgYFAwMFDAYGBgYGAwMDAwcHBwcHBwMFBQUFBQUFBQUFBQMHBQUGAwcG"
        "BQUGBggIBAQECQkJBwYDBwUHBgcGBgMDBgUFBQUFBQQFBQAACg0GAAMDBAUI"
        "BgoJAwQEBQgDBAMDBgYGBgYGBgYGBgMDCAgIBQsHBwgIBwYICAMDBwYJCAgH"
        "CAcHBQgHCQYHBgQDBAgFBQYGBQYGBAYGAgIFAgoGBgYGBAUEBgYIBgYFBgMG"
        "CAcHCAcICAgGBgYGBgYFBgYGBgICAgIGBgYGBgYGBgYGBQUGBgUGBgYKCgoF"
        "BQgKCAgICAgGBwUHCAYFBQUICgYFBAgGBggHBgYKBgcHCAsKBQoFBQMDCAUG"
        "BwIGBAQGBgUDAwUNBwcHBwcDAwMDCAgICAgIAgUFBQUFBQUFBQUGAgcFBgUD"
        "CAYHBgcGCAgEBAQKCgoIBgMHBQgFCAUGBAMGBQUFBQUFBAUFAAALDwcAAwMD"
        "BQkHCgoDBAQGCQQEBAQHBwcHBwcHBwcHBAQJCQkGDAcICAgHBgkIAwMHBgkI"
        "CQcJBwgHCAcJBwcJBAQECQYGBwcGBwcEBwcDAwYDCwcHBwcFBwQHBgkGBgUH"
        "BAcJBwcIBwgJCAcHBwcHBwYHBwcHAwMDAwcHBwcHBwcHBwcGBgcHBgYHBwsL"
        "CwYGCQsJCQkJCQcHBgcIBgYFBQgLBwYDCQcHCQcHBwsHBwcJDAsGCwYGBAQJ"
        "BQYHAgcEBAcHBgQEBg8HBwcHBwMDAwMJCQkICAgDBgYGBgYGBgYGBgYDCAcJ"
        "BQQIBwcGBwcJCQQEBAsLCwkHAwgHCAYIBgcEBAcGBgYGBgYEBgYAAAwQBwAE"
        "BAUFCggLCgMFBQYKBAQEBAgICAgICAgICAgEBAoKCgYNCAgICQgHCQkDAwcG"
        "CgkJCAkICAcJCAsJBwkFBAUKBgYICAcICAQICAMDBwMLCAgICAUHBQgGCQYG"
        "BQgECAoICAgICQkJCAgICAgIBwgICAgDAwMDCAgICAgICAgICAYGCAgGBwgI"
        "DAwMBgYKDAkKCgoKCAgGCAkHBgYGCQwIBgUKCAgKCAcHDAgICAkNDQYMBgYE"
        "BAoGBgcCCAUFBwcGBAQGEAgICAgIAwMDAwkJCQkJCQMGBgYGBgYGBgYGBgMI"
        "BwkFBAkIBwYICAoKBQUFDAwMCQgDCAcIBwgHCAQECAYGBgYGBgUGBgAADREI"
        "AAQEBQULCAwLAwUFBwsEBQQECAgICAgICAgICAQECwsLBw0JCQkKCAcKCgMD"
        "CAcLCgoICggJBwoJCwoHCgUEBQsHBwgIBwgIBAgIAwMHAw0ICAgIBQcFCAcJ"
        "BwcHCAQICwkJCQgKCgoICAgICAgHCAgICAMDAwMICAgICAgICAgIBwcICAcI"
        "CAgNDQ0HBwsNCgsLCwsICAcJCggHBgYKDQgHBQsICAsJCAgNCAkJCg4NBw0H"
        "BwQECwYHBwIIBQUICAcEBAcRCQgJCAgDAwMDCgoKCgoKAwcHBwcHBwcHBwcH"
        "AwkHCgcECggHBwgICwsFBQUNDQ0KCAMJBwkHCQcIBQQIBwcHBwcHBQcHAAAO"
        "EwgABAQFBQwJDQwDBQUHDAQFBAUJCQkJCQkJCQkJBQUMDAwHDgkKCgsJCAsK"
        "AwMJBwwKCwkLCgkJCgkNCwkKBQUFDAcHCAkICQkECQkDAwgDDQkJCQkFCAUJ"
        "BwsHBwgJBQkMCQkKCQoLCggICAgICAgJCQkJAwMDAwkJCQkJCQkJCQkHBwkJ"
        "BwgJCQ4ODgcHDA4LDAwMDAkJBwkLCAcHBwsOCQcFDAkJDAkJCQ4JCQkLDw8H"
        "DgcHBAQMBwcJAgkGBggIBwQEBxMJCQkJCQMDAwMLCwsKCgoDBwcHBwcHBwcH"
        "BwcDCQgKCAULCQkHCQkMDAYGBg4ODgsJAwkICggKCAkFBAkHBwcHBwcGBwcA"
        "AA8UCQAFBQYGDQoOCwMGBggNBQUFBQoKCgoKCgoKCgoFBQ0NDQgPCgoKCwkJ"
        "DAsDAwoIDQsMCQwKCgkLCg8KCQsGBQYNCAgJCQgJCQUJCQMDCAMPCQkJCQYI"
        "BgkIDQoICAoFCg0KCgoJCwwLCQkJCQkJCAkJCQkDAwMDCQkJCQkJCQkJCQgI"
        "CgoICQoJDw8PCAgNDwwNDQ0NCgkICgsJCAcHCw8JCAYNCgoNCgkJDwoKCgwQ"
        "DwgPCAgFBQ0HCAkDCgYGCQkIBQUIFAoJCgkJAwMDAwwMDAsLCwMICAgICAgI"
        "CAgICAMKCAsIBQwJCQgJCQ0NBgYGDw8PDAkDCggKCAoICQUFCggICAgICAYI"
        "CAAAEBUKAAUFBgYNCg8MAwYGCA0FBgUFCgoKCgoKCgoKCgUFDQ0NCRALCwsM"
        "CgkMDAUFCgkNDA0KDQsKCQwLEQsJDAYFBg0ICAkKCQoJBgoKAwMJAw8KCgoK"
        "BwkGCgkNCgkJCgUKDQsLCwoMDQwJCQkJCQkJCQkJCQMDAwMKCgoKCgoKCgoK"
        "CAgKCggJCgoQEBAICA0QDQ0NDQ0KCggLDAkICAgMEAoJBg0KCg0LCgoQCgsL"
        "DREPCBAICAUFDQgJCQMKBgYKCggFBQgVCwoLCgoFBQUFDQ0NDAwMAwgICAgI"
        "CAgICAgJBAoJDAkFDQoJCQoKDQ0GBgYQEBAMCgUKCQsJCwkKBgUKCAgICAgI"
        "BggIAAARFwoABQUHCA4LEA0FBwcJDgUGBQYLCwsLCwsLCwsLBgYODg4JEQsM"
        "DA0LCg0NBgYMCQ8NDQoNDAsKDQsSDQoMBwYHDgkJCgoICgoGCgsFBQoFEQsK"
        "CgoHCAcLCg8KCggLBgsOCwsMCw0NDQoKCgoKCggKCgoKBQUFBQsKCgoKCgsL"
        "CwsJCQsLCQoLCxEREQkJDhANDg4ODgsLCQsNCgkICA0QCgkHDgsLDgsKChEL"
        "CwsNEhEJEQkJBQUOCAoKAwsHBwoKCQUFCRcLCwsLCwYGBgYNDQ0NDQ0FCQkJ"
        "CQkJCQkJCQoFCwgMCAYNCgoKCgoODgcHBxAQEA0KBgsIDAgMCAoGBQsJCQkJ"
        "CQkHCQkAABIYCwAGBgcIDwsRDQQHBwkPBgcGBgsLCwsLCwsLCwsGBg8PDwoS"
        "DAwNDgsKDg4GBgwKEA4OCw4NCwwODBMNDA0HBgcPCQkKCwkLCwYLCwUFCgUR"
        "CwsLCwgIBwsLEAsLCQsGCw8MDA0LDg4OCgoKCgoKCQsLCwsFBQUFCwsLCwsL"
        "CwsLCwkJCwsJCwsLEhISCQkPEQ4PDw8PCwsJDA4LCQgIDhELCgcPCwsPDAsL"
        "EgsMDA4TEwkSCQkGBg8JCwwDCwcHCwsJBgYJGAwLDAsLBgYGBg4ODg4ODgUJ"
        "CQkJCQkJCQkJCgULCA0JBg4LDAsLCw8PBwcHERERDgsGCwgNCQ0JCwcGCwkJ"
        "CQkJCQcJCQAAExoLAAYGCAgQDBIOBAcHChAGBwYGDAwMDAwMDAwMDAYGEBAQ"
        "ChMNDQ0PDAsPDgYGDAsQDg8MDw0MDA4NFA4MDQcGBxAKCgsLCQsLBgsMBQUL"
        "BREMCwsLCAkIDAsRCwsLDAYMEA0NDQwODw4LCwsLCwsJCwsLCwUFBQUMCwsL"
        "CwsMDAwMCgoMDAoLDAwTExMKChASDxAQEBAMDAoNDgsKCQkPEgsKCBAMDBAN"
        "DAwTDA0NDxQTChMKCgYGEAkLDAMMCAgLCwoGBgoaDQwNDAwGBgYGDw8PDg4O"
        "BQoKCgoKCgoKCgoLBQwJDQsGDwsMCwwLEBAICAgSEhIPCwYMCQ0JDQkLBwYM"
        "CgoKCgoKCAoKAAAUGwwABgYIChENEw8GCAgKEQYHBgcNDQ0NDQ0NDQ0NBwcR"
        "ERELFA4ODg8NDBAPBgYNCxEPEAwQDg0MDw4UDgwOCAcIEQoKDA0LDQwHDQ0G"
        "BgwGFA0MDQ0ICggNCxELCwsNBw0RDg4ODQ8QDwwMDAwMDAsMDAwMBgYGBg0M"
        "DAwMDA0NDQ0KCg0NCgwNDRQUFAoKERQQEREREQ0NCg0PDAoJCQ8TDAsIEQ0N"
        "EQ0MDBQNDg4QFRQKFAoKBgYRCgsMAw0ICA0NCgYGChsODQ4NDQYGBgYQEBAP"
        "Dw8GCgoKCgoKCgoKCgsGDQoOCwcPDAwLDA0REQgICBMTExANBg0KDgsOCw0H"
        "Bg0KCgoKCgoICgoAABUcDQAHBwgKEg0UEAYICAsSBwgHBw0NDQ0NDQ0NDQ0H"
        "BxISEgsVDg4PEA0MEBAGBg4MEhARDREPDQ0PDhUODQ4IBwgSCwsNDQwNDQcN"
        "DQYGDAYUDg0NDQkLCA4LEQ0NCw0HDRIODg8NEBEPDQ0NDQ0NDA0NDQ0GBgYG"
        "Dg0NDQ0NDg4ODgsLDQ0LDA0NFRUVCwsSFBESEhISDQ4LDhAMCwoKEBQNCwgS"
        "DQ0SDg0NFQ0ODhEWFgsVCwsHBxIKDQ0EDQgIDQ0LBwcLHA4NDg0NBgYGBhER"
        "EQ8PDwYLCwsLCwsLCwsLDAYNCw4LBxANDQ0NDRISCAgIFBQUEA0GDQsPDA8M"
        "DQgHDQsLCwsLCwgLCwAAFh4NAAcHCQoSDhURBgkJCxIHCAcHDg4ODg4ODg4O"
        "DgcHEhISDBYPDw8RDg0REQYGDgwTEBENEQ8ODRAPFg8NDwkHCRILCw0ODA4N"
        "CA4OBgYNBhYODQ4OCQsJDg4SDg4MDgcOEg8PDw4QERANDQ0NDQ0MDQ0NDQYG"
        "BgYODQ0NDQ0ODg4OCwsODgsNDg4WFhYLCxIVERISEhIODgsPEQ0LCgoRFQ0M"
        "CRIODhIPDQ0WDg8PERgWCxYLCwcHEgsODQQOCQkNDQsHBwseDw4PDg4GBgYG"
        "EREREBAQBgsLCwsLCwsLCwsMBg4LDwwHEQ0NDg0OEhIJCQkVFRURDgYOCw8M"
        "DwwOCAcOCwsLCwsLCQsLAAAXHw4ABwcJCxMPFhIGCQkMEwcIBwgPDw8PDw8P"
        "Dw8PCAgTExMMFxAQEBIPDRIRBwcPDRQREg4SEA8OERAXEA4QCQgJEwwMDg8N"
        "Dw4IDw8GBg0GFg8ODg4JDAkPDhMODgwPCA8TEBAQDxESEQ4ODg4ODg0ODg4O"
        "BgYGBg8ODg4ODg8PDw8MDA8PDA4PDhcXFwwMExYSExMTEw8PDBARDgwLCxIV"
        "DgwJEw8PEw8ODhcPEBASGRgMFwwMBwcTCw4OBA8JCQ8PDAcHDB8QDxAPDwcH"
        "BwcSEhIREREGDAwMDAwMDAwMDA0GDwwQDAgSDg4ODg4TEwkJCRYWFhIPBw8M"
        "EA0QDQ8IBw8MDAwMDAwJDAwAABggDgAICAoLFA8XEwcJCQwUCAkICA8PDw8P"
        "Dw8PDw8ICBQUFA0YEBAREg8OExIHBxANFRITDhMRDw8SEBgQDxAJCAkUDAwO"
        "Dw0PDggPDwcHDgYYDw4PDwoMCQ8PFA8PDQ8IDxQQEBEPEhMSDg4ODg4ODQ4O"
        "Dg4HBwcHDw4ODg4ODw8PDwwMDw8MDg8PGBgYDAwUFxMUFBQUDw8MEBIODQsL"
        "EhcODQoUDw8UEA8PGA8QEBMaGAwYDAwICBQMDw8EDwoKDw8MCAgMIBAPEA8P"
        "BwcHBxMTExISEgcMDAwMDAwMDAwMDQYPDBANCBMODw8PDxQUCgoKFxcXEw8H"
        "DwwRDRENDwkIDwwMDAwMDAoMDAAAGSIPAAgICgwVEBgUBwoKDRUICQgIEBAQ"
        "EBAQEBAQEAgIFRUVDRkRERETEA4TEwcHEA4WExQPFBEQDxIRGREPEQoIChUN"
        "DQ8QDhAPCRAQBwcOBxgQDxAQCg0KEA8UDw8NEAgQFRERERATFBIPDw8PDw8O"
        "Dw8PDwcHBwcQDw8PDw8QEBAQDQ0QEA0PEBAZGRkNDRUYFBUVFRUQEA0REw8N"
        "DAwTGQ8NChUQEBURDw8ZEBERFBsaDRkNDQgIFQwPDwQQCgoQEA0ICA0iERAR"
        "EBAHBwcHFBQUEhISBw0NDQ0NDQ0NDQ0OBxANEQ0IEw8PDw8QFRUKCgoYGBgT"
        "EAcQDREOEQ4QCQgQDQ0NDQ0NCg0NAAAaIxAACAgKDBYRGRQHCgoNFggJCAkR"
        "ERERERERERERCQkWFhYOGhISEhQQDxQUCAgRDhYTFBAUEhEQExIaEhASCgkK"
        "Fg0NEBEOERAJERAHBw8HGRAQERELDgoQDxUPDw4RCREWEhISEBMUExAQEBAQ"
        "EA4QEBAQBwcHBxAQEBAQEBAQEBANDRERDQ8REBoaGg0NFhkUFhYWFhERDRIU"
        "Dw4MDBQaEA4KFhERFhEQEBoREhIUHBsNGg0NCAgWDQ8QBBEKChAQDQgIDSMS"
        "EBIQEAgICAgUFBQTExMHDQ0NDQ0NDQ0NDQ8HEQ4SDgkUEBAPEBEWFgoKChkZ"
        "GRQRCBEOEg4SDhEJCBENDQ0NDQ0KDQ0AABskEAAJCQsMFxEaFQcLCw4XCQoJ"
        "CREREREREREREREJCRcXFw4bEhMTFREQFRQICBIPFxQVEBUTEREUEhsTERML"
        "CQsXDg4REQ8REQoREQgIEAgaEREREQsOCxEQFhAQDhEJERcSEhMRFBUUERER"
        "ERERDxEREREICAgIEREREREREREREQ4OEREOEBERGxsbDg4XGhUWFxcXEREO"
        "EhQQDg0NFRsRDgsXEREXEhERGxESEhUdHA4bDg4JCRcNEBEFEQsLEREOCQkO"
        "JBIREhERCAgICBUVFRQUFAgODg4ODg4ODg4ODwgRDhMOCRURERAQERcXCwsL"
        "GhoaFREIEQ4TDxMPEQoJEQ4ODg4ODgsODgAAHCYRAAkJCw0XEhsWCAsLDhcJ"
        "CgkJEhISEhISEhISEgkJFxcXDxwTExQWEhAWFQgIEhAYFRYRFhMSERUTHBMR"
        "EwsJCxcODhESDxIRChISCAgQCBsSERISDA8LEhEXEREPEgkSFxMTFBIVFhUR"
        "EREREREPEREREQgICAgSERERERESEhISDg4SEg4REhIcHBwODhcbFhcXFxcS"
        "Eg4TFRAPDQ0VHBEPCxcSEhcTEREcEhMTFh4dDhwPDwkJFw4REQUSCwsSEg4J"
        "CQ8mExITEhIICAgIFhYWFRUVCA4ODg4ODg4ODg4QCBIPEw8JFhERERESFxcL"
        "CwsbGxsWEggSDxQPFA8SCgkSDg4ODg4OCw4OAAAAAAACAAgAAv//AAMAAQAA"
        "AAIAAAxQCuxfDzz1AB8IAAAAAAC6ufC4AAAAALrCZ5H+if4dCkwHbQAAAAgA"
        "AQAAAAAAAA";

    const std::vector<unsigned char>& VeraTTFBytes()
    {
        static std::vector<unsigned char> retval;
        if (retval.empty()) {
            std::string chars(VERA_TTF_PART_1);
            chars += VERA_TTF_PART_2;
            DecodeBase64(retval, chars);
        }
        return retval;
    }

    const char* DEFAULT_FONT_NAME = "HopefullyUniqueDefaultFontName!!!!!!!!!!!!!!11111eleven";
}
